@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -53,7 +53,7 @@ import org.hsqldb.rights.UserManager;
 import org.hsqldb.types.Collation;
 
 // incorporates following contributions
-// boucherb@users - javadoc comments
+// campbell-burnet@users - javadoc comments
 
 /**
  * Database is the root class for HSQL Database Engine database. <p>
@@ -61,7 +61,7 @@ import org.hsqldb.types.Collation;
  * It holds the data structures that form an HSQLDB database instance.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.5
  * @since 1.9.0
  */
 public class Database {
@@ -96,6 +96,7 @@ public class Database {
      *  databaseProperties or URL properties for new databases.
      */
     public int                    sqlAvgScale            = 0;
+    public boolean                sqlRestrictExec          = false;
     public boolean                sqlCharLiteral         = true;
     public boolean                sqlConcatNulls         = true;
     public boolean                sqlConvertTruncate     = true;
@@ -111,6 +112,7 @@ public class Database {
     public boolean                sqlLongvarIsLob        = false;
     public boolean                sqlNullsFirst          = true;
     public boolean                sqlNullsOrder          = true;
+    public boolean                sqlSysIndexNames       = false;
     public boolean                sqlRegularNames        = true;
     public boolean                sqlTranslateTTI        = true;
     public boolean                sqlUniqueNulls         = true;
@@ -289,16 +291,15 @@ public class Database {
 
     public void createObjectStructures() {
 
+        nameManager = new HsqlNameManager(this);
+        databaseUniqueName = nameManager.newHsqlName("", false,
+                SchemaObject.DATABASE);
         lobManager     = new LobManager(this);
-        nameManager    = new HsqlNameManager(this);
         granteeManager = new GranteeManager(this);
         userManager    = new UserManager(this);
         schemaManager  = new SchemaManager(this);
         persistentStoreCollection =
             new PersistentStoreCollectionDatabase(this);
-        databaseUniqueName = nameManager.newHsqlName("", false,
-                SchemaObject.DATABASE);
-
         isReferentialIntegrity = true;
         sessionManager         = new SessionManager(this);
         collation              = Collation.newDatabaseInstance();
@@ -309,7 +310,6 @@ public class Database {
         sessionManager.getSysLobSession().setSchema(SqlInvariants.LOBS_SCHEMA);
         schemaManager.setSchemaChangeTimestamp();
         schemaManager.createSystemTables();
-
     }
 
     /**
@@ -475,6 +475,10 @@ public class Database {
         resultMaxMemoryRows = size;
     }
 
+    public void setRestrictExec(boolean mode) {
+        sqlRestrictExec = mode;
+    }
+
     public void setStrictNames(boolean mode) {
         sqlEnforceNames = mode;
     }
@@ -550,6 +554,10 @@ public class Database {
         sqlIgnoreCase = mode;
     }
 
+    public void setSysIndexNames(boolean mode) {
+        sqlSysIndexNames = mode;
+    }
+
     public void setSyntaxDb2(boolean mode) {
         sqlSyntaxDb2 = mode;
     }
