@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -52,21 +52,23 @@ import org.hsqldb.types.ArrayType;
 import org.hsqldb.types.CharacterType;
 import org.hsqldb.types.Collation;
 import org.hsqldb.types.NullType;
+import org.hsqldb.types.RowType;
 import org.hsqldb.types.Type;
 import org.hsqldb.types.Types;
 
 /**
  * Expression class.
  *
- * @author Campbell Burnet (boucherb@users dot sourceforge.net)
+ * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.3.5
  * @since 1.9.0
  */
 public class Expression implements Cloneable {
 
     public static final int LEFT    = 0;
     public static final int RIGHT   = 1;
+    public static final int THIRD   = 2;
     public static final int UNARY   = 1;
     public static final int BINARY  = 2;
     public static final int TERNARY = 3;
@@ -85,20 +87,20 @@ public class Expression implements Cloneable {
 
     static {
         aggregateFunctionSet.add(OpTypes.COUNT);
-        aggregateFunctionSet.add(OpTypes.SUM);
-        aggregateFunctionSet.add(OpTypes.MIN);
-        aggregateFunctionSet.add(OpTypes.MAX);
         aggregateFunctionSet.add(OpTypes.AVG);
+        aggregateFunctionSet.add(OpTypes.MAX);
+        aggregateFunctionSet.add(OpTypes.MIN);
+        aggregateFunctionSet.add(OpTypes.SUM);
         aggregateFunctionSet.add(OpTypes.EVERY);
         aggregateFunctionSet.add(OpTypes.SOME);
         aggregateFunctionSet.add(OpTypes.STDDEV_POP);
         aggregateFunctionSet.add(OpTypes.STDDEV_SAMP);
         aggregateFunctionSet.add(OpTypes.VAR_POP);
         aggregateFunctionSet.add(OpTypes.VAR_SAMP);
-        aggregateFunctionSet.add(OpTypes.GROUP_CONCAT);
         aggregateFunctionSet.add(OpTypes.ARRAY_AGG);
-        aggregateFunctionSet.add(OpTypes.MEDIAN);
         aggregateFunctionSet.add(OpTypes.USER_AGGREGATE);
+        aggregateFunctionSet.add(OpTypes.GROUP_CONCAT);
+        aggregateFunctionSet.add(OpTypes.MEDIAN);
     }
 
     static final OrderedIntHashSet columnExpressionSet =
@@ -120,25 +122,8 @@ public class Expression implements Cloneable {
         new OrderedIntHashSet();
 
     static {
-        subqueryAggregateExpressionSet.add(OpTypes.COUNT);
-        subqueryAggregateExpressionSet.add(OpTypes.SUM);
-        subqueryAggregateExpressionSet.add(OpTypes.MIN);
-        subqueryAggregateExpressionSet.add(OpTypes.MAX);
-        subqueryAggregateExpressionSet.add(OpTypes.AVG);
-        subqueryAggregateExpressionSet.add(OpTypes.EVERY);
-        subqueryAggregateExpressionSet.add(OpTypes.SOME);
-        subqueryAggregateExpressionSet.add(OpTypes.STDDEV_POP);
-        subqueryAggregateExpressionSet.add(OpTypes.STDDEV_SAMP);
-        subqueryAggregateExpressionSet.add(OpTypes.VAR_POP);
-        subqueryAggregateExpressionSet.add(OpTypes.VAR_SAMP);
-        subqueryAggregateExpressionSet.add(OpTypes.GROUP_CONCAT);
-        subqueryAggregateExpressionSet.add(OpTypes.ARRAY_AGG);
-        subqueryAggregateExpressionSet.add(OpTypes.MEDIAN);
-        subqueryAggregateExpressionSet.add(OpTypes.USER_AGGREGATE);
-
-        //
-        subqueryAggregateExpressionSet.add(OpTypes.TABLE_SUBQUERY);
-        subqueryAggregateExpressionSet.add(OpTypes.ROW_SUBQUERY);
+        subqueryAggregateExpressionSet.addAll(aggregateFunctionSet);
+        subqueryAggregateExpressionSet.addAll(subqueryExpressionSet);
     }
 
     static final OrderedIntHashSet functionExpressionSet =
@@ -170,7 +155,7 @@ public class Expression implements Cloneable {
     SimpleName alias;
 
     // aggregate
-    private boolean isAggregate;
+    private boolean hasAggregate;
     boolean         isDistinctAggregate;
 
     // VALUE
@@ -191,6 +176,9 @@ public class Expression implements Cloneable {
     //
     int queryTableColumnIndex = -1;    // >= 0 when it is used for order by
 
+    // self position in the result table
+    int resultTableColumnIndex = -1;
+
     // index of a session-dependent field
     int parameterIndex = -1;
 
@@ -469,9 +457,6 @@ public class Expression implements Cloneable {
 
         switch (opType) {
 
-            case OpTypes.SIMPLE_COLUMN :
-                return this.columnIndex == other.columnIndex;
-
             case OpTypes.VALUE :
                 return equals(valueData, other.valueData);
 
@@ -579,6 +564,9 @@ public class Expression implements Cloneable {
 
         switch (opType) {
 
+            case OpTypes.COLUMN :
+                return false;
+
             case OpTypes.LIKE :
             case OpTypes.MATCH_SIMPLE :
             case OpTypes.MATCH_PARTIAL :
@@ -591,19 +579,6 @@ public class Expression implements Cloneable {
             case OpTypes.ARRAY :
             case OpTypes.ARRAY_SUBQUERY :
             case OpTypes.TABLE_SUBQUERY :
-
-            //
-            case OpTypes.COUNT :
-            case OpTypes.SUM :
-            case OpTypes.MIN :
-            case OpTypes.MAX :
-            case OpTypes.AVG :
-            case OpTypes.EVERY :
-            case OpTypes.SOME :
-            case OpTypes.STDDEV_POP :
-            case OpTypes.STDDEV_SAMP :
-            case OpTypes.VAR_POP :
-            case OpTypes.VAR_SAMP :
                 return false;
 
             case OpTypes.ROW_SUBQUERY : {
@@ -630,10 +605,14 @@ public class Expression implements Cloneable {
             }
         }
 
-        if (nodes.length == 0) {
+        if (aggregateFunctionSet.contains(opType)) {
             return false;
         }
 
+        if (nodes.length == 0) {
+            return true;
+        }
+
         boolean result = true;
 
         for (int i = 0; i < nodes.length; i++) {
@@ -651,9 +630,14 @@ public class Expression implements Cloneable {
     boolean isComposedOf(OrderedHashSet expressions, RangeGroup[] rangeGroups,
                          OrderedIntHashSet excludeSet) {
 
-        if (opType == OpTypes.VALUE || opType == OpTypes.DYNAMIC_PARAM
-                || opType == OpTypes.PARAMETER || opType == OpTypes.VARIABLE) {
-            return true;
+        switch (opType) {
+
+            case OpTypes.VALUE :
+            case OpTypes.DYNAMIC_PARAM :
+            case OpTypes.PARAMETER :
+            case OpTypes.VARIABLE : {
+                return true;
+            }
         }
 
         if (excludeSet.contains(opType)) {
@@ -680,19 +664,6 @@ public class Expression implements Cloneable {
 
         switch (opType) {
 
-            case OpTypes.COUNT :
-            case OpTypes.SUM :
-            case OpTypes.MIN :
-            case OpTypes.MAX :
-            case OpTypes.AVG :
-            case OpTypes.EVERY :
-            case OpTypes.SOME :
-            case OpTypes.STDDEV_POP :
-            case OpTypes.STDDEV_SAMP :
-            case OpTypes.VAR_POP :
-            case OpTypes.VAR_SAMP :
-                return false;
-
             case OpTypes.FUNCTION :
             case OpTypes.SQL_FUNCTION :
                 if (nodes.length == 0) {
@@ -716,6 +687,10 @@ public class Expression implements Cloneable {
         case OpCodes.TABLE_SUBQUERY :
         case OpCodes.ROW_SUBQUERY :
 */
+        if (aggregateFunctionSet.contains(opType)) {
+            return false;
+        }
+
         if (nodes.length == 0) {
             return false;
         }
@@ -776,28 +751,25 @@ public class Expression implements Cloneable {
         }
     }
 
-    void convertToSimpleColumn(OrderedHashSet expressions,
-                               OrderedHashSet replacements) {
+    Expression replaceExpressions(OrderedHashSet expressions,
+                                  int resultRangePosition) {
 
         if (opType == OpTypes.VALUE) {
-            return;
+            return this;
         }
 
         if (opType == OpTypes.SIMPLE_COLUMN) {
-            return;
+            return this;
         }
 
-        int index = expressions.getIndex(this);
+        Expression exp = (Expression) expressions.get(this);
 
-        if (index != -1) {
-            Expression e = (Expression) replacements.get(index);
+        if (exp != null) {
+            Expression col = new ExpressionColumn(this,
+                                                  exp.resultTableColumnIndex,
+                                                  resultRangePosition);
 
-            nodes         = emptyArray;
-            opType        = OpTypes.SIMPLE_COLUMN;
-            columnIndex   = e.columnIndex;
-            rangePosition = e.rangePosition;
-
-            return;
+            return col;
         }
 
         for (int i = 0; i < nodes.length; i++) {
@@ -805,28 +777,22 @@ public class Expression implements Cloneable {
                 continue;
             }
 
-            nodes[i].convertToSimpleColumn(expressions, replacements);
+            nodes[i] = nodes[i].replaceExpressions(expressions,
+                                                   resultRangePosition);
         }
 
         if (table != null) {
             if (table.queryExpression != null) {
-                OrderedHashSet set = new OrderedHashSet();
-
-                table.queryExpression.collectAllExpressions(set,
-                        Expression.columnExpressionSet,
-                        Expression.emptyExpressionSet);
-
-                for (int i = 0; i < set.size(); i++) {
-                    Expression e = (Expression) set.get(i);
-
-                    e.convertToSimpleColumn(expressions, replacements);
-                }
+                table.queryExpression.replaceExpressions(expressions,
+                        resultRangePosition);
             }
         }
+
+        return this;
     }
 
-    boolean isAggregate() {
-        return isAggregate;
+    boolean hasAggregate() {
+        return hasAggregate;
     }
 
     boolean isDistinctAggregate() {
@@ -834,7 +800,7 @@ public class Expression implements Cloneable {
     }
 
     void setAggregate() {
-        isAggregate = true;
+        hasAggregate = true;
     }
 
     boolean isSelfAggregate() {
@@ -1227,6 +1193,8 @@ public class Expression implements Cloneable {
                         nodeDataTypes[i] = nodes[i].dataType;
                     }
                 }
+
+                dataType = new RowType(nodeDataTypes);
                 break;
 
             case OpTypes.ARRAY : {
@@ -1575,13 +1543,6 @@ public class Expression implements Cloneable {
             case OpTypes.VALUE :
                 return valueData;
 
-            case OpTypes.SIMPLE_COLUMN : {
-                Object value =
-                    session.sessionContext.rangeIterators[rangePosition]
-                        .getCurrent(columnIndex);
-
-                return value;
-            }
             case OpTypes.ROW : {
                 if (nodes.length == 1) {
                     return nodes[0].getValue(session);
@@ -1613,8 +1574,8 @@ public class Expression implements Cloneable {
 
                 nav.beforeFirst();
 
-                for (int i = 0; nav.hasNext(); i++) {
-                    Object[] data = nav.getNextRowData();
+                for (int i = 0; nav.next(); i++) {
+                    Object[] data = nav.getCurrent();
 
                     array[i] = data[0];
                 }
@@ -1648,8 +1609,8 @@ public class Expression implements Cloneable {
 
                 navigator.beforeFirst();
 
-                for (int i = 0; navigator.hasNext(); i++) {
-                    Object[] data = navigator.getNext();
+                for (int i = 0; navigator.next(); i++) {
+                    Object[] data = navigator.getCurrent();
 
                     array[i] = data[0];
                 }
