@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,9 +41,9 @@ import org.hsqldb.types.Types;
 /**
  * Implementation of arithmetic and concatenation operations
  *
- * @author Campbell Burnet (boucherb@users dot sourceforge.net)
+ * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.5
  * @since 1.9.0
  */
 public class ExpressionArithmetic extends Expression {
@@ -313,6 +313,23 @@ public class ExpressionArithmetic extends Expression {
 
     void resolveTypesForArithmetic(Session session, Expression parent) {
 
+        if (nodes[LEFT].isUnresolvedParam()
+                && nodes[RIGHT].isUnresolvedParam()) {
+            if (parent instanceof ExpressionLogical) {
+                Expression e = parent.nodes[LEFT];
+
+                if (e == this) {
+                    e = parent.nodes[RIGHT];
+                }
+
+                if (e.dataType != null) {
+                    if (e.dataType.isDateOrTimestampType()) {
+                        nodes[LEFT].dataType = e.dataType;
+                    }
+                }
+            }
+        }
+
         if (nodes[LEFT].isUnresolvedParam()
                 && nodes[RIGHT].isUnresolvedParam()) {
             nodes[LEFT].dataType  = Type.SQL_INTEGER;
@@ -451,33 +468,36 @@ public class ExpressionArithmetic extends Expression {
         }
 
         // datetime subtract - type predetermined
-        if (dataType != null && dataType.isIntervalType()) {
-            if (nodes[LEFT].dataType.isDateTimeType()
-                    && nodes[RIGHT].dataType.isDateTimeType()) {
-                if (nodes[LEFT].dataType.typeComparisonGroup
-                        != nodes[RIGHT].dataType.typeComparisonGroup) {
-                    throw Error.error(ErrorCode.X_42562);
-                }
-            } else {
-                Type type = nodes[LEFT].dataType.getCombinedType(session,
-                    nodes[RIGHT].dataType, opType);
-
-                if (type == null) {
-                    throw Error.error(ErrorCode.X_42562);
-                }
-
-                if (type.isIntervalType()) {
-                    if (type.typeCode != dataType.typeCode) {
+        if (dataType != null) {
+            if (dataType.isIntervalType()) {
+                if (nodes[LEFT].dataType.isDateTimeType()
+                        && nodes[RIGHT].dataType.isDateTimeType()) {
+                    if (nodes[LEFT].dataType.typeComparisonGroup
+                            != nodes[RIGHT].dataType.typeComparisonGroup) {
                         throw Error.error(ErrorCode.X_42562);
                     }
-                } else if (type.isNumberType()) {
-                    nodes[LEFT]  = new ExpressionOp(nodes[LEFT], dataType);
-                    nodes[RIGHT] = new ExpressionOp(nodes[RIGHT], dataType);
-
-                    nodes[LEFT].resolveTypes(session, this);
-                    nodes[RIGHT].resolveTypes(session, this);
                 } else {
-                    throw Error.error(ErrorCode.X_42562);
+                    Type type = nodes[LEFT].dataType.getCombinedType(session,
+                        nodes[RIGHT].dataType, opType);
+
+                    if (type == null) {
+                        throw Error.error(ErrorCode.X_42562);
+                    }
+
+                    if (type.isIntervalType()) {
+                        if (type.typeCode != dataType.typeCode) {
+                            throw Error.error(ErrorCode.X_42562);
+                        }
+                    } else if (type.isNumberType()) {
+                        nodes[LEFT] = new ExpressionOp(nodes[LEFT], dataType);
+                        nodes[RIGHT] = new ExpressionOp(nodes[RIGHT],
+                                                        dataType);
+
+                        nodes[LEFT].resolveTypes(session, this);
+                        nodes[RIGHT].resolveTypes(session, this);
+                    } else {
+                        throw Error.error(ErrorCode.X_42562);
+                    }
                 }
             }
         } else {
@@ -632,13 +652,6 @@ public class ExpressionArithmetic extends Expression {
             case OpTypes.VALUE :
                 return valueData;
 
-            case OpTypes.SIMPLE_COLUMN : {
-                Object value =
-                    session.sessionContext.rangeIterators[rangePosition]
-                        .getCurrent(columnIndex);
-
-                return value;
-            }
             case OpTypes.NEGATE :
                 return dataType.negate(nodes[LEFT].getValue(session,
                         nodes[LEFT].dataType));
