@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,7 @@ import org.hsqldb.types.Type;
  * Implementation of column, variable, parameter, etc. access operations.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.3.5
  * @since 1.9.0
  */
 public class ExpressionColumn extends Expression {
@@ -133,15 +133,15 @@ public class ExpressionColumn extends Expression {
     }
 
     /**
-     * Creates a temporary OpTypes.COLUMN expression
+     * Creates a temporary OpTypes.SIMPLE_COLUMN expression
      */
     ExpressionColumn(Expression e, int colIndex, int rangePosition) {
 
         super(OpTypes.SIMPLE_COLUMN);
 
-        dataType           = e.dataType;
-        columnIndex        = colIndex;
-        alias              = e.alias;
+        this.dataType      = e.dataType;
+        this.columnIndex   = colIndex;
+        this.alias         = e.getSimpleName();
         this.rangePosition = rangePosition;
     }
 
@@ -666,9 +666,8 @@ public class ExpressionColumn extends Expression {
             case OpTypes.COLUMN : {
                 RangeIterator[] iterators =
                     session.sessionContext.rangeIterators;
-                Object value =
-                    iterators[rangeVariable.rangePosition].getCurrent(
-                        columnIndex);
+                Object value = iterators[rangeVariable.rangePosition].getField(
+                    columnIndex);
 
                 if (dataType != column.dataType) {
                     value = dataType.convertToType(session, value,
@@ -680,7 +679,7 @@ public class ExpressionColumn extends Expression {
             case OpTypes.SIMPLE_COLUMN : {
                 Object value =
                     session.sessionContext.rangeIterators[rangePosition]
-                        .getCurrent(columnIndex);
+                        .getField(columnIndex);
 
                 return value;
             }
@@ -901,11 +900,11 @@ public class ExpressionColumn extends Expression {
     static void checkColumnsResolved(HsqlList set) {
 
         if (set != null && !set.isEmpty()) {
-            StringBuffer sb = new StringBuffer();
-            Expression   e  = (Expression) set.get(0);
+            Expression e = (Expression) set.get(0);
 
             if (e instanceof ExpressionColumn) {
-                ExpressionColumn c = (ExpressionColumn) e;
+                StringBuffer     sb = new StringBuffer();
+                ExpressionColumn c  = (ExpressionColumn) e;
 
                 if (c.schema != null) {
                     sb.append(c.schema + '.');
@@ -1121,7 +1120,7 @@ public class ExpressionColumn extends Expression {
     }
 
     /**
-     * SIMPLE_COLUMN expressions can be of different Expression subclass types
+     * SIMPLE_COLUMN expressions are now (2.3.5) always ExpressionColumn
      */
     public boolean equals(Expression other) {
 
@@ -1140,7 +1139,9 @@ public class ExpressionColumn extends Expression {
         switch (opType) {
 
             case OpTypes.SIMPLE_COLUMN :
-                return this.columnIndex == other.columnIndex;
+                return this.columnIndex == other.columnIndex
+                       && rangeVariable
+                          == ((ExpressionColumn) other).rangeVariable;
 
             case OpTypes.COALESCE :
                 return nodes == other.nodes;
