@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -46,9 +46,9 @@ import org.hsqldb.types.Type;
 import org.hsqldb.types.Types;
 
 /**
- * @author Campbell Burnet (boucherb@users dot sourceforge.net)
+ * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.3.5
  * @since 1.9.0
  */
 public class ExpressionLogical extends Expression {
@@ -934,8 +934,7 @@ public class ExpressionLogical extends Expression {
 
             if (!nodes[LEFT].dataType.canCompareDirect(
                     nodes[RIGHT].dataType)) {
-                if (convertDateTimeLiteral(session, nodes[LEFT],
-                                           nodes[RIGHT])) {
+                if (convertDateTime(session)) {
 
                     // compatibility for BIT with number and BOOLEAN - convert bit to other type
                 } else if (nodes[LEFT].dataType.isBitType()
@@ -1098,39 +1097,49 @@ public class ExpressionLogical extends Expression {
      * for compatibility, convert a datetime character string to a datetime
      * value for comparison
      */
-    private boolean convertDateTimeLiteral(Session session, Expression a,
-                                           Expression b) {
+    private boolean convertDateTime(Session session) {
 
-        if (a.dataType.isDateTimeType()) {
+        int a = LEFT;
+        int b = RIGHT;
+
+        if (nodes[a].dataType.isDateTimeType()) {
 
             //
-        } else if (b.dataType.isDateTimeType()) {
-            Expression c = a;
-
-            a = b;
-            b = c;
+        } else if (nodes[b].dataType.isDateTimeType()) {
+            a = RIGHT;
+            b = LEFT;
         } else {
             return false;
         }
 
-        if (a.dataType.isDateTimeTypeWithZone()) {
+        if (nodes[a].dataType.isDateTimeTypeWithZone()) {
             return false;
         }
 
-        if (b.opType == OpTypes.VALUE && b.dataType.isCharacterType()) {
-            try {
-                b.valueData = a.dataType.castToType(session, b.valueData,
-                                                    b.dataType);
-                b.dataType = a.dataType;
-            } catch (HsqlException e) {
-                if (a.dataType == Type.SQL_DATE) {
-                    b.valueData = Type.SQL_TIMESTAMP.castToType(session,
-                            b.valueData, b.dataType);
-                    b.dataType = Type.SQL_TIMESTAMP;
+        if (nodes[b].dataType.isCharacterType()) {
+            if (nodes[b].opType == OpTypes.VALUE) {
+                try {
+                    nodes[b].valueData = nodes[a].dataType.castToType(session,
+                            nodes[b].valueData, nodes[b].dataType);
+                    nodes[b].dataType = nodes[a].dataType;
+                } catch (HsqlException e) {
+                    if (nodes[a].dataType == Type.SQL_DATE) {
+                        nodes[b].valueData =
+                            Type.SQL_TIMESTAMP.castToType(session,
+                                                          nodes[b].valueData,
+                                                          nodes[b].dataType);
+                        nodes[b].dataType = Type.SQL_TIMESTAMP;
+                    }
                 }
-            }
 
-            return true;
+                return true;
+            } else {
+                nodes[b] = new ExpressionOp(nodes[b], nodes[a].dataType);
+
+                nodes[b].resolveTypes(session, this);
+
+                return true;
+            }
         }
 
         return false;
@@ -1248,13 +1257,6 @@ public class ExpressionLogical extends Expression {
             case OpTypes.VALUE :
                 return valueData;
 
-            case OpTypes.SIMPLE_COLUMN : {
-                Object value =
-                    session.sessionContext.rangeIterators[rangePosition]
-                        .getCurrent(columnIndex);
-
-                return value;
-            }
             case OpTypes.NEGATE :
                 return ((NumberType) dataType).negate(
                     nodes[LEFT].getValue(session, nodes[LEFT].dataType));
@@ -1748,8 +1750,8 @@ public class ExpressionLogical extends Expression {
                     RowIterator it =
                         nodes[RIGHT].getTable().rowIterator(session);
 
-                    while (it.hasNext()) {
-                        Object[] rowData = it.getNextRow().getData();
+                    while (it.next()) {
+                        Object[] rowData = it.getCurrent();
                         Boolean result = compareValues(session, data, rowData);
 
                         if (result == null) {
@@ -1773,10 +1775,10 @@ public class ExpressionLogical extends Expression {
                                     : Boolean.FALSE;
                 }
 
-                RowIterator it =
-                    nodes[RIGHT].getTable().getPrimaryIndex().findFirstRow(
-                        session, store, data);
-                boolean result = it.hasNext();
+                // now nulls == 0;
+                RowIterator it = nodes[RIGHT].getTable().getFullIndex(
+                    session).findFirstRow(session, store, data);
+                boolean result = it.next();
 
                 if (!result) {
                     return Boolean.FALSE;
@@ -1790,21 +1792,23 @@ public class ExpressionLogical extends Expression {
                         return Boolean.TRUE;
                 }
 
-                it.getNextRow();
+                // only one match allowed for MATCH UNIQUE xxxx
+                while (true) {
+                    result = it.next();
 
-                result = it.hasNext();
+                    if (!result) {
+                        break;
+                    }
 
-                if (!result) {
-                    return Boolean.TRUE;
+                    Object[] rowData = it.getCurrent();
+
+                    if (Boolean.TRUE.equals(compareValues(session, data,
+                                                          rowData))) {
+                        return Boolean.FALSE;
+                    }
                 }
 
-                Object[] rowData = it.getNextRow().getData();
-                Boolean returnValue =
-                    Boolean.TRUE.equals(compareValues(session, data, rowData))
-                    ? Boolean.FALSE
-                    : Boolean.TRUE;
-
-                return returnValue;
+                return Boolean.TRUE;
             }
             default : {
                 throw Error.runtimeError(ErrorCode.U_S0500,
@@ -1841,8 +1845,6 @@ public class ExpressionLogical extends Expression {
         Index           index = table.getFullIndex(session);
         RowIterator     it;
         PersistentStore store = table.getRowStore(session);
-        Row             firstrow;
-        Row             lastrow;
         Object[]        firstdata;
         Object[]        lastdata;
         boolean         hasNullValue = false;
@@ -1868,7 +1870,7 @@ public class ExpressionLogical extends Expression {
                 if (opType == OpTypes.EQUAL) {
                     it = index.findFirstRow(session, store, data);
 
-                    if (it.hasNext()) {
+                    if (it.next()) {
                         return Boolean.TRUE;
                     } else {
                         if (hasNullValue) {
@@ -1885,15 +1887,17 @@ public class ExpressionLogical extends Expression {
                     it = index.findFirstRowNotNull(session, store);
                 }
 
-                firstrow = it.getNextRow();
-
-                if (firstrow == null) {
+                if (!it.next()) {
                     return null;
                 }
 
-                firstdata = firstrow.getData();
-                lastrow = index.lastRow(session, store, 0, null).getNextRow();
-                lastdata  = lastrow.getData();
+                firstdata = it.getCurrent();
+
+                RowIterator lastIt = index.lastRow(session, store, 0, null);
+
+                lastIt.next();
+
+                lastdata = lastIt.getCurrent();
 
                 Boolean comparefirst = compareValues(session, data, firstdata);
                 Boolean comparelast  = compareValues(session, data, lastdata);
@@ -1937,9 +1941,11 @@ public class ExpressionLogical extends Expression {
                     return null;
                 }
 
-                it        = index.firstRow(session, store, 0, null);
-                firstrow  = it.getNextRow();
-                firstdata = firstrow.getData();
+                it = index.firstRow(session, store, 0, null);
+
+                it.next();
+
+                firstdata = it.getCurrent();
 
                 if (countNulls(firstdata) == data.length) {
                     return null;
@@ -1951,7 +1957,7 @@ public class ExpressionLogical extends Expression {
                 it = index.findFirstRow(session, store, data);
 
                 if (opType == OpTypes.EQUAL) {
-                    if (it.hasNext()) {
+                    if (it.next()) {
                         return store.elementCount(session) == 1 ? Boolean.TRUE
                                                                 : Boolean
                                                                 .FALSE;
@@ -1961,12 +1967,15 @@ public class ExpressionLogical extends Expression {
                 }
 
                 if (opType == OpTypes.NOT_EQUAL) {
-                    return it.hasNext() ? Boolean.FALSE
-                                        : Boolean.TRUE;
+                    return it.next() ? Boolean.FALSE
+                                     : Boolean.TRUE;
                 }
 
-                lastrow  = index.lastRow(session, store, 0, null).getNextRow();
-                lastdata = lastrow.getData();
+                RowIterator lastIt = index.lastRow(session, store, 0, null);
+
+                lastIt.next();
+
+                lastdata = lastIt.getCurrent();
 
                 Boolean comparefirst = compareValues(session, data, firstdata);
                 Boolean comparelast  = compareValues(session, data, lastdata);
