@@ -44,7 +44,7 @@ import org.hsqldb.types.Type;
  * Implementation of SQL-invoked user-defined function calls - PSM and JRT
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.0.1
+ * @version 2.3.5
  * @since 1.9.0
  */
 public class FunctionSQLInvoked extends Expression {
@@ -119,7 +119,7 @@ public class FunctionSQLInvoked extends Expression {
 
     private Object getValueInternal(Session session, Object[] aggregateData) {
 
-        boolean  isValue       = false;
+        boolean  isValue = false;
         Result   result;
         int      extraArg = routine.javaMethodWithConnection ? 1
                                                              : 0;
@@ -188,14 +188,6 @@ public class FunctionSQLInvoked extends Expression {
 
     public Object getValue(Session session) {
 
-        if (opType == OpTypes.SIMPLE_COLUMN) {
-            Object value =
-                session.sessionContext.rangeIterators[rangePosition]
-                    .getCurrent(columnIndex);
-
-            return value;
-        }
-
         Object returnValue = getValueInternal(session, null);
 
         if (returnValue instanceof Result) {
@@ -273,8 +265,9 @@ public class FunctionSQLInvoked extends Expression {
         if (other instanceof FunctionSQLInvoked) {
             FunctionSQLInvoked o = (FunctionSQLInvoked) other;
 
-            return super.equals(other) && opType == other.opType && routineSchema == o.routineSchema
-                    && routine == o.routine && condition.equals(o.condition);
+            return super.equals(other) && opType == other.opType
+                   && routineSchema == o.routineSchema && routine == o.routine
+                   && condition.equals(o.condition);
         }
 
         return false;
