@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -60,7 +60,7 @@ import org.hsqldb.types.Types;
  * timezone.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.3.5
  * @since 1.7.0
  */
 public class HsqlDateTime {
@@ -69,8 +69,7 @@ public class HsqlDateTime {
      * A reusable static value for today's date. Should only be accessed
      * by getToday()
      */
-    public static Locale         defaultLocale = Locale.UK;
-    private static long          currentDateMillis;
+    public static Locale         defaultLocale  = Locale.UK;
     public static final Calendar tempCalDefault = new GregorianCalendar();
     public static final Calendar tempCalGMT =
         new GregorianCalendar(TimeZone.getTimeZone("GMT"), defaultLocale);
@@ -97,10 +96,6 @@ public class HsqlDateTime {
         sdfts.setLenient(false);
     }
 
-    static {
-        currentDateMillis = getNormalisedDate(System.currentTimeMillis());
-    }
-
     public static long getDateSeconds(String s) {
 
         try {
@@ -159,15 +154,6 @@ public class HsqlDateTime {
         }
     }
 
-    public static synchronized long getCurrentDateMillis(long millis) {
-
-        if (millis - currentDateMillis >= 24 * 3600 * 1000) {
-            currentDateMillis = getNormalisedDate(millis);
-        }
-
-        return currentDateMillis;
-    }
-
     private static java.util.Date sysDate = new java.util.Date();
 
     public static String getSystemTimeString() {
@@ -249,17 +235,6 @@ public class HsqlDateTime {
         cal.setTimeInMillis(millis);
     }
 
-    /**
-     * Gets the time from the given Calendar as a milliseconds value; wrapper method to
-     * allow use of more efficient JDK1.4 method on JDK1.4 (was protected in earlier versions).
-     *
-     * @param       cal                             the Calendar
-     * @return      the time value in milliseconds
-     */
-    public static long getTimeInMillis(Calendar cal) {
-        return cal.getTimeInMillis();
-    }
-
     public static long convertToNormalisedTime(long t) {
         return convertToNormalisedTime(t, tempCalGMT);
     }
@@ -270,7 +245,7 @@ public class HsqlDateTime {
             setTimeInMillis(cal, t);
             resetToDate(cal);
 
-            long t1 = getTimeInMillis(cal);
+            long t1 = cal.getTimeInMillis();
 
             return t - t1;
         }
@@ -284,7 +259,7 @@ public class HsqlDateTime {
             setTimeInMillis(cal, t);
             resetToTime(cal);
 
-            return getTimeInMillis(cal);
+            return cal.getTimeInMillis();
         }
     }
 
@@ -294,7 +269,7 @@ public class HsqlDateTime {
             setTimeInMillis(cal, t);
             resetToTime(cal);
 
-            return getTimeInMillis(cal);
+            return cal.getTimeInMillis();
         }
     }
 
@@ -304,7 +279,7 @@ public class HsqlDateTime {
             setTimeInMillis(tempCalGMT, d);
             resetToDate(tempCalGMT);
 
-            return getTimeInMillis(tempCalGMT);
+            return tempCalGMT.getTimeInMillis();
         }
     }
 
@@ -314,7 +289,7 @@ public class HsqlDateTime {
             setTimeInMillis(cal, t);
             resetToDate(cal);
 
-            return getTimeInMillis(cal);
+            return cal.getTimeInMillis();
         }
     }
 
@@ -499,7 +474,7 @@ public class HsqlDateTime {
         "MMM", "MMMMM",
         "MM",
         "EEEE", "EE",
-        "'*WW'", "w", "dd", "D",
+        "'*WW'", "'*IW'", "dd", "D",
         "'*W'",
         "HH", "KK", "KK",
         "mm", "ss",
@@ -529,7 +504,8 @@ public class HsqlDateTime {
     private static final char e = 0xffff;
 
     public static TimestampData toDate(String string, String pattern,
-                                       SimpleDateFormat format) {
+                                       SimpleDateFormat format,
+                                       boolean fraction) {
 
         long   millis;
         int    nanos       = 0;
@@ -569,7 +545,7 @@ public class HsqlDateTime {
             throw Error.error(ErrorCode.X_22007, e.toString());
         }
 
-        if (matchIndex >= 0) {
+        if (matchIndex >= 0 && fraction) {
             javaPattern = tempPattern;
 
             try {
@@ -624,6 +600,8 @@ public class HsqlDateTime {
 
             if (weekOfYear == 1 && cal.get(Calendar.DAY_OF_YEAR) > 360) {
                 year++;
+            } else if (weekOfYear > 51 && cal.get(Calendar.DAY_OF_YEAR) < 4) {
+                year--;
             }
 
             String yearString = String.valueOf(year);
@@ -654,6 +632,21 @@ public class HsqlDateTime {
             result = sb.toString();
         }
 
+        matchIndex = result.indexOf("*IW");
+
+        if (matchIndex >= 0) {
+            Calendar cal         = format.getCalendar();
+            int      matchLength = 3;
+            int weekOfYear = getDateTimePart(date.getTime(),
+                                             Calendar.WEEK_OF_YEAR);
+            StringBuilder sb = new StringBuilder(result);
+
+            sb.replace(matchIndex, matchIndex + matchLength,
+                       String.valueOf(weekOfYear));
+
+            result = sb.toString();
+        }
+
         matchIndex = result.indexOf("*W");
 
         if (matchIndex >= 0) {
