@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -31,13 +31,13 @@
 
 package org.hsqldb;
 
+import java.util.concurrent.atomic.AtomicLong;
+
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.lib.StringConverter;
 import org.hsqldb.rights.Grantee;
 
-import java.util.concurrent.atomic.AtomicInteger;
-
 /**
  * Provides Name Management for SQL objects. <p>
  *
@@ -61,7 +61,7 @@ import java.util.concurrent.atomic.AtomicInteger;
  * than all the existing names.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.2
+ * @version 2.3.5
  * @since 1.7.2
  */
 public final class HsqlNameManager {
@@ -69,7 +69,7 @@ public final class HsqlNameManager {
     private static final HsqlNameManager staticManager = new HsqlNameManager();
 
     static {
-        staticManager.serialNumber.set(Integer.MIN_VALUE);
+        staticManager.serialNumber.set(Long.MIN_VALUE);
     }
 
     private static final HsqlName[] autoColumnNames       = new HsqlName[32];
@@ -83,8 +83,8 @@ public final class HsqlNameManager {
         }
     }
 
-    private AtomicInteger serialNumber = new AtomicInteger(1);    // 0 is reserved in lookups
-    private int      sysNumber = 10000;                           // avoid name clash in older scripts
+    private AtomicLong serialNumber = new AtomicLong(1);     // 0 is reserved in lookups
+    private AtomicLong sysNumber = new AtomicLong(10000);    // avoid name clash in older scripts
     private HsqlName catalogName;
     private boolean  sqlRegularNames;
     HsqlName         subqueryTableName;
@@ -217,11 +217,36 @@ public final class HsqlNameManager {
         return name;
     }
 
+    public HsqlName newConstraintIndexName(HsqlName tableName,
+                                           HsqlName constName,
+                                           boolean useConstraint) {
+
+        if (constName == null) {
+            useConstraint = false;
+        }
+
+        if (useConstraint) {
+            HsqlName name = new HsqlName(this, constName.name,
+                                         SchemaObject.INDEX, false);
+
+            name.schema = tableName.schema;
+            name.parent = tableName;
+
+            return name;
+        } else {
+            String constNameString = constName == null ? null
+                                                       : constName.name;
+
+            return newAutoName("IDX", constNameString, tableName.schema,
+                               tableName, SchemaObject.INDEX);
+        }
+    }
+
     public HsqlName newSpecificRoutineName(HsqlName name) {
 
         StringBuffer sb = new StringBuffer();
 
-        sb.append(name.name).append('_').append(++sysNumber);
+        sb.append(name.name).append('_').append(sysNumber.incrementAndGet());
 
         HsqlName hsqlName = new HsqlName(this, sb.toString(),
                                          SchemaObject.SPECIFIC_ROUTINE,
@@ -283,7 +308,7 @@ public final class HsqlNameManager {
                     sb.append('_');
                 }
 
-                sb.append(++sysNumber);
+                sb.append(sysNumber.incrementAndGet());
             }
         } else {
             sb.append(namepart);
@@ -320,8 +345,7 @@ public final class HsqlNameManager {
         public boolean equals(Object other) {
 
             if (other instanceof SimpleName) {
-                return ((SimpleName) other).isNameQuoted == isNameQuoted
-                       && ((SimpleName) other).name.equals(name);
+                return ((SimpleName) other).name.equals(name);
             }
 
             return false;
@@ -353,7 +377,7 @@ public final class HsqlNameManager {
         public final int type;
 
         //
-        private final int hashCode;
+        private final long hashCode;
 
         private HsqlName(HsqlNameManager man, int type) {
 
@@ -460,8 +484,8 @@ public final class HsqlNameManager {
                 try {
                     int temp = Integer.parseInt(name.substring(length));
 
-                    if (temp > manager.sysNumber) {
-                        manager.sysNumber = temp;
+                    if (temp > manager.sysNumber.get()) {
+                        manager.sysNumber.set(temp);
                     }
                 } catch (NumberFormatException e) {}
             }
@@ -496,7 +520,7 @@ public final class HsqlNameManager {
          * hash code for this object is its unique serial number.
          */
         public int hashCode() {
-            return hashCode;
+            return (int) hashCode;
         }
 
         /**
@@ -539,10 +563,6 @@ public final class HsqlNameManager {
                    + ", isNameQuoted=" + isNameQuoted + "]";
         }
 
-        public int compareTo(Object o) {
-            return hashCode - o.hashCode();
-        }
-
         /**
          * Returns true if the identifier consists of all uppercase letters
          * digits and underscore, beginning with a letter and is not in the
