@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -107,6 +107,12 @@ public class ParserCommand extends ParserDDL {
             list.add(cs);
         }
 
+        if (list.size() > 1) {
+            if (database.sqlRestrictExec) {
+                throw Error.error(ErrorCode.X_07502);
+            }
+        }
+
         int returnType = cmd.getStatementType();
 
         if (returnType != StatementTypes.RETURN_ANY) {
@@ -118,8 +124,11 @@ public class ParserCommand extends ParserDDL {
                 }
             } else if (returnType == StatementTypes.RETURN_RESULT) {
 
-                // allow update count statements to return an empty result set
-                // throw Error.error(ErrorCode.X_07504);
+                // allow update count statements with Statement.executeQuery()
+                // to return an empty result set
+                if (database.sqlRestrictExec) {
+                    throw Error.error(ErrorCode.X_07504);
+                }
             }
         }
 
@@ -398,7 +407,8 @@ public class ParserCommand extends ParserDDL {
             new ExpressionValue(password, Type.SQL_VARCHAR)
         };
         Statement cs =
-            new StatementSession(StatementTypes.SET_SESSION_AUTHORIZATION,
+            new StatementSession(session, compileContext,
+                                 StatementTypes.SET_SESSION_AUTHORIZATION,
                                  args);
 
         return cs;
@@ -563,7 +573,8 @@ public class ParserCommand extends ParserDDL {
 
                 Expression[] args = new Expression[]{ e };
 
-                return new StatementSession(StatementTypes.SET_CATALOG, args);
+                return new StatementSession(session, compileContext,
+                                            StatementTypes.SET_CATALOG, args);
             }
             case Tokens.SCHEMA : {
                 read();
@@ -578,19 +589,35 @@ public class ParserCommand extends ParserDDL {
                                                 args);
                 }
 
+                if (e.isUnresolvedParam()) {
+                    e.setDataType(session, Type.SQL_VARCHAR_DEFAULT);
+                }
+
                 if (!e.getDataType().isCharacterType()) {
                     throw Error.error(ErrorCode.X_0P000);
                 }
 
-                if (e.getType() != OpTypes.VALUE
-                        && (e.getType() != OpTypes.SQL_FUNCTION
-                            || !((FunctionSQL) e).isValueFunction())) {
-                    throw Error.error(ErrorCode.X_0P000);
+                switch (e.getType()) {
+
+                    case OpTypes.PARAMETER :
+                    case OpTypes.DYNAMIC_PARAM :
+                    case OpTypes.VALUE :
+                        break;
+
+                    case OpTypes.SQL_FUNCTION :
+                        if (((FunctionSQL) e).isValueFunction()) {
+                            break;
+                        }
+
+                    // fall through
+                    default :
+                        throw Error.error(ErrorCode.X_0P000);
                 }
 
                 Expression[] args = new Expression[]{ e };
 
-                return new StatementSession(StatementTypes.SET_SCHEMA, args);
+                return new StatementSession(session, compileContext,
+                                            StatementTypes.SET_SCHEMA, args);
             }
             case Tokens.NO : {
                 read();
@@ -1153,16 +1180,20 @@ public class ParserCommand extends ParserDDL {
 
                 switch (token.tokenType) {
 
+                    case Tokens.RESTRICT :
+                        read();
+                        readThis(Tokens.EXEC);
+
+                        property = HsqlDatabaseProperties.sql_restrict_exec;
+                        flag     = processTrueOrFalseObject();
+                        break;
+
                     case Tokens.LIVE :
                         read();
                         readThis(Tokens.OBJECT);
 
                         property = HsqlDatabaseProperties.sql_live_object;
                         flag     = processTrueOrFalseObject();
-
-                        Object[] args = new Object[] {
-                            property, flag, value
-                        };
                         break;
 
                     case Tokens.NAMES :
@@ -1333,6 +1364,16 @@ public class ParserCommand extends ParserDDL {
                         flag = processTrueOrFalseObject();
                         break;
 
+                    case Tokens.SYS : {
+                        read();
+                        readThis(Tokens.INDEX);
+                        readThis(Tokens.NAMES);
+
+                        flag     = processTrueOrFalseObject();
+                        property = HsqlDatabaseProperties.sql_sys_index_names;
+
+                        break;
+                    }
                     default :
                         throw unexpectedToken();
                 }
@@ -2003,6 +2044,7 @@ public class ParserCommand extends ParserDDL {
                 };
 
                 return new StatementSession(
+                    session, compileContext,
                     StatementTypes.SET_SESSION_AUTHORIZATION, args);
             }
             case Tokens.RESULT : {
@@ -2059,7 +2101,8 @@ public class ParserCommand extends ParserDDL {
             }
         }
 
-        return new StatementSession(StatementTypes.SET_ROLE,
+        return new StatementSession(session, compileContext,
+                                    StatementTypes.SET_ROLE,
                                     new Expression[]{ e });
     }
 
@@ -2091,7 +2134,8 @@ public class ParserCommand extends ParserDDL {
             }
         }
 
-        return new StatementSession(StatementTypes.SET_TIME_ZONE,
+        return new StatementSession(session, compileContext,
+                                    StatementTypes.SET_TIME_ZONE,
                                     new Expression[]{ e });
     }
 
@@ -2273,24 +2317,32 @@ public class ParserCommand extends ParserDDL {
 
     private Statement compilePerform() {
 
+        Integer type   = Integer.valueOf(1);
+        Integer number = Integer.valueOf(-1);
+
         read();
 
         switch (token.tokenType) {
 
             case Tokens.CHECK : {
                 read();
+
+                HsqlName tableName = null;
+
                 readThis(Tokens.TABLE);
+                readThis(Tokens.SPACE);
 
-                Table table = readTableName();
+                if (readIfThis(Tokens.AND)) {
+                    readThis("FIX");
 
-                readThis(Tokens.INDEX);
+                    type = Integer.valueOf(2);
+                }
 
                 Object[] args = new Object[] {
-                    table.getName(), Integer.valueOf(1), null
+                    tableName, type, number
                 };
                 HsqlName[] names =
-                    database.schemaManager.getCatalogAndBaseTableNames(
-                        table.getName());
+                    database.schemaManager.getCatalogAndBaseTableNames();
 
                 return new StatementCommand(StatementTypes.CHECK_INDEX, args,
                                             null, names);
