@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -388,7 +388,7 @@ public class QueryExpression implements RangeGroup {
             throw Error.error(ErrorCode.X_42576);
         }
 
-        sortAndSlice.prepare(null);
+        sortAndSlice.prepare(0);
     }
 
     private void addUnresolvedExpressions(HsqlList expressions) {
@@ -1126,5 +1126,17 @@ public class QueryExpression implements RangeGroup {
         rightQueryExpression.replaceRangeVariables(ranges, newRanges);
     }
 
+    /**
+     * non-working temp code for replacing aggregate functions with simple column
+     */
+    public void replaceExpressions(OrderedHashSet expressions,
+                                   int resultRangePosition) {
+
+        leftQueryExpression.replaceExpressions(expressions,
+                                               resultRangePosition);
+        rightQueryExpression.replaceExpressions(expressions,
+                resultRangePosition);
+    }
+
     public void setAsExists() {}
 }
