@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -34,7 +34,6 @@ package org.hsqldb;
 import org.hsqldb.HsqlNameManager.HsqlName;
 import org.hsqldb.HsqlNameManager.SimpleName;
 import org.hsqldb.ParserDQL.CompileContext;
-import org.hsqldb.RangeGroup.RangeGroupSimple;
 import org.hsqldb.RangeVariable.RangeIteratorRight;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
@@ -65,7 +64,7 @@ import org.hsqldb.types.Types;
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
  *
- * @version 2.3.4
+ * @version 2.3.5
  * @since 1.9.0
  */
 public class QuerySpecification extends QueryExpression {
@@ -87,10 +86,10 @@ public class QuerySpecification extends QueryExpression {
     Expression            rowExpression;
     Expression[]          exprColumns;
     HsqlArrayList         exprColumnList;
-    public int            indexLimitVisible;
-    private int           indexLimitRowId;
     private int           groupByColumnCount;    // columns in 'group by'
     private int           havingColumnCount;     // columns in 'having' (0 or 1)
+    public int            indexLimitVisible;
+    private int           indexLimitRowId;
     private int           indexStartHaving;
     public int            indexStartOrderBy;
     public int            indexStartAggregates;
@@ -118,9 +117,8 @@ public class QuerySpecification extends QueryExpression {
     private OrderedHashSet tempSet = new OrderedHashSet();
 
     //
-    int[]                  columnMap;
-    private Table          baseTable;
-    private OrderedHashSet conditionTables;      // for view super-view references
+    int[]         columnMap;
+    private Table baseTable;
 
     //
     public Index groupIndex;
@@ -257,6 +255,10 @@ public class QuerySpecification extends QueryExpression {
         queryCondition = e;
     }
 
+    void setDistinctSelect() {
+        isDistinctSelect = true;
+    }
+
     void addGroupByColumnExpression(Expression e) {
 
         if (e.getType() == OpTypes.ROW) {
@@ -295,9 +297,9 @@ public class QuerySpecification extends QueryExpression {
         finaliseColumns();
         resolveColumnReferences(session, rangeGroups);
         setReferenceableColumns();
-        Expression.resolveColumnSet(session, RangeVariable.emptyArray,
-                                    rangeGroups, unresolvedExpressions);
 
+        unresolvedExpressions = Expression.resolveColumnSet(session,
+                RangeVariable.emptyArray, rangeGroups, unresolvedExpressions);
         unionColumnTypes     = new Type[indexLimitVisible];
         isReferencesResolved = true;
     }
@@ -564,7 +566,7 @@ public class QuerySpecification extends QueryExpression {
                     this, rangeGroups, unresolvedExpressions);
         }
 
-        sortAndSlice.prepare(this);
+        sortAndSlice.prepare(indexStartOrderBy);
     }
 
     private boolean resolveColumnReferences(Session session, Expression e,
@@ -1330,7 +1332,7 @@ public class QuerySpecification extends QueryExpression {
                     continue;
                 }
 
-                if (!e.isAggregate()
+                if (!e.hasAggregate()
                         && !e.isComposedOf(
                             exprColumns, 0,
                             indexLimitVisible + groupByColumnCount,
@@ -1344,29 +1346,26 @@ public class QuerySpecification extends QueryExpression {
             return;
         }
 
-        OrderedHashSet expressions       = new OrderedHashSet();
-        OrderedHashSet columnExpressions = new OrderedHashSet();
+        OrderedHashSet expressions = new OrderedHashSet();
 
         for (int i = indexStartAggregates; i < indexLimitExpressions; i++) {
             Expression e = exprColumns[i];
-            Expression c = new ExpressionColumn(e, i, resultRangePosition);
+
+            e.resultTableColumnIndex = i;
 
             expressions.add(e);
-            columnExpressions.add(c);
         }
 
         for (int i = 0; i < indexStartHaving; i++) {
-            if (exprColumns[i].isAggregate()) {
+            if (exprColumns[i].hasAggregate()) {
                 continue;
             }
 
             Expression e = exprColumns[i];
 
-            if (expressions.add(e)) {
-                Expression c = new ExpressionColumn(e, i, resultRangePosition);
+            e.resultTableColumnIndex = i;
 
-                columnExpressions.add(c);
-            }
+            expressions.add(e);
         }
 
         // order by with aggregate
@@ -1375,13 +1374,13 @@ public class QuerySpecification extends QueryExpression {
         for (int i = 0; i < orderCount; i++) {
             Expression e = (Expression) sortAndSlice.exprList.get(i);
 
-            if (e.getLeftNode().isAggregate()) {
+            if (e.getLeftNode().hasAggregate()) {
                 e.setAggregate();
             }
         }
 
         for (int i = indexStartOrderBy; i < indexStartAggregates; i++) {
-            if (exprColumns[i].getLeftNode().isAggregate()) {
+            if (exprColumns[i].getLeftNode().hasAggregate()) {
                 exprColumns[i].setAggregate();
             }
         }
@@ -1389,28 +1388,20 @@ public class QuerySpecification extends QueryExpression {
         for (int i = 0; i < indexStartAggregates; i++) {
             Expression e = exprColumns[i];
 
-            if (!e.isAggregate() /* && !e.isCorrelated() */) {
+            if (!e.hasAggregate() /* && !e.isCorrelated() */) {
                 continue;
             }
 
             aggregateCheck[i] = true;
-
-            if (e.isAggregate()) {
-                e.convertToSimpleColumn(expressions, columnExpressions);
-            }
-        }
-
-        for (int i = 0; i < aggregateSet.size(); i++) {
-            Expression e = (Expression) aggregateSet.get(i);
-
-            e.convertToSimpleColumn(expressions, columnExpressions);
+            exprColumns[i] = e.replaceExpressions(expressions,
+                                                  resultRangePosition);
         }
 
         if (resolvedSubqueryExpressions != null) {
             for (int i = 0; i < resolvedSubqueryExpressions.size(); i++) {
                 Expression e = (Expression) resolvedSubqueryExpressions.get(i);
 
-                e.convertToSimpleColumn(expressions, columnExpressions);
+                e.replaceExpressions(expressions, resultRangePosition);
             }
         }
     }
@@ -1605,7 +1596,7 @@ public class QuerySpecification extends QueryExpression {
 
             for (int i = indexLimitVisible; i < indexLimitRowId; i++) {
                 if (i == indexLimitVisible) {
-                    data[i] = it.getRowidObject();
+                    data[i] = Long.valueOf(it.getRowId());
                 } else {
                     data[i] = it.getCurrentRow();
                 }
@@ -1718,8 +1709,8 @@ public class QuerySpecification extends QueryExpression {
         navigator.reset();
 
         if (havingCondition != null) {
-            while (navigator.hasNext()) {
-                Object[] data = navigator.getNext();
+            while (navigator.next()) {
+                Object[] data = navigator.getCurrent();
 
                 if (!Boolean.TRUE.equals(
                         data[indexLimitVisible + groupByColumnCount])) {
@@ -2418,10 +2409,6 @@ public class QuerySpecification extends QueryExpression {
         return baseTable;
     }
 
-    public OrderedHashSet collectAllSubqueries(OrderedHashSet set) {
-        return set;
-    }
-
     public OrderedHashSet collectOuterColumnExpressions(OrderedHashSet set,
             OrderedHashSet exclude) {
 
@@ -2541,6 +2528,30 @@ public class QuerySpecification extends QueryExpression {
         }
     }
 
+    public void replaceExpressions(OrderedHashSet expressions,
+                                   int resultRangePosition) {
+
+        for (int i = 0; i < indexStartAggregates; i++) {
+            exprColumns[i] = exprColumns[i].replaceExpressions(expressions,
+                    resultRangePosition);
+        }
+
+        if (queryCondition != null) {
+            queryCondition = queryCondition.replaceExpressions(expressions,
+                    resultRangePosition);
+        }
+
+        if (havingCondition != null) {
+            havingCondition = havingCondition.replaceExpressions(expressions,
+                    resultRangePosition);
+        }
+
+        for (int i = 0, len = rangeVariables.length; i < len; i++) {
+            rangeVariables[i].replaceExpressions(expressions,
+                                                 resultRangePosition);
+        }
+    }
+
     /**
      * Not for views. Only used on root node.
      */
