@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,6 @@ import org.hsqldb.lib.HsqlList;
 import org.hsqldb.lib.OrderedHashSet;
 import org.hsqldb.lib.OrderedIntHashSet;
 import org.hsqldb.lib.OrderedLongHashSet;
-import org.hsqldb.map.ValuePool;
 import org.hsqldb.navigator.RangeIterator;
 import org.hsqldb.navigator.RowIterator;
 import org.hsqldb.persist.PersistentStore;
@@ -55,7 +54,7 @@ import org.hsqldb.types.Type;
  * Metadata for range variables, including conditions.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.3.5
  * @since 1.9.0
  */
 public class RangeVariable {
@@ -68,6 +67,9 @@ public class RangeVariable {
     public static final int PARAMETER_RANGE  = 3;
     public static final int VARIALBE_RANGE   = 4;
 
+    //
+    private static final RowIterator emptyIterator = new RangeIteratorEmpty();
+
     //
     Table                  rangeTable;
     final SimpleName       tableAlias;
@@ -76,7 +78,6 @@ public class RangeVariable {
     private OrderedHashSet columnNames;
     OrderedHashSet         namedJoinColumns;
     HashMap                namedJoinColumnExpressions;
-    private Object[]       emptyData;
     boolean[]              columnsInGroupBy;
     boolean                hasKeyedColumnInGroupBy;
     boolean[]              usedColumns;
@@ -130,7 +131,6 @@ public class RangeVariable {
         this.rangeType   = rangeType;
         rangeTable       = null;
         tableAlias       = rangeName;
-        emptyData        = null;
         columnsInGroupBy = null;
         usedColumns      = null;
         joinConditions = new RangeVariableConditions[]{
@@ -178,7 +178,6 @@ public class RangeVariable {
         rangeType        = TABLE_RANGE;
         rangeTable       = table;
         tableAlias       = null;
-        emptyData        = rangeTable.getEmptyRowData();
         columnsInGroupBy = rangeTable.getNewColumnCheckList();
         usedColumns      = rangeTable.getNewColumnCheckList();
         rangePosition    = position;
@@ -195,7 +194,6 @@ public class RangeVariable {
             throw Error.error(ErrorCode.X_42593);
         }
 
-        emptyData                     = rangeTable.getEmptyRowData();
         columnsInGroupBy              = rangeTable.getNewColumnCheckList();
         usedColumns                   = rangeTable.getNewColumnCheckList();
         joinConditions[0].rangeIndex  = rangeTable.getPrimaryIndex();
@@ -404,7 +402,7 @@ public class RangeVariable {
 
         if (namedJoinColumnExpressions != null
                 && namedJoinColumnExpressions.containsKey(columnName)) {
-            if (tableName != null) {
+            if (tableName != null && !resolvesTableName(tableName)) {
                 return -1;
             }
         }
@@ -761,6 +759,38 @@ public class RangeVariable {
         }
     }
 
+    public void replaceExpressions(OrderedHashSet expressions,
+                                   int resultRangePosition) {
+
+        QueryExpression queryExpression = rangeTable.getQueryExpression();
+        Expression      dataExpression  = rangeTable.getDataExpression();
+
+        if (dataExpression != null) {
+            dataExpression = dataExpression.replaceExpressions(expressions,
+                    resultRangePosition);
+        }
+
+        if (queryExpression != null) {
+            queryExpression.replaceExpressions(expressions,
+                                               resultRangePosition);
+        }
+
+        if (joinCondition != null) {
+            joinCondition = joinCondition.replaceExpressions(expressions,
+                    resultRangePosition);
+        }
+
+        for (int i = 0; i < joinConditions.length; i++) {
+            joinConditions[i].replaceExpressions(expressions,
+                                                 resultRangePosition);
+        }
+
+        for (int i = 0; i < whereConditions.length; i++) {
+            whereConditions[i].replaceExpressions(expressions,
+                                                  resultRangePosition);
+        }
+    }
+
     public void resolveRangeTable(Session session, RangeGroup rangeGroup,
                                   RangeGroup[] rangeGroups) {
 
@@ -1076,10 +1106,8 @@ public class RangeVariable {
 
         Session         session;
         int             rangePosition;
-        RowIterator     it;
+        RowIterator     it = emptyIterator;
         PersistentStore store;
-        Object[]        currentData;
-        Row             currentRow;
         boolean         isBeforeFirst;
         RangeVariable   rangeVar;
 
@@ -1094,7 +1122,7 @@ public class RangeVariable {
             if (isBeforeFirst) {
                 isBeforeFirst = false;
             } else {
-                if (it == null) {
+                if (it == emptyIterator) {
                     return false;
                 }
             }
@@ -1107,49 +1135,31 @@ public class RangeVariable {
                 throw Error.error(ErrorCode.X_40502);
             }
 
-            currentRow = it.getNextRow();
-
-            if (currentRow == null) {
-                return false;
-            } else {
-                currentData = currentRow.getData();
-
+            if (it.next()) {
                 return true;
+            } else {
+                return false;
             }
         }
 
         public Row getCurrentRow() {
-            return currentRow;
+            return it.getCurrentRow();
         }
 
         public Object[] getCurrent() {
-            return currentData;
+            return it.getCurrent();
         }
 
-        public Object getCurrent(int i) {
-            return currentData == null ? null
-                                       : currentData[i];
+        public Object getField(int i) {
+            return it.getField(i);
         }
 
         public void setCurrent(Object[] data) {
-            currentData = data;
+            throw Error.runtimeError(ErrorCode.U_S0500, "RangeVariable");
         }
 
         public long getRowId() {
-
-            return currentRow == null ? 0
-                                      : ((long) rangeVar.rangeTable.getId() << 32)
-                                        + currentRow.getPos();
-        }
-
-        public TableBase getCurrentTable() {
-            return currentRow == null ? null
-                                      : currentRow.getTable();
-        }
-
-        public Object getRowidObject() {
-            return currentRow == null ? null
-                                      : ValuePool.getLong(getRowId());
+            return ((long) rangeVar.rangeTable.getId() << 32) + it.getRowId();
         }
 
         public void removeCurrent() {}
@@ -1160,8 +1170,7 @@ public class RangeVariable {
                 it.release();
             }
 
-            it            = null;
-            currentRow    = null;
+            it            = emptyIterator;
             isBeforeFirst = true;
         }
 
@@ -1169,18 +1178,10 @@ public class RangeVariable {
             return rangePosition;
         }
 
-        public Row getNextRow() {
-            throw Error.runtimeError(ErrorCode.U_S0500, "RangeVariable");
-        }
-
         public boolean hasNext() {
             throw Error.runtimeError(ErrorCode.U_S0500, "RangeVariable");
         }
 
-        public Object[] getNext() {
-            throw Error.runtimeError(ErrorCode.U_S0500, "RangeVariable");
-        }
-
         public void release() {
 
             if (it != null) {
@@ -1214,7 +1215,6 @@ public class RangeVariable {
             this.store         = rangeVar.rangeTable.getRowStore(session);
             this.session       = session;
             this.rangeVar      = rangeVar;
-            currentData        = rangeVar.emptyData;
             isBeforeFirst      = true;
             whereConditions    = rangeVar.whereConditions;
             joinConditions     = rangeVar.joinConditions;
@@ -1275,9 +1275,7 @@ public class RangeVariable {
                 it.release();
             }
 
-            it            = null;
-            currentData   = rangeVar.emptyData;
-            currentRow    = null;
+            it            = emptyIterator;
             isBeforeFirst = true;
         }
 
@@ -1440,14 +1438,13 @@ public class RangeVariable {
                     throw Error.error(ErrorCode.X_40502);
                 }
 
-                currentRow = it.getNextRow();
+                if (it.next()) {}
+                else {
+                    it = emptyIterator;
 
-                if (currentRow == null) {
                     break;
                 }
 
-                currentData = currentRow.getData();
-
                 if (conditions[condIndex].terminalCondition != null) {
                     if (!conditions[condIndex].terminalCondition.testCondition(
                             session)) {
@@ -1499,8 +1496,7 @@ public class RangeVariable {
 
             it.release();
 
-            currentRow  = null;
-            currentData = rangeVar.emptyData;
+            it = emptyIterator;
 
             if (hasLeftOuterRow && condIndex == conditions.length - 1) {
                 result =
@@ -1516,7 +1512,9 @@ public class RangeVariable {
         private void addFoundRow() {
 
             if (rangeVar.isRightJoin) {
-                lookup.add(currentRow.getPos());
+                long position = it.getRowId();
+
+                lookup.add(position);
             }
         }
     }
@@ -1546,7 +1544,7 @@ public class RangeVariable {
         public boolean next() {
 
             if (isOnRightOuterRows) {
-                if (it == null) {
+                if (it == emptyIterator) {
                     return false;
                 }
 
@@ -1561,14 +1559,13 @@ public class RangeVariable {
             boolean result = false;
 
             while (true) {
-                currentRow = it.getNextRow();
+                if (it.next()) {}
+                else {
+                    it = emptyIterator;
 
-                if (currentRow == null) {
                     break;
                 }
 
-                currentData = currentRow.getData();
-
                 if (conditions[condIndex].indexEndCondition != null
                         && !conditions[condIndex].indexEndCondition
                             .testCondition(session)) {
@@ -1596,19 +1593,15 @@ public class RangeVariable {
 
             it.release();
 
-            currentRow  = null;
-            currentData = rangeVar.emptyData;
-
             return result;
         }
 
         private boolean lookupAndTest() {
 
-            boolean result = !lookup.contains(currentRow.getPos());
+            long    position = it.getRowId();
+            boolean result   = !lookup.contains(position);
 
             if (result) {
-                currentData = currentRow.getData();
-
                 if (conditions[condIndex].nonIndexCondition != null
                         && !conditions[condIndex].nonIndexCondition
                             .testCondition(session)) {
@@ -1624,12 +1617,21 @@ public class RangeVariable {
 
         RangeIteratorMain[] rangeIterators;
         int                 currentIndex = 0;
+        RangeIterator       currentRange = null;
 
         public RangeIteratorJoined(RangeIteratorMain[] rangeIterators) {
             this.rangeIterators = rangeIterators;
             isBeforeFirst       = true;
         }
 
+        public Row getCurrentRow() {
+            return currentRange.getCurrentRow();
+        }
+
+        public Object[] getCurrent() {
+            return currentRange.getCurrent();
+        }
+
         public boolean isBeforeFirst() {
             return isBeforeFirst;
         }
@@ -1637,21 +1639,18 @@ public class RangeVariable {
         public boolean next() {
 
             while (currentIndex >= 0) {
-                RangeIteratorMain it = rangeIterators[currentIndex];
+                currentRange = rangeIterators[currentIndex];
 
-                if (it.next()) {
+                if (currentRange.next()) {
                     if (currentIndex < rangeIterators.length - 1) {
                         currentIndex++;
 
                         continue;
                     }
 
-                    currentRow  = rangeIterators[currentIndex].currentRow;
-                    currentData = currentRow.getData();
-
                     return true;
                 } else {
-                    it.reset();
+                    currentRange.reset();
 
                     currentIndex--;
 
@@ -1659,9 +1658,7 @@ public class RangeVariable {
                 }
             }
 
-            currentData =
-                rangeIterators[rangeIterators.length - 1].rangeVar.emptyData;
-            currentRow = null;
+            currentRange = null;
 
             for (int i = 0; i < rangeIterators.length; i++) {
                 rangeIterators[i].reset();
@@ -1697,6 +1694,109 @@ public class RangeVariable {
         }
     }
 
+    static final class RangeIteratorEmpty implements RowIterator {
+
+        public Object getField(int col) {
+            return null;
+        }
+
+        public boolean next() {
+            return false;
+        }
+
+        public Row getCurrentRow() {
+            return null;
+        }
+
+        public Object[] getCurrent() {
+            return null;
+        }
+
+        public boolean hasNext() {
+            return false;
+        }
+
+        public long getPos() {
+            return -1;
+        }
+
+        public int getStorageSize() {
+            return 0;
+        }
+
+        public void release() {}
+
+        public void removeCurrent() {}
+
+        public long getRowId() {
+            return 0L;
+        }
+    }
+
+    static final class RangeIteratorCheck implements RangeIterator {
+
+        final int rangePosition;
+        Object[]  currentData;
+
+        RangeIteratorCheck() {
+            this.rangePosition = 0;
+        }
+
+        RangeIteratorCheck(int rangePosition) {
+            this.rangePosition = rangePosition;
+        }
+
+        public Object getField(int col) {
+            return currentData[col];
+        }
+
+        public boolean next() {
+            return false;
+        }
+
+        public Row getCurrentRow() {
+            return null;
+        }
+
+        public Object[] getCurrent() {
+            return currentData;
+        }
+
+        public boolean hasNext() {
+            return false;
+        }
+
+        public long getPos() {
+            return -1;
+        }
+
+        public int getStorageSize() {
+            return 0;
+        }
+
+        public void release() {}
+
+        public void removeCurrent() {}
+
+        public long getRowId() {
+            return 0L;
+        }
+
+        public boolean isBeforeFirst() {
+            return false;
+        }
+
+        public void setCurrent(Object[] data) {
+            this.currentData = data;
+        }
+
+        public void reset() {}
+
+        public int getRangePosition() {
+            return rangePosition;
+        }
+    }
+
     public static class RangeVariableConditions {
 
         final RangeVariable rangeVar;
@@ -1884,7 +1984,7 @@ public class RangeVariable {
 
         /**
          *
-         * @param exprList list of expressions
+         * @param exprList has the same length as index column count
          * @param index Index to use
          * @param colCount number of columns searched
          */
@@ -2098,5 +2198,51 @@ public class RangeVariable {
                     terminalCondition.replaceColumnReferences(range, list);
             }
         }
+
+        private void replaceExpressions(OrderedHashSet expressions,
+                                        int resultRangePosition) {
+
+            if (indexCond != null) {
+                for (int i = 0; i < indexCond.length; i++) {
+                    if (indexCond[i] != null) {
+                        indexCond[i] = indexCond[i].replaceExpressions(
+                            expressions, resultRangePosition);
+                    }
+                }
+            }
+
+            if (indexEndCond != null) {
+                for (int i = 0; i < indexEndCond.length; i++) {
+                    if (indexEndCond[i] != null) {
+                        indexEndCond[i] = indexEndCond[i].replaceExpressions(
+                            expressions, resultRangePosition);
+                    }
+                }
+            }
+
+            if (indexEndCondition != null) {
+                indexEndCondition =
+                    indexEndCondition.replaceExpressions(expressions,
+                        resultRangePosition);
+            }
+
+            if (excludeConditions != null) {
+                excludeConditions =
+                    excludeConditions.replaceExpressions(expressions,
+                        resultRangePosition);
+            }
+
+            if (nonIndexCondition != null) {
+                nonIndexCondition =
+                    nonIndexCondition.replaceExpressions(expressions,
+                        resultRangePosition);
+            }
+
+            if (terminalCondition != null) {
+                terminalCondition =
+                    terminalCondition.replaceExpressions(expressions,
+                        resultRangePosition);
+            }
+        }
     }
 }
