@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -58,7 +58,7 @@ import org.hsqldb.types.Types;
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
  *
- * @version 2.3.3
+ * @version 2.3.4
  * @since 1.9.0
  */
 public class Routine implements SchemaObject, RangeGroup, Cloneable {
@@ -116,6 +116,7 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
 
     //
     int variableCount;
+    int cursorCount;
 
     //
     OrderedHashSet references;
@@ -182,7 +183,8 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
     public void compile(Session session, SchemaObject parentObject) {
 
         ParserRoutine p = new ParserRoutine(session,
-                                            new Scanner(statement.getSQL()));
+                                            new Scanner(session,
+                                                statement.getSQL()));
 
         session.sessionContext.pushRoutineTables();
 
@@ -412,7 +414,7 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
 
         StringBuffer sb = new StringBuffer();
 
-        switch (this.dataImpact) {
+        switch (dataImpact) {
 
             case NO_SQL :
                 sb.append(Tokens.T_NO).append(' ').append(Tokens.T_SQL);
@@ -564,9 +566,10 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
             parameterTypes[i] = param.dataType;
 
             if (i < 4) {
-                BitMap.setByte(typeGroups,
-                               (byte) param.dataType.typeComparisonGroup,
-                               i * 8);
+                typeGroups =
+                    BitMap.setByte(typeGroups,
+                                   (byte) param.dataType.typeComparisonGroup,
+                                   i * 8);
             }
         }
 
@@ -649,7 +652,7 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
         isRecursive = false;
 
         if (set.contains(getSpecificName())) {
-            set.remove(this.getSpecificName());
+            set.remove(getSpecificName());
 
             isRecursive = true;
         }
@@ -773,6 +776,10 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
         return variableCount;
     }
 
+    public int getCursorCount() {
+        return cursorCount;
+    }
+
     public boolean isLibraryRoutine() {
         return isLibraryRoutine;
     }
@@ -810,6 +817,7 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
             statement     = null;
             references    = null;
             variableCount = 0;
+            cursorCount   = 0;
             ranges        = RangeVariable.emptyArray;
         }
     }
@@ -832,6 +840,7 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
         statement                = routine.statement;
         references               = routine.references;
         variableCount            = routine.variableCount;
+        cursorCount              = routine.cursorCount;
         ranges                   = routine.ranges;
     }
 
@@ -902,7 +911,8 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
                         head.addChainedResult(r);
                     }
                 } else {
-                    Error.error(ErrorCode.X_46000, "ResultSet not native");
+                    throw Error.error(ErrorCode.X_46000,
+                                      "ResultSet not native");
                 }
             }
         }
@@ -918,8 +928,8 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
             returnValue = returnType.convertJavaToSQL(null, returnValue);
             result      = Result.newPSMResult(returnValue);
         } catch (Throwable t) {
-            result = Result.newErrorResult(
-                Error.error(t, ErrorCode.X_46000, getName().name), null);
+            result = Result.newErrorResult(Error.error(t, ErrorCode.X_46000,
+                    getName().name));
         }
 
         return result;
@@ -960,14 +970,14 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
                 result = Result.newPSMResult(returnValue);
             }
         } catch (InvocationTargetException e) {
-            result = Result.newErrorResult(
-                Error.error(e, ErrorCode.X_46000, getName().name), null);
+            result = Result.newErrorResult(Error.error(e, ErrorCode.X_46000,
+                    getName().name));
         } catch (IllegalAccessException e) {
-            result = Result.newErrorResult(
-                Error.error(e, ErrorCode.X_46000, getName().name), null);
+            result = Result.newErrorResult(Error.error(e, ErrorCode.X_46000,
+                    getName().name));
         } catch (Throwable e) {
-            result = Result.newErrorResult(
-                Error.error(e, ErrorCode.X_46000, getName().name), null);
+            result = Result.newErrorResult(Error.error(e, ErrorCode.X_46000,
+                    getName().name));
         }
 
         session.setCurrentSchemaHsqlName(oldSessionSchema);
@@ -995,6 +1005,13 @@ public class Routine implements SchemaObject, RangeGroup, Cloneable {
                         new Object[variableCount];
                 }
 
+                session.sessionContext.routineCursors = Result.emptyArray;
+
+                if (cursorCount > 0) {
+                    session.sessionContext.routineCursors =
+                        new Result[cursorCount];
+                }
+
                 result = statement.execute(session);
 
                 if (aggregateData != null) {
