@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,7 +55,7 @@ import org.hsqldb.types.Type;
  * Manages all SCHEMA related database objects
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.3.5
  * @since 1.8.0
  */
 public class SchemaManager {
@@ -101,6 +101,8 @@ public class SchemaManager {
             schema.typeLookup.add(TypeInvariants.CHARACTER_DATA, false);
             schema.typeLookup.add(TypeInvariants.SQL_IDENTIFIER, false);
             schema.typeLookup.add(TypeInvariants.TIME_STAMP, false);
+            schema.typeLookup.add(TypeInvariants.NCNAME, false);
+            schema.typeLookup.add(TypeInvariants.URI, false);
         } catch (HsqlException e) {}
     }
 
@@ -937,9 +939,8 @@ public class SchemaManager {
         if (!table.isView() && table.hasLobColumn()) {
             RowIterator it = table.rowIterator(session);
 
-            while (it.hasNext()) {
-                Row      row  = it.getNextRow();
-                Object[] data = row.getData();
+            while (it.next()) {
+                Object[] data = it.getCurrent();
 
                 session.sessionData.adjustLobUsageCount(table, data, -1);
             }
@@ -1991,7 +1992,7 @@ public class SchemaManager {
                     HsqlName tableName = name.parent;
                     Table table = (Table) schema.tableList.get(tableName.name);
 
-                    return table.getIndex(name.name);
+                    return table.getUserIndex(name.name);
 
                 case SchemaObject.REFERENCE :
                     return schema.referenceLookup.getObject(name.name);
