@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@ import org.hsqldb.map.ValuePool;
  * Collection of SQL schema objects of a specific type in a schema
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.4
  * @since 1.9.0
  */
 public class SchemaObjectSet {
@@ -71,6 +71,7 @@ public class SchemaObjectSet {
             case SchemaObject.SPECIFIC_ROUTINE :
             case SchemaObject.ASSERTION :
             case SchemaObject.TRIGGER :
+            case SchemaObject.REFERENCE :
                 map = new HashMappedList();
                 break;
 
@@ -79,6 +80,9 @@ public class SchemaObjectSet {
             case SchemaObject.INDEX :
                 map = new HashMap();
                 break;
+
+            default :
+                throw Error.runtimeError(ErrorCode.U_S0500, "SchemaObjectSet");
         }
     }
 
@@ -98,6 +102,7 @@ public class SchemaObjectSet {
             case SchemaObject.FUNCTION :
             case SchemaObject.ASSERTION :
             case SchemaObject.TRIGGER :
+            case SchemaObject.REFERENCE :
                 SchemaObject object = ((SchemaObject) map.get(name));
 
                 return object == null ? null
@@ -129,6 +134,7 @@ public class SchemaObjectSet {
             case SchemaObject.FUNCTION :
             case SchemaObject.ASSERTION :
             case SchemaObject.TRIGGER :
+            case SchemaObject.REFERENCE :
                 return (SchemaObject) map.get(name);
 
             default :
@@ -181,12 +187,12 @@ public class SchemaObjectSet {
         switch (name.type) {
 
             case SchemaObject.COLUMN :
-                value = name;
-                break;
-
             case SchemaObject.CONSTRAINT :
             case SchemaObject.INDEX :
                 value = name;
+                break;
+
+            default :
         }
 
         map.put(name.name, value);
@@ -239,7 +245,8 @@ public class SchemaObjectSet {
             case SchemaObject.TYPE :
             case SchemaObject.ASSERTION :
             case SchemaObject.TRIGGER :
-            case SchemaObject.ROUTINE : {
+            case SchemaObject.ROUTINE :
+            case SchemaObject.REFERENCE : {
                 int i = ((HashMappedList) map).getIndex(name.name);
 
                 if (i == -1) {
@@ -265,6 +272,7 @@ public class SchemaObjectSet {
 
                 break;
             }
+            default :
         }
     }
 
@@ -286,6 +294,7 @@ public class SchemaObjectSet {
             case SchemaObject.SPECIFIC_ROUTINE :
             case SchemaObject.ASSERTION :
             case SchemaObject.TRIGGER :
+            case SchemaObject.REFERENCE :
             case SchemaObject.COLUMN :
             case SchemaObject.CONSTRAINT :
             case SchemaObject.INDEX :
@@ -317,6 +326,7 @@ public class SchemaObjectSet {
             case SchemaObject.SPECIFIC_ROUTINE :
             case SchemaObject.ASSERTION :
             case SchemaObject.TRIGGER :
+            case SchemaObject.REFERENCE :
             case SchemaObject.COLUMN :
             case SchemaObject.CONSTRAINT :
             case SchemaObject.INDEX :
@@ -337,9 +347,6 @@ public class SchemaObjectSet {
             case SchemaObject.VIEW :
                 return Tokens.T_VIEW;
 
-            case SchemaObject.COLUMN :
-                return Tokens.T_COLUMN;
-
             case SchemaObject.TABLE :
                 return Tokens.T_TABLE;
 
@@ -355,9 +362,6 @@ public class SchemaObjectSet {
             case SchemaObject.TYPE :
                 return Tokens.T_TYPE;
 
-            case SchemaObject.CONSTRAINT :
-                return Tokens.T_CONSTRAINT;
-
             case SchemaObject.COLLATION :
                 return Tokens.T_COLLATION;
 
@@ -370,27 +374,36 @@ public class SchemaObjectSet {
             case SchemaObject.ASSERTION :
                 return Tokens.T_ASSERTION;
 
-            case SchemaObject.INDEX :
-                return Tokens.T_INDEX;
-
             case SchemaObject.TRIGGER :
                 return Tokens.T_TRIGGER;
 
+            case SchemaObject.REFERENCE :
+                return Tokens.T_SYNONYM;
+
+            case SchemaObject.COLUMN :
+                return Tokens.T_COLUMN;
+
+            case SchemaObject.CONSTRAINT :
+                return Tokens.T_CONSTRAINT;
+
+            case SchemaObject.INDEX :
+                return Tokens.T_INDEX;
+
             default :
                 throw Error.runtimeError(ErrorCode.U_S0500, "SchemaObjectSet");
         }
     }
 
-    String[] getSQL(OrderedHashSet resolved, OrderedHashSet unresolved) {
-
-        HsqlArrayList list = new HsqlArrayList();
+    void getSQL(HsqlArrayList list, OrderedHashSet resolved,
+                OrderedHashSet unresolved) {
 
+        // HashMap lists are not persisted with this method
         if (!(map instanceof HashMappedList)) {
-            return null;
+            return;
         }
 
         if (map.isEmpty()) {
-            return ValuePool.emptyStringArray;
+            return;
         }
 
         Iterator it = map.values().iterator();
@@ -416,12 +429,6 @@ public class SchemaObjectSet {
         }
 
         addAllSQL(resolved, unresolved, list, it, null);
-
-        String[] array = new String[list.size()];
-
-        list.toArray(array);
-
-        return array;
     }
 
     static void addAllSQL(OrderedHashSet resolved, OrderedHashSet unresolved,
@@ -488,6 +495,8 @@ public class SchemaObjectSet {
                         if (name.schema == null) {
                             continue;
                         }
+
+                    // fall through
                     case SchemaObject.TYPE :
                     case SchemaObject.DOMAIN :
                     case SchemaObject.FUNCTION :
@@ -496,6 +505,8 @@ public class SchemaObjectSet {
                         if (!resolved.contains(name)) {
                             isResolved = false;
                         }
+                        break;
+
                     default :
                 }
             }
@@ -528,12 +539,16 @@ public class SchemaObjectSet {
 
                     case SchemaObject.FUNCTION :
                     case SchemaObject.PROCEDURE :
-                        if (((Routine) object).isRecursive) {
+                        Routine routine = ((Routine) object);
+
+                        if (routine.isRecursive) {
                             list.add(((Routine) object).getSQLDeclaration());
                             list.add(((Routine) object).getSQLAlter());
-
-                            break;
+                        } else {
+                            list.add(object.getSQL());
                         }
+                        break;
+
                     default :
                         list.add(object.getSQL());
                 }
