@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -76,7 +76,7 @@ import org.hsqldb.types.Type.TypedComparator;
  * Implementation of SQL sessions.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.3.5
  * @since 1.7.0
  */
 public class Session implements SessionInterface {
@@ -895,7 +895,7 @@ public class Session implements SessionInterface {
         return database.getNameString();
     }
 
-// boucherb@users 20020810 metadata 1.7.2
+// campbell-burnet@users 20020810 metadata 1.7.2
 //----------------------------------------------------------------
     private final long connectTime = System.currentTimeMillis();
 
@@ -1340,6 +1340,12 @@ public class Session implements SessionInterface {
 
             database.txManager.beginAction(this, cs);
 
+            if (redoAction) {
+                redoAction = false;
+
+                continue;
+            }
+
             cs = sessionContext.currentStatement;
 
             if (cs == null) {
@@ -1377,6 +1383,7 @@ public class Session implements SessionInterface {
             }
 
             database.txManager.beginActionResume(this);
+            timeoutManager.resumeTimeout();
 
             //        tempActionHistory.add("sql execute " + cs.sql + " " + actionTimestamp + " " + rowActionList.size());
             sessionContext.setDynamicArguments(pvals);
@@ -1487,8 +1494,8 @@ public class Session implements SessionInterface {
 
         Result error = null;
 
-        while (nav.hasNext()) {
-            Object[] pvals = nav.getNext();
+        while (nav.next()) {
+            Object[] pvals = nav.getCurrent();
             Result in = executeCompiledStatement(cs, pvals, cmd.queryTimeout);
 
             // On the client side, iterate over the vals and throw
@@ -1499,8 +1506,8 @@ public class Session implements SessionInterface {
                     RowSetNavigator navgen =
                         in.getChainedResult().getNavigator();
 
-                    while (navgen.hasNext()) {
-                        Object[] generatedRow = navgen.getNext();
+                    while (navgen.next()) {
+                        Object[] generatedRow = navgen.getCurrent();
 
                         generatedResult.getNavigator().add(generatedRow);
                     }
@@ -1544,9 +1551,9 @@ public class Session implements SessionInterface {
 
         Result error = null;
 
-        while (nav.hasNext()) {
+        while (nav.next()) {
             Result   in;
-            Object[] data = nav.getNext();
+            Object[] data = nav.getCurrent();
             String   sql  = (String) data[0];
 
             try {
@@ -1980,7 +1987,7 @@ public class Session implements SessionInterface {
     public Result cancel(Result result) {
 
         if (result.getType() == ResultConstants.SQLCANCEL) {
-            if (result.getSessionRandomID()  == randomId) {
+            if (result.getSessionRandomID() == randomId) {
                 database.txManager.resetSession(
                     null, this, TransactionManager.resetSessionAbort);
             }
@@ -2318,6 +2325,10 @@ public class Session implements SessionInterface {
             database.timeoutRunner.addSession(Session.this);
         }
 
+        void resumeTimeout() {
+            actionTimestamp = Session.this.actionTimestamp;
+        }
+
         boolean endTimeout() {
 
             boolean aborted = this.aborted;
