@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,23 +55,23 @@ import org.hsqldb.types.Types;
  * This reduces temporary Object creation by SUM and AVG functions for
  * INTEGER and narrower types.
  *
- * @author Campbell Burnet (boucherb@users dot sourceforge.net)
+ * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.2.9
+ * @version 2.3.5
  * @since 1.7.2
  *
  */
 public class SetFunction {
 
-    private HashSet distinctValues;
-    private boolean isDistinct;
+    private HashSet       distinctValues;
+    private final boolean isDistinct;
 
     //
-    private int       setType;
-    private int       typeCode;
-    private Type      type;
-    private ArrayType arrayType;
-    private Type      returnType;
+    private final int       setType;
+    private final int       typeCode;
+    private final Type      type;
+    private final ArrayType arrayType;
+    private final Type      returnType;
 
     //
     private long count;
@@ -91,11 +91,11 @@ public class SetFunction {
         this.setType    = setType;
         this.type       = type;
         this.returnType = returnType;
+        this.isDistinct = isDistinct;
+        this.arrayType  = arrayType;
 
         if (isDistinct) {
-            this.isDistinct = true;
-            this.arrayType  = arrayType;
-            distinctValues  = new HashSet();
+            distinctValues = new HashSet();
 
             if (type.isRowType() || type.isArrayType()) {
                 TypedComparator comparator = Type.newComparator(session);
@@ -114,11 +114,15 @@ public class SetFunction {
             this.sample = true;
         }
 
-        if (type != null) {
-            typeCode = type.typeCode;
-
-            if (type.isIntervalType()) {
-                typeCode = Types.SQL_INTERVAL;
+        if (type == null) {
+            typeCode = 0;
+        } else {
+            if (type.isIntervalYearMonthType()) {
+                typeCode = Types.SQL_INTERVAL_MONTH;
+            } else if (type.isIntervalDaySecondType()) {
+                typeCode = Types.SQL_INTERVAL_SECOND;
+            } else {
+                typeCode = type.typeCode;
             }
         }
     }
@@ -153,7 +157,7 @@ public class SetFunction {
 
                         return;
 
-                    case Types.SQL_INTERVAL : {
+                    case Types.SQL_INTERVAL_SECOND : {
                         if (item instanceof IntervalSecondData) {
                             addLong(((IntervalSecondData) item).getSeconds());
 
@@ -167,7 +171,12 @@ public class SetFunction {
 
                                 currentLong %= DTIType.nanoScaleFactors[0];
                             }
-                        } else if (item instanceof IntervalMonthData) {
+                        }
+
+                        return;
+                    }
+                    case Types.SQL_INTERVAL_MONTH : {
+                        if (item instanceof IntervalMonthData) {
                             addLong(((IntervalMonthData) item).units);
                         }
 
@@ -318,7 +327,8 @@ public class SetFunction {
                     case Types.SQL_SMALLINT :
                     case Types.SQL_INTEGER :
                         if (returnType.scale != 0) {
-                            return returnType.divide(session, Long.valueOf(currentLong),
+                            return returnType.divide(session,
+                                                     Long.valueOf(currentLong),
                                                      Long.valueOf(count));
                         }
 
@@ -342,24 +352,30 @@ public class SetFunction {
                                 new BigDecimal(count), BigDecimal.ROUND_DOWN);
                         } else {
                             return returnType.divide(session,
-                                                     currentBigDecimal, Long.valueOf(count));
+                                                     currentBigDecimal,
+                                                     Long.valueOf(count));
                         }
-                    case Types.SQL_INTERVAL : {
-                        BigInteger bi =
-                            getLongSum().divide(BigInteger.valueOf(count));
+                    case Types.SQL_INTERVAL_SECOND :
+                    case Types.SQL_INTERVAL_MONTH : {
+                        BigInteger[] bi = getLongSum().divideAndRemainder(
+                            BigInteger.valueOf(count));
 
-                        if (NumberType.compareToLongLimits(bi) != 0) {
+                        if (NumberType.compareToLongLimits(bi[0]) != 0) {
                             throw Error.error(ErrorCode.X_22015);
                         }
 
-                        if (((IntervalType) type).isDaySecondIntervalType()) {
-                            return new IntervalSecondData(bi.longValue(),
-                                                          currentLong,
+                        if (((IntervalType) type).isIntervalDaySecondType()) {
+                            long nanos =
+                                (bi[1].longValue() * DTIType
+                                    .limitNanoseconds + currentLong) / count;
+
+                            return new IntervalSecondData(bi[0].longValue(),
+                                                          nanos,
                                                           (IntervalType) type,
                                                           true);
                         } else {
                             return IntervalMonthData.newIntervalMonth(
-                                bi.longValue(), (IntervalType) type);
+                                bi[0].longValue(), (IntervalType) type);
                         }
                     }
                     case Types.SQL_DATE :
@@ -401,14 +417,15 @@ public class SetFunction {
                     case Types.SQL_DECIMAL :
                         return currentBigDecimal;
 
-                    case Types.SQL_INTERVAL : {
+                    case Types.SQL_INTERVAL_SECOND :
+                    case Types.SQL_INTERVAL_MONTH : {
                         BigInteger bi = getLongSum();
 
                         if (NumberType.compareToLongLimits(bi) != 0) {
                             throw Error.error(ErrorCode.X_22015);
                         }
 
-                        if (((IntervalType) type).isDaySecondIntervalType()) {
+                        if (((IntervalType) type).isIntervalDaySecondType()) {
                             return new IntervalSecondData(bi.longValue(),
                                                           currentLong,
                                                           (IntervalType) type,
@@ -463,8 +480,13 @@ public class SetFunction {
             return Type.SQL_BIGINT;
         }
 
-        int typeCode = type.isIntervalType() ? Types.SQL_INTERVAL
-                                             : type.typeCode;
+        int typeCode = type.typeCode;
+
+        if (type.isIntervalYearMonthType()) {
+            typeCode = Types.SQL_INTERVAL_MONTH;
+        } else if (type.isIntervalDaySecondType()) {
+            typeCode = Types.SQL_INTERVAL_SECOND;
+        }
 
         switch (setType) {
 
@@ -492,7 +514,8 @@ public class SetFunction {
                     case Types.SQL_REAL :
                     case Types.SQL_FLOAT :
                     case Types.SQL_DOUBLE :
-                    case Types.SQL_INTERVAL :
+                    case Types.SQL_INTERVAL_MONTH :
+                    case Types.SQL_INTERVAL_SECOND :
                     case Types.SQL_DATE :
                     case Types.SQL_TIMESTAMP :
                     case Types.SQL_TIMESTAMP_WITH_TIME_ZONE :
@@ -523,7 +546,8 @@ public class SetFunction {
                         return Type.getType(type.typeCode, null, null,
                                             type.precision * 2, type.scale);
 
-                    case Types.SQL_INTERVAL :
+                    case Types.SQL_INTERVAL_MONTH :
+                    case Types.SQL_INTERVAL_SECOND :
                         return IntervalType.newIntervalType(
                             type.typeCode, DTIType.maxIntervalPrecision,
                             type.scale);
@@ -651,7 +675,7 @@ public class SetFunction {
         sk  += xi;
     }
 
-    private Number getVariance() {
+    private Double getVariance() {
 
         if (!initialized) {
             return null;
@@ -662,7 +686,7 @@ public class SetFunction {
                       : new Double(vk / (double) (n));
     }
 
-    private Number getStdDev() {
+    private Double getStdDev() {
 
         if (!initialized) {
             return null;
