@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,12 +54,12 @@ import org.hsqldb.scriptio.ScriptWriterText;
  * Implementation of Statement for SQL commands.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.5
  * @since 1.9.0
  */
 public class StatementCommand extends Statement {
 
-    Object[] parameters;
+    Object[] arguments;
 
     StatementCommand(int type, Object[] args) {
         this(type, args, null, null);
@@ -71,7 +71,7 @@ public class StatementCommand extends Statement {
         super(type);
 
         this.isTransactionStatement = true;
-        this.parameters             = args;
+        this.arguments              = args;
 
         if (readNames != null) {
             this.readTableNames = readNames;
@@ -100,7 +100,7 @@ public class StatementCommand extends Statement {
                 break;
 
             case StatementTypes.DATABASE_SCRIPT : {
-                String name = (String) parameters[0];
+                String name = (String) arguments[0];
 
                 if (name == null) {
                     statementReturnType = StatementTypes.RETURN_RESULT;
@@ -111,6 +111,13 @@ public class StatementCommand extends Statement {
 
                 break;
             }
+            case StatementTypes.CHECK_INDEX : {
+                statementReturnType = StatementTypes.RETURN_RESULT;
+                group = StatementTypes.X_HSQLDB_DATABASE_OPERATION;
+                isLogged            = false;
+
+                break;
+            }
             case StatementTypes.DATABASE_BACKUP :
                 group = StatementTypes.X_HSQLDB_DATABASE_OPERATION;
 
@@ -121,6 +128,10 @@ public class StatementCommand extends Statement {
                 isLogged = false;
                 break;
 
+            case StatementTypes.SET_DATABASE_TRANSACTION_CONTROL :
+                group = StatementTypes.X_HSQLDB_DATABASE_OPERATION;
+                break;
+
             case StatementTypes.SET_DATABASE_UNIQUE_NAME :
             case StatementTypes.SET_DATABASE_FILES_WRITE_DELAY :
             case StatementTypes.SET_DATABASE_FILES_TEMP_PATH :
@@ -129,7 +140,6 @@ public class StatementCommand extends Statement {
                 group                  = StatementTypes.X_HSQLDB_SETTING;
                 break;
 
-//
             case StatementTypes.SET_DATABASE_DEFAULT_INITIAL_SCHEMA :
             case StatementTypes.SET_DATABASE_DEFAULT_TABLE_TYPE :
             case StatementTypes.SET_DATABASE_FILES_CACHE_ROWS :
@@ -150,12 +160,9 @@ public class StatementCommand extends Statement {
             case StatementTypes.SET_DATABASE_RESULT_MEMORY_ROWS :
             case StatementTypes.SET_DATABASE_SQL_REFERENTIAL_INTEGRITY :
             case StatementTypes.SET_DATABASE_SQL :
-            case StatementTypes.SET_DATABASE_TRANSACTION_CONTROL :
             case StatementTypes.SET_DATABASE_DEFAULT_ISOLATION_LEVEL :
             case StatementTypes.SET_DATABASE_TRANSACTION_CONFLICT :
             case StatementTypes.SET_DATABASE_GC :
-
-//
             case StatementTypes.SET_DATABASE_SQL_COLLATION :
             case StatementTypes.SET_DATABASE_FILES_BACKUP_INCREMENT :
             case StatementTypes.SET_DATABASE_TEXT_SOURCE :
@@ -261,17 +268,17 @@ public class StatementCommand extends Statement {
                 return getTruncateResult(session);
             }
             case StatementTypes.EXPLAIN_PLAN : {
-                Statement statement = (Statement) parameters[0];
+                Statement statement = (Statement) arguments[0];
 
                 return Result.newSingleColumnStringResult("OPERATION",
                         statement.describe(session));
             }
             case StatementTypes.DATABASE_BACKUP : {
-                String  path       = (String) parameters[0];
-                boolean blocking   = ((Boolean) parameters[1]).booleanValue();
-                boolean script     = ((Boolean) parameters[2]).booleanValue();
-                boolean compressed = ((Boolean) parameters[3]).booleanValue();
-                boolean files      = ((Boolean) parameters[4]).booleanValue();
+                String  path       = (String) arguments[0];
+                boolean blocking   = ((Boolean) arguments[1]).booleanValue();
+                boolean script     = ((Boolean) arguments[2]).booleanValue();
+                boolean compressed = ((Boolean) arguments[3]).booleanValue();
+                boolean files      = ((Boolean) arguments[4]).booleanValue();
 
                 try {
                     session.checkAdmin();
@@ -297,7 +304,7 @@ public class StatementCommand extends Statement {
                 }
             }
             case StatementTypes.DATABASE_CHECKPOINT : {
-                boolean defrag = ((Boolean) parameters[0]).booleanValue();
+                boolean defrag = ((Boolean) arguments[0]).booleanValue();
 
                 try {
                     session.checkAdmin();
@@ -311,7 +318,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_BACKUP_INCREMENT : {
                 try {
-                    boolean mode = ((Boolean) parameters[0]).booleanValue();
+                    boolean mode = ((Boolean) arguments[0]).booleanValue();
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -324,8 +331,8 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_CACHE_ROWS : {
                 try {
-                    int     value = ((Integer) parameters[0]).intValue();
-                    boolean check = parameters[1] == null;
+                    int     value = ((Integer) arguments[0]).intValue();
+                    boolean check = arguments[1] == null;
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -346,8 +353,8 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_CACHE_SIZE : {
                 try {
-                    int     value = ((Integer) parameters[0]).intValue();
-                    boolean check = parameters[1] == null;
+                    int     value = ((Integer) arguments[0]).intValue();
+                    boolean check = arguments[1] == null;
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -368,8 +375,8 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_CHECK : {
                 try {
-                    long value1 = ((Long) parameters[0]).longValue();
-                    long value2 = ((Long) parameters[1]).longValue();
+                    long value1 = ((Long) arguments[0]).longValue();
+                    long value2 = ((Long) arguments[1]).longValue();
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -385,7 +392,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_LOBS_SCALE : {
                 try {
-                    int value = ((Integer) parameters[0]).intValue();
+                    int value = ((Integer) arguments[0]).intValue();
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -403,7 +410,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_LOBS_COMPRESSED : {
                 try {
-                    boolean mode = ((Boolean) parameters[0]).booleanValue();
+                    boolean mode = ((Boolean) arguments[0]).booleanValue();
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -422,7 +429,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_SCALE : {
                 try {
-                    int value = ((Integer) parameters[0]).intValue();
+                    int value = ((Integer) arguments[0]).intValue();
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -451,13 +458,13 @@ public class StatementCommand extends Statement {
                         return Result.updateZeroResult;
                     }
 
-                    if (parameters[0] instanceof Boolean) {
+                    if (arguments[0] instanceof Boolean) {
                         boolean value =
-                            ((Boolean) parameters[0]).booleanValue();
+                            ((Boolean) arguments[0]).booleanValue();
 
                         session.database.logger.setDataFileSpaces(value);
                     } else {
-                        int value = ((Integer) parameters[0]).intValue();
+                        int value = ((Integer) arguments[0]).intValue();
 
                         session.database.logger.setDataFileSpaces(value);
                     }
@@ -469,7 +476,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_DEFRAG : {
                 try {
-                    int value = ((Integer) parameters[0]).intValue();
+                    int value = ((Integer) arguments[0]).intValue();
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -489,8 +496,8 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_EVENT_LOG : {
                 try {
-                    int     value = ((Integer) parameters[0]).intValue();
-                    boolean isSql = ((Boolean) parameters[1]).booleanValue();
+                    int     value = ((Integer) arguments[0]).intValue();
+                    boolean isSql = ((Boolean) arguments[1]).booleanValue();
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -506,15 +513,15 @@ public class StatementCommand extends Statement {
                     session.checkAdmin();
                     session.checkDDLWrite();
 
-                    Object v = parameters[0];
+                    Object v = arguments[0];
 
                     if (v instanceof Boolean) {
                         boolean value =
-                            ((Boolean) parameters[0]).booleanValue();
+                            ((Boolean) arguments[0]).booleanValue();
 
                         session.database.logger.setNioDataFile(value);
                     } else {
-                        int value = ((Integer) parameters[0]).intValue();
+                        int value = ((Integer) arguments[0]).intValue();
 
                         session.database.logger.setNioMaxSize(value);
                     }
@@ -526,7 +533,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_LOG : {
                 try {
-                    boolean value = ((Boolean) parameters[0]).booleanValue();
+                    boolean value = ((Boolean) arguments[0]).booleanValue();
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -539,7 +546,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_LOG_SIZE : {
                 try {
-                    int value = ((Integer) parameters[0]).intValue();
+                    int value = ((Integer) arguments[0]).intValue();
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -552,7 +559,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_TEMP_PATH : {
                 try {
-                    String value = (String) parameters[0];
+                    String value = (String) arguments[0];
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -565,7 +572,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_SCRIPT_FORMAT : {
                 try {
-                    int value = ((Integer) parameters[0]).intValue();
+                    int value = ((Integer) arguments[0]).intValue();
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -578,7 +585,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_FILES_WRITE_DELAY : {
                 try {
-                    int value = ((Integer) parameters[0]).intValue();
+                    int value = ((Integer) arguments[0]).intValue();
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -591,7 +598,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_AUTHENTICATION : {
                 try {
-                    Routine routine = (Routine) parameters[0];
+                    Routine routine = (Routine) arguments[0];
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -605,7 +612,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_PASSWORD_CHECK : {
                 try {
-                    Routine routine = (Routine) parameters[0];
+                    Routine routine = (Routine) arguments[0];
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -619,7 +626,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_PASSWORD_DIGEST : {
                 try {
-                    String algo = (String) parameters[0];
+                    String algo = (String) arguments[0];
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -637,9 +644,9 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_SQL_COLLATION : {
                 try {
-                    String name = (String) parameters[0];
+                    String name = (String) arguments[0];
                     boolean padSpaces =
-                        ((Boolean) parameters[1]).booleanValue();
+                        ((Boolean) arguments[1]).booleanValue();
 
                     /** @todo 1.9.0 - ensure no data in character columns */
                     session.checkAdmin();
@@ -653,7 +660,7 @@ public class StatementCommand extends Statement {
                 }
             }
             case StatementTypes.SET_DATABASE_SQL_REFERENTIAL_INTEGRITY : {
-                boolean mode = ((Boolean) parameters[0]).booleanValue();
+                boolean mode = ((Boolean) arguments[0]).booleanValue();
 
                 session.checkAdmin();
                 session.checkDDLWrite();
@@ -662,15 +669,18 @@ public class StatementCommand extends Statement {
                 return Result.updateZeroResult;
             }
             case StatementTypes.SET_DATABASE_SQL : {
-                String  property = (String) parameters[0];
-                boolean mode     = ((Boolean) parameters[1]).booleanValue();
-                int     value    = ((Number) parameters[2]).intValue();
+                String  property = (String) arguments[0];
+                boolean mode     = ((Boolean) arguments[1]).booleanValue();
+                int     value    = ((Number) arguments[2]).intValue();
 
                 session.checkAdmin();
                 session.checkDDLWrite();
 
                 if (HsqlDatabaseProperties.sql_live_object.equals(property)) {
                     session.database.setLiveObject(mode);
+                } else if (HsqlDatabaseProperties.sql_restrict_exec.equals(
+                        property)) {
+                    session.database.setRestrictExec(mode);
                 } else if (HsqlDatabaseProperties.sql_enforce_names.equals(
                         property)) {
                     session.database.setStrictNames(mode);
@@ -741,12 +751,15 @@ public class StatementCommand extends Statement {
                 } else if (HsqlDatabaseProperties.sql_syntax_pgs.equals(
                         property)) {
                     session.database.setSyntaxPgs(mode);
+                } else if (HsqlDatabaseProperties.sql_sys_index_names.equals(
+                        property)) {
+                    session.database.setSysIndexNames(mode);
                 }
 
                 return Result.updateZeroResult;
             }
             case StatementTypes.SET_DATABASE_DEFAULT_INITIAL_SCHEMA : {
-                HsqlName schema = (HsqlName) parameters[0];
+                HsqlName schema = (HsqlName) arguments[0];
 
                 session.checkAdmin();
                 session.checkDDLWrite();
@@ -760,7 +773,7 @@ public class StatementCommand extends Statement {
                 return Result.updateZeroResult;
             }
             case StatementTypes.SET_DATABASE_DEFAULT_TABLE_TYPE : {
-                Integer type = (Integer) parameters[0];
+                Integer type = (Integer) arguments[0];
 
                 session.checkAdmin();
                 session.checkDDLWrite();
@@ -774,7 +787,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_TRANSACTION_CONTROL : {
                 try {
-                    int mode = ((Integer) parameters[0]).intValue();
+                    int mode = ((Integer) arguments[0]).intValue();
 
                     session.checkAdmin();
                     session.database.txManager.setTransactionControl(session,
@@ -787,7 +800,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_DEFAULT_ISOLATION_LEVEL : {
                 try {
-                    int mode = ((Integer) parameters[0]).intValue();
+                    int mode = ((Integer) arguments[0]).intValue();
 
                     session.checkAdmin();
 
@@ -800,7 +813,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_TRANSACTION_CONFLICT : {
                 try {
-                    boolean mode = ((Boolean) parameters[0]).booleanValue();
+                    boolean mode = ((Boolean) arguments[0]).booleanValue();
 
                     session.checkAdmin();
 
@@ -813,7 +826,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_GC : {
                 try {
-                    int count = ((Integer) parameters[0]).intValue();
+                    int count = ((Integer) arguments[0]).intValue();
 
                     session.checkAdmin();
 
@@ -826,8 +839,8 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_PROPERTY : {
                 try {
-                    String property = (String) parameters[0];
-                    Object value    = parameters[1];
+                    String property = (String) arguments[0];
+                    Object value    = arguments[1];
 
                     session.checkAdmin();
                     session.checkDDLWrite();
@@ -839,7 +852,7 @@ public class StatementCommand extends Statement {
                 }
             }
             case StatementTypes.SET_DATABASE_RESULT_MEMORY_ROWS : {
-                int size = ((Integer) parameters[0]).intValue();
+                int size = ((Integer) arguments[0]).intValue();
 
                 session.checkAdmin();
                 session.database.setResultMaxMemoryRows(size);
@@ -848,7 +861,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_TEXT_SOURCE : {
                 try {
-                    String         source = (String) parameters[0];
+                    String         source = (String) arguments[0];
                     HsqlProperties props  = null;
 
                     session.checkAdmin();
@@ -873,7 +886,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_DATABASE_UNIQUE_NAME : {
                 try {
-                    String name = (String) parameters[0];
+                    String name = (String) arguments[0];
 
                     session.checkAdmin();
                     session.database.setDatabaseName(name);
@@ -885,7 +898,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.DATABASE_SCRIPT : {
                 ScriptWriterText dsw  = null;
-                String           name = (String) parameters[0];
+                String           name = (String) arguments[0];
 
                 try {
                     session.checkAdmin();
@@ -907,7 +920,7 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.DATABASE_SHUTDOWN : {
                 try {
-                    int mode = ((Integer) parameters[0]).intValue();
+                    int mode = ((Integer) arguments[0]).intValue();
 
                     session.checkAdmin();
                     session.database.close(mode);
@@ -919,19 +932,18 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.CHECK_INDEX : {
                 try {
-                    HsqlName name = (HsqlName) parameters[0];
-                    Table table =
-                        session.database.schemaManager.getUserTable(name.name,
-                            name.schema.name);
+                    int type = ((Integer) arguments[1]).intValue();
+                    Result result = Result.newDoubleColumnResult("TABLE_NAME",
+                        "INFO");
 
-                    return Result.updateZeroResult;
+                    return result;
                 } catch (HsqlException e) {
                     return Result.newErrorResult(e, sql);
                 }
             }
             case StatementTypes.SET_TABLE_NEW_TABLESPACE : {
                 try {
-                    HsqlName name = (HsqlName) parameters[0];
+                    HsqlName name = (HsqlName) arguments[0];
                     Table table =
                         session.database.schemaManager.getUserTable(name.name,
                             name.schema.name);
@@ -983,8 +995,8 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_TABLE_SET_TABLESPACE : {
                 try {
-                    HsqlName name    = (HsqlName) parameters[0];
-                    int      spaceid = ((Integer) parameters[1]).intValue();
+                    HsqlName name    = (HsqlName) arguments[0];
+                    int      spaceid = ((Integer) arguments[1]).intValue();
                     Table table =
                         session.database.schemaManager.getUserTable(name.name,
                             name.schema.name);
@@ -1027,8 +1039,8 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_TABLE_CLUSTERED : {
                 try {
-                    HsqlName name     = (HsqlName) parameters[0];
-                    int[]    colIndex = (int[]) parameters[1];
+                    HsqlName name     = (HsqlName) arguments[0];
+                    int[]    colIndex = (int[]) arguments[1];
                     Table table =
                         session.database.schemaManager.getUserTable(name.name,
                             name.schema.name);
@@ -1059,9 +1071,9 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_TABLE_INDEX : {
                 try {
-                    HsqlName name  = (HsqlName) parameters[0];
-                    String   value = (String) parameters[1];
-                    Integer  type  = (Integer) parameters[2];
+                    HsqlName name  = (HsqlName) arguments[0];
+                    String   value = (String) arguments[1];
+                    Integer  type  = (Integer) arguments[2];
                     Table table =
                         session.database.schemaManager.getUserTable(name.name,
                             name.schema.name);
@@ -1077,11 +1089,11 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_TABLE_READONLY : {
                 try {
-                    HsqlName name = (HsqlName) parameters[0];
+                    HsqlName name = (HsqlName) arguments[0];
                     Table table =
                         session.database.schemaManager.getUserTable(name.name,
                             name.schema.name);
-                    boolean mode = ((Boolean) parameters[1]).booleanValue();
+                    boolean mode = ((Boolean) arguments[1]).booleanValue();
 
                     StatementSchema.checkSchemaUpdateAuthorisation(session,
                             table.getSchemaName());
@@ -1096,7 +1108,7 @@ public class StatementCommand extends Statement {
             case StatementTypes.SET_TABLE_SOURCE :
             case StatementTypes.SET_TABLE_SOURCE_HEADER : {
                 try {
-                    HsqlName name = (HsqlName) parameters[0];
+                    HsqlName name = (HsqlName) arguments[0];
                     Table table =
                         session.database.schemaManager.getUserTable(name.name,
                             name.schema.name);
@@ -1110,9 +1122,8 @@ public class StatementCommand extends Statement {
                         return Result.newErrorResult(e, sql);
                     }
 
-                    if (parameters[1] != null) {
-                        boolean mode =
-                            ((Boolean) parameters[1]).booleanValue();
+                    if (arguments[1] != null) {
+                        boolean mode = ((Boolean) arguments[1]).booleanValue();
 
                         if (mode) {
                             ((TextTable) table).connect(session);
@@ -1126,10 +1137,9 @@ public class StatementCommand extends Statement {
                         return Result.updateZeroResult;
                     }
 
-                    String  source = (String) parameters[2];
-                    boolean isDesc = ((Boolean) parameters[3]).booleanValue();
-                    boolean isHeader =
-                        ((Boolean) parameters[4]).booleanValue();
+                    String  source   = (String) arguments[2];
+                    boolean isDesc   = ((Boolean) arguments[3]).booleanValue();
+                    boolean isHeader = ((Boolean) arguments[4]).booleanValue();
 
                     if (isHeader) {
                         ((TextTable) table).setHeader(source);
@@ -1159,8 +1169,8 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_TABLE_TYPE : {
                 try {
-                    HsqlName name = (HsqlName) parameters[0];
-                    int      type = ((Integer) parameters[1]).intValue();
+                    HsqlName name = (HsqlName) arguments[0];
+                    int      type = ((Integer) arguments[1]).intValue();
 
                     //
                     Table table =
@@ -1193,8 +1203,8 @@ public class StatementCommand extends Statement {
                 }
             }
             case StatementTypes.SET_USER_LOCAL : {
-                User    user = (User) parameters[0];
-                boolean mode = ((Boolean) parameters[1]).booleanValue();
+                User    user = (User) arguments[0];
+                boolean mode = ((Boolean) arguments[1]).booleanValue();
 
                 session.checkAdmin();
                 session.checkDDLWrite();
@@ -1207,8 +1217,8 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_USER_INITIAL_SCHEMA : {
                 try {
-                    User     user   = (User) parameters[0];
-                    HsqlName schema = (HsqlName) parameters[1];
+                    User     user   = (User) arguments[0];
+                    HsqlName schema = (HsqlName) arguments[1];
 
                     session.checkDDLWrite();
 
@@ -1240,10 +1250,10 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.SET_USER_PASSWORD : {
                 try {
-                    User    user = parameters[0] == null ? session.getUser()
-                                                         : (User) parameters[0];
-                    String  password = (String) parameters[1];
-                    boolean isDigest = (Boolean) parameters[2];
+                    User    user = arguments[0] == null ? session.getUser()
+                                                        : (User) arguments[0];
+                    String  password = (String) arguments[1];
+                    boolean isDigest = (Boolean) arguments[2];
 
                     session.checkDDLWrite();
                     session.database.userManager.setPassword(session, user,
@@ -1256,8 +1266,8 @@ public class StatementCommand extends Statement {
             }
             case StatementTypes.ALTER_SESSION : {
                 try {
-                    long sessionID = ((Number) parameters[0]).longValue();
-                    int  action    = ((Number) parameters[1]).intValue();
+                    long sessionID = ((Number) arguments[0]).longValue();
+                    int  action    = ((Number) arguments[1]).intValue();
                     Session targetSession =
                         session.database.sessionManager.getSession(sessionID);
 
@@ -1302,7 +1312,6 @@ public class StatementCommand extends Statement {
                                     targetSession,
                                     TransactionManager.resetSessionAbort);
                             break;
-
                     }
                 } catch (HsqlException e) {
                     return Result.newErrorResult(e, sql);
@@ -1319,9 +1328,9 @@ public class StatementCommand extends Statement {
     Result getTruncateResult(Session session) {
 
         try {
-            HsqlName name            = (HsqlName) parameters[0];
-            boolean  restartIdentity = (Boolean) parameters[1];
-            boolean  noCheck         = (Boolean) parameters[2];
+            HsqlName name            = (HsqlName) arguments[0];
+            boolean  restartIdentity = (Boolean) arguments[1];
+            boolean  noCheck         = (Boolean) arguments[2];
             Table[]  tables;
 
             if (name.type == SchemaObject.TABLE) {
@@ -1424,7 +1433,8 @@ public class StatementCommand extends Statement {
 
             case StatementTypes.DATABASE_SCRIPT :
                 if (statementReturnType == StatementTypes.RETURN_RESULT) {
-                    return ResultMetaData.newSingleColumnMetaData("COMMANDS");
+                    return ResultMetaData.newSingleColumnMetaData(
+                        "STATEMENTS");
                 }
 
             // fall through
