@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -496,9 +496,7 @@ public class StatementCompound extends Statement implements RangeGroup {
 
         Result result = Result.updateZeroResult;
 
-        while (queryResult.navigator.hasNext()) {
-            queryResult.navigator.next();
-
+        while (queryResult.navigator.next()) {
             Object[] data = queryResult.navigator.getCurrent();
 
             initialiseVariables(session, data,
@@ -748,6 +746,11 @@ public class StatementCompound extends Statement implements RangeGroup {
             }
         }
 
+        if (loopCursor != null) {
+            set.addAll(loopCursor.getReferences());
+            readTableNamesSet.addAll(loopCursor.getTableNamesForRead());
+        }
+
         if (condition != null) {
             set.addAll(condition.getReferences());
             readTableNamesSet.addAll(condition.getTableNamesForRead());
