@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,7 +54,7 @@ import org.hsqldb.types.Types;
  * Implementation of Statement for DML statements.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.3.5
  * @since 1.9.0
  */
 
@@ -76,7 +76,7 @@ public class StatementDML extends StatementDMQL {
     ResultMetaData generatedInputMetaData;
 
     //
-    int limit = Integer.MAX_VALUE;
+    SortAndSlice sortAndSlice;
 
     /** column indexes for generated values */
     int[] generatedIndexes;
@@ -105,13 +105,7 @@ public class StatementDML extends StatementDMQL {
                                                             .getBaseTable();
         this.targetRangeVariables = rangeVars;
         this.restartIdentity      = restartIdentity;
-
-        if (sortAndSlice != null) {
-            int[] limits = sortAndSlice.getLimits(session, null,
-                                                  Integer.MAX_VALUE);
-
-            limit = limits[1];
-        }
+        this.sortAndSlice         = sortAndSlice;
 
         setDatabaseObjects(session, compileContext);
         checkAccessRights(session);
@@ -144,13 +138,7 @@ public class StatementDML extends StatementDMQL {
         this.updateExpressions    = colExpressions;
         this.updateCheckColumns   = checkColumns;
         this.targetRangeVariables = rangeVars;
-
-        if (sortAndSlice != null) {
-            int[] limits = sortAndSlice.getLimits(session, null,
-                                                  Integer.MAX_VALUE);
-
-            limit = limits[1];
-        }
+        this.sortAndSlice         = sortAndSlice;
 
         setupChecks();
         setDatabaseObjects(session, compileContext);
@@ -220,11 +208,19 @@ public class StatementDML extends StatementDMQL {
     Result getResult(Session session) {
 
         Result result = null;
+        int    limit  = Integer.MAX_VALUE;
+
+        if (sortAndSlice != null) {
+            int[] limits = sortAndSlice.getLimits(session, null,
+                                                  Integer.MAX_VALUE);
+
+            limit = limits[1];
+        }
 
         switch (type) {
 
             case StatementTypes.UPDATE_WHERE :
-                result = executeUpdateStatement(session);
+                result = executeUpdateStatement(session, limit);
                 break;
 
             case StatementTypes.MERGE :
@@ -235,7 +231,7 @@ public class StatementDML extends StatementDMQL {
                 if (isTruncate) {
                     result = executeDeleteTruncateStatement(session);
                 } else {
-                    result = executeDeleteStatement(session);
+                    result = executeDeleteStatement(session, limit);
                 }
                 break;
 
@@ -417,11 +413,18 @@ public class StatementDML extends StatementDMQL {
             case ResultConstants.RETURN_GENERATED_KEYS_COL_NAMES :
                 String[] columnNames = meta.getGeneratedColumnNames();
 
-                generatedIndexes = baseTable.getColumnIndexes(columnNames);
+                generatedIndexes = baseTable.findColumnIndexes(columnNames);
 
                 for (int i = 0; i < generatedIndexes.length; i++) {
                     if (generatedIndexes[i] < 0) {
-                        throw Error.error(ErrorCode.X_42501, columnNames[0]);
+                        String upperName = columnNames[i].toUpperCase();
+
+                        generatedIndexes[i] = baseTable.findColumn(upperName);
+
+                        if (generatedIndexes[i] < 0) {
+                            throw Error.error(ErrorCode.X_42501,
+                                              columnNames[0]);
+                        }
                     }
                 }
                 break;
@@ -517,7 +520,7 @@ public class StatementDML extends StatementDMQL {
      *
      * @return Result object
      */
-    Result executeUpdateStatement(Session session) {
+    Result executeUpdateStatement(Session session, int limit) {
 
         int count = 0;
         RowSetNavigatorDataChange rowset =
@@ -548,18 +551,6 @@ public class StatementDML extends StatementDMQL {
                                               updateExpressions, colTypes,
                                               data);
 
-            if (updatableTableCheck != null) {
-                it.setCurrent(newData);
-
-                boolean check = updatableTableCheck.testCondition(session);
-
-                if (!check) {
-                    it.release();
-
-                    throw Error.error(ErrorCode.X_44000);
-                }
-            }
-
             rowset.addRow(session, row, newData, colTypes, updateColumnMap);
 
             session.sessionContext.rownum++;
@@ -882,14 +873,15 @@ public class StatementDML extends StatementDMQL {
         RangeIterator   checkIterator = null;
 
         if (updatableTableCheck != null) {
-            checkIterator = checkRangeVariable.getIterator(session);
+            checkIterator =
+                session.sessionContext.getCheckIterator(checkRangeVariable);
         }
 
         newData.beforeFirst();
 
         if (baseTable.identityColumn != -1) {
-            while (newData.hasNext()) {
-                Object[] data = newData.getNext();
+            while (newData.next()) {
+                Object[] data = newData.getCurrent();
 
                 session.sessionData.startRowProcessing();
                 baseTable.setIdentityColumn(session, data);
@@ -899,8 +891,8 @@ public class StatementDML extends StatementDMQL {
         }
 
         if (baseTable.triggerLists[Trigger.INSERT_BEFORE_ROW].length > 0) {
-            while (newData.hasNext()) {
-                Object[] data = newData.getNext();
+            while (newData.next()) {
+                Object[] data = newData.getCurrent();
 
                 session.sessionData.startRowProcessing();
                 baseTable.fireTriggers(session, Trigger.INSERT_BEFORE_ROW,
@@ -910,14 +902,14 @@ public class StatementDML extends StatementDMQL {
             newData.beforeFirst();
         }
 
-        while (newData.hasNext()) {
-            Object[] data = newData.getNext();
+        while (newData.next()) {
+            Object[] data = newData.getCurrent();
 
             // for identity using global sequence
             session.sessionData.startRowProcessing();
             baseTable.insertSingleRow(session, store, data, null);
 
-            if (checkIterator != null) {
+            if (updatableTableCheck != null) {
                 checkIterator.setCurrent(data);
 
                 boolean check = updatableTableCheck.testCondition(session);
@@ -936,8 +928,8 @@ public class StatementDML extends StatementDMQL {
 
         newData.beforeFirst();
 
-        while (newData.hasNext()) {
-            Object[] data = newData.getNext();
+        while (newData.next()) {
+            Object[] data = newData.getCurrent();
 
             performIntegrityChecks(session, baseTable, null, data, null);
         }
@@ -945,8 +937,8 @@ public class StatementDML extends StatementDMQL {
         newData.beforeFirst();
 
         if (baseTable.triggerLists[Trigger.INSERT_AFTER_ROW].length > 0) {
-            while (newData.hasNext()) {
-                Object[] data = newData.getNext();
+            while (newData.next()) {
+                Object[] data = newData.getCurrent();
 
                 baseTable.fireTriggers(session, Trigger.INSERT_AFTER_ROW,
                                        null, data, null);
@@ -970,14 +962,6 @@ public class StatementDML extends StatementDMQL {
         baseTable.insertSingleRow(session, store, data, null);
         performIntegrityChecks(session, baseTable, null, data, null);
 
-        if (session.database.isReferentialIntegrity()) {
-            for (int i = 0, size = baseTable.fkConstraints.length; i < size;
-                    i++) {
-                baseTable.fkConstraints[i].checkInsert(session, baseTable,
-                                                       data, true);
-            }
-        }
-
         if (baseTable.triggerLists[Trigger.INSERT_AFTER_ROW].length > 0) {
             baseTable.fireTriggers(session, Trigger.INSERT_AFTER_ROW, null,
                                    data, null);
@@ -1079,8 +1063,14 @@ public class StatementDML extends StatementDMQL {
                RowSetNavigatorDataChange navigator,
                RowSetNavigator generatedNavigator) {
 
-        int     rowCount          = navigator.getSize();
-        boolean autoUpdatedColumn = table.hasUpdatedColumn(updateColumnMap);
+        int           rowCount      = navigator.getSize();
+        boolean autoUpdatedColumn   = table.hasUpdatedColumn(updateColumnMap);
+        RangeIterator checkIterator = null;
+
+        if (updatableTableCheck != null) {
+            checkIterator =
+                session.sessionContext.getCheckIterator(checkRangeVariable);
+        }
 
         // set identity column where null and check columns
         for (int i = 0; i < rowCount; i++) {
@@ -1140,6 +1130,17 @@ public class StatementDML extends StatementDMQL {
                                           row.getData(), data, changedColumns);
                 currentTable.enforceRowConstraints(session, data);
             }
+
+            // check the view condition after all the triggered changed
+            if (updatableTableCheck != null) {
+                checkIterator.setCurrent(data);
+
+                boolean check = updatableTableCheck.testCondition(session);
+
+                if (!check) {
+                    throw Error.error(ErrorCode.X_44000);
+                }
+            }
         }
 
         if (table.isView) {
@@ -1247,7 +1248,7 @@ public class StatementDML extends StatementDMQL {
      *
      * @return the result of executing the statement
      */
-    Result executeDeleteStatement(Session session) {
+    Result executeDeleteStatement(Session session, int limit) {
 
         int count = 0;
         RangeIterator it = RangeVariable.getIterator(session,
@@ -1294,7 +1295,7 @@ public class StatementDML extends StatementDMQL {
 
         PersistentStore store   = targetTable.getRowStore(session);
         RowIterator     it = targetTable.getPrimaryIndex().firstRow(store);
-        boolean         hasData = it.hasNext();
+        boolean         hasData = false;
 
         for (int i = 0; i < targetTable.fkMainConstraints.length; i++) {
             if (targetTable.fkMainConstraints[i].getRef() != targetTable) {
@@ -1311,11 +1312,13 @@ public class StatementDML extends StatementDMQL {
         }
 
         try {
-            while (it.hasNext()) {
-                Row row = it.getNextRow();
+            while (it.next()) {
+                Row row = it.getCurrentRow();
 
                 session.addDeleteAction((Table) row.getTable(), store, row,
                                         null);
+
+                hasData = true;
             }
 
             if (restartIdentity && targetTable.identitySequence != null) {
@@ -1601,14 +1604,8 @@ public class StatementDML extends StatementDMQL {
 
             RowIterator refiterator = c.findFkRef(session, row.getData());
 
-            if (!refiterator.hasNext()) {
-                refiterator.release();
-
-                continue;
-            }
-
-            while (refiterator.hasNext()) {
-                Row      refRow  = refiterator.getNextRow();
+            while (refiterator.next()) {
+                Row      refRow  = refiterator.getCurrentRow();
                 Object[] refData = null;
 
                 /** @todo use MATCH */
