@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -46,22 +46,19 @@ import org.hsqldb.rights.Grantee;
 /**
  * Statement implementation for DML and base DQL statements.
  *
- * @author Campbell Burnet (boucherb@users dot sourceforge.net)
+ * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.3.5
  * @since 1.7.2
  */
 
 // fredt@users 20040404 - patch 1.7.2 - fixed type resolution for parameters
-// boucherb@users 200404xx - patch 1.7.2 - changed parameter naming scheme for SQLCI client usability/support
+// campbell-burnet@users 200404xx - patch 1.7.2 - changed parameter naming scheme for SQLCI client usability/support
 // fredt@users 20050609 - 1.8.0 - fixed EXPLAIN PLAN by implementing describe(Session)
 // fredt@users - 1.9.0 - support for generated column reporting
 // fredt@users - 1.9.0 - support for multi-row inserts
 public abstract class StatementDMQL extends Statement {
 
-    public static final String PCOL_PREFIX        = "@p";
-    static final String        RETURN_COLUMN_NAME = "@p0";
-
     /** target table for INSERT_XXX, UPDATE and DELETE and MERGE */
     Table targetTable;
     Table baseTable;
@@ -119,18 +116,6 @@ public abstract class StatementDMQL extends Statement {
      */
     HsqlName cursorName;
 
-    /**
-     * Parse-order array of Expression objects, all of type PARAMETER ,
-     * involved in some way in any INSERT_XXX, UPDATE, DELETE, SELECT or
-     * CALL CompiledStatement
-     */
-    ExpressionColumn[] parameters;
-
-    /**
-     * ResultMetaData for parameters
-     */
-    ResultMetaData parameterMetaData;
-
     /**
      * Subqueries inverse usage depth order
      */
@@ -510,71 +495,6 @@ public abstract class StatementDMQL extends Statement {
         return parameterMetaData;
     }
 
-    void setParameterMetaData() {
-
-        int     offset;
-        int     idx;
-        boolean hasReturnValue;
-
-        offset = 0;
-
-        if (parameters.length == 0) {
-            parameterMetaData = ResultMetaData.emptyParamMetaData;
-
-            return;
-        }
-
-// NO:  Not yet
-//        hasReturnValue = (type == CALL && !expression.isProcedureCall());
-//
-//        if (hasReturnValue) {
-//            outlen++;
-//            offset = 1;
-//        }
-        parameterMetaData =
-            ResultMetaData.newParameterMetaData(parameters.length);
-
-// NO: Not yet
-//        if (hasReturnValue) {
-//            e = expression;
-//            out.sName[0]       = DIProcedureInfo.RETURN_COLUMN_NAME;
-//            out.sClassName[0]  = e.getValueClassName();
-//            out.colType[0]     = e.getDataType();
-//            out.colSize[0]     = e.getColumnSize();
-//            out.colScale[0]    = e.getColumnScale();
-//            out.nullability[0] = e.nullability;
-//            out.isIdentity[0]  = false;
-//            out.paramMode[0]   = expression.PARAM_OUT;
-//        }
-        for (int i = 0; i < parameters.length; i++) {
-            idx = i + offset;
-
-            // always i + 1.  We currently use the convention of @p0 to name the
-            // return value OUT parameter
-            parameterMetaData.columnLabels[idx] = StatementDMQL.PCOL_PREFIX
-                                                  + (i + 1);
-            parameterMetaData.columnTypes[idx] = parameters[i].dataType;
-
-            if (parameters[i].dataType == null) {
-                throw Error.error(ErrorCode.X_42567);
-            }
-
-            byte parameterMode = SchemaObject.ParameterModes.PARAM_IN;
-
-            if (parameters[i].column != null
-                    && parameters[i].column.getParameterMode()
-                       != SchemaObject.ParameterModes.PARAM_UNKNOWN) {
-                parameterMode = parameters[i].column.getParameterMode();
-            }
-
-            parameterMetaData.paramModes[idx] = parameterMode;
-            parameterMetaData.paramNullable[idx] =
-                parameters[i].column == null
-                ? SchemaObject.Nullability.NULLABLE
-                : parameters[i].column.getNullability();
-        }
-    }
-
     /**
      * Retrieves a String representation of this object.
      */
@@ -807,7 +727,7 @@ public abstract class StatementDMQL extends Statement {
 
     public void resolve(Session session) {}
 
-    public final boolean isCatalogLock() {
+    public final boolean isCatalogLock(int model) {
         return false;
     }
 
