@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -32,7 +32,6 @@
 package org.hsqldb;
 
 import org.hsqldb.HsqlNameManager.HsqlName;
-import org.hsqldb.RangeVariable.RangeIteratorBase;
 import org.hsqldb.error.Error;
 import org.hsqldb.error.ErrorCode;
 import org.hsqldb.index.Index;
@@ -45,6 +44,7 @@ import org.hsqldb.lib.OrderedIntHashSet;
 import org.hsqldb.lib.Set;
 import org.hsqldb.lib.StringUtil;
 import org.hsqldb.map.ValuePool;
+import org.hsqldb.navigator.RangeIterator;
 import org.hsqldb.navigator.RowIterator;
 import org.hsqldb.navigator.RowSetNavigator;
 import org.hsqldb.navigator.RowSetNavigatorDataChange;
@@ -62,7 +62,7 @@ import org.hsqldb.types.Type;
  * Holds the data structures and methods for creation of a named database table.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.3.5
  * @since 1.6.1
  */
 public class Table extends TableBase implements SchemaObject {
@@ -192,6 +192,7 @@ public class Table extends TableBase implements SchemaObject {
                 isView           = true;
                 break;
 
+            case MODULE_TABLE :
             case RESULT_TABLE :
                 persistenceScope = SCOPE_SESSION;
                 isSessionBased   = true;
@@ -724,6 +725,9 @@ public class Table extends TableBase implements SchemaObject {
             case TableBase.TEXT_TABLE :
                 return Tokens.T_TEXT;
 
+            case TableBase.MODULE_TABLE :
+                return Tokens.T_MODULE;
+
             case TableBase.FUNCTION_TABLE :
                 return Tokens.T_FUNCTION;
 
@@ -1186,10 +1190,6 @@ public class Table extends TableBase implements SchemaObject {
         return identityColumn != -1;
     }
 
-    public long getNextIdentity() {
-        return identitySequence.peek();
-    }
-
     /**
      * Match two valid, equal length, columns arrays for type of columns for
      * referential constraints
@@ -1617,11 +1617,12 @@ public class Table extends TableBase implements SchemaObject {
 
         switch (tableType) {
 
+            case TableBase.TEMP_TABLE :
+            case TableBase.INFO_SCHEMA_TABLE :
+            case TableBase.MODULE_TABLE :
             case TableBase.FUNCTION_TABLE :
             case TableBase.SYSTEM_SUBQUERY :
-            case TableBase.INFO_SCHEMA_TABLE :
-            case TableBase.VIEW_TABLE :
-            case TableBase.TEMP_TABLE : {
+            case TableBase.VIEW_TABLE : {
                 Index index = createIndexForColumns(session, new int[]{ col });
 
                 return index;
@@ -1725,11 +1726,12 @@ public class Table extends TableBase implements SchemaObject {
             colTypes = new Type[columnCount];
         }
 
-        colDefaults      = new Expression[columnCount];
-        colNotNull       = new boolean[columnCount];
-        colGenerated     = new boolean[columnCount];
-        colUpdated       = new boolean[columnCount];
-        defaultColumnMap = new int[columnCount];
+        colDefaults          = new Expression[columnCount];
+        colNotNull           = new boolean[columnCount];
+        emptyColumnCheckList = new boolean[columnCount];
+        colGenerated         = new boolean[columnCount];
+        colUpdated           = new boolean[columnCount];
+        defaultColumnMap     = new int[columnCount];
 
         for (int i = 0; i < columnCount; i++) {
             setSingleColumnTypeVars(i);
@@ -1794,12 +1796,12 @@ public class Table extends TableBase implements SchemaObject {
         return columnCheckList;
     }
 
-    int[] getColumnIndexes(String[] list) {
+    int[] findColumnIndexes(String[] list) {
 
         int[] cols = new int[list.length];
 
         for (int i = 0; i < cols.length; i++) {
-            cols[i] = getColumnIndex(list[i]);
+            cols[i] = findColumn(list[i]);
         }
 
         return cols;
@@ -2069,6 +2071,42 @@ public class Table extends TableBase implements SchemaObject {
                        : indexes[i];
     }
 
+    /**
+     * Returns the Index object of the given name or null if not found.
+     */
+    Index getUserIndex(String indexName) {
+
+        Index[] indexes = indexList;
+
+        for (int i = 0; i < indexes.length; i++) {
+            if (indexName.equals(indexes[i].getName().name)) {
+                if (!indexes[i].isConstraint()) {
+                    return indexes[i];
+                }
+            }
+        }
+
+        return null;
+    }
+
+    /**
+     * Returns the Index object of the given name or null if not found.
+     */
+    Index getSystemIndex(String indexName) {
+
+        Index[] indexes = indexList;
+
+        for (int i = 0; i < indexes.length; i++) {
+            if (indexName.equals(indexes[i].getName().name)) {
+                if (indexes[i].isConstraint()) {
+                    return indexes[i];
+                }
+            }
+        }
+
+        return null;
+    }
+
     /**
      *  Return the position of the constraint within the list
      */
@@ -2412,12 +2450,12 @@ public class Table extends TableBase implements SchemaObject {
 
         switch (tableType) {
 
-//            case TableBase.MEMORY_TABLE :
+            case TableBase.TEMP_TABLE :
+            case TableBase.INFO_SCHEMA_TABLE :
+            case TableBase.MODULE_TABLE :
             case TableBase.FUNCTION_TABLE :
             case TableBase.SYSTEM_SUBQUERY :
-            case TableBase.INFO_SCHEMA_TABLE :
-            case TableBase.VIEW_TABLE :
-            case TableBase.TEMP_TABLE : {
+            case TableBase.VIEW_TABLE : {
                 return Index.INDEX_NON_UNIQUE;
             }
         }
@@ -2426,7 +2464,7 @@ public class Table extends TableBase implements SchemaObject {
     }
 
     /**
-     *  Finds an existing index for a column group
+     *  Finds an existing index for a column group - for persistent tables
      */
     synchronized Index getIndexForColumns(Session session, int[] cols) {
 
@@ -2436,20 +2474,6 @@ public class Table extends TableBase implements SchemaObject {
             return indexList[i];
         }
 
-        switch (tableType) {
-
-//            case TableBase.MEMORY_TABLE :
-            case TableBase.FUNCTION_TABLE :
-            case TableBase.SYSTEM_SUBQUERY :
-            case TableBase.INFO_SCHEMA_TABLE :
-            case TableBase.VIEW_TABLE :
-            case TableBase.TEMP_TABLE : {
-                Index index = createIndexForColumns(session, cols);
-
-                return index;
-            }
-        }
-
         return null;
     }
 
@@ -2500,23 +2524,26 @@ public class Table extends TableBase implements SchemaObject {
             ordered);
 
         if (indexUse.length == 0) {
+            Index selected = null;
 
             // index is not full;
             switch (tableType) {
 
+                case TableBase.TEMP_TABLE :
+                case TableBase.INFO_SCHEMA_TABLE :
+                case TableBase.MODULE_TABLE :
                 case TableBase.FUNCTION_TABLE :
                 case TableBase.SYSTEM_SUBQUERY :
-                case TableBase.INFO_SCHEMA_TABLE :
-                case TableBase.VIEW_TABLE :
-                case TableBase.TEMP_TABLE : {
-                    Index selected = createIndexForColumns(session,
-                                                           set.toArray());
+                case TableBase.VIEW_TABLE : {
+                    selected = createIndexForColumns(session, set.toArray());
 
-                    if (selected != null) {
-                        indexUse = selected.asArray();
-                    }
+                    break;
                 }
             }
+
+            if (selected != null) {
+                indexUse = selected.asArray();
+            }
         }
 
         return indexUse;
@@ -2727,8 +2754,8 @@ public class Table extends TableBase implements SchemaObject {
         PersistentStore store = getRowStore(session);
         RowSetNavigator nav   = result.initialiseNavigator();
 
-        while (nav.hasNext()) {
-            Object[] data = nav.getNext();
+        while (nav.next()) {
+            Object[] data = nav.getCurrent();
             Object[] newData =
                 (Object[]) ArrayUtil.resizeArrayIfDifferent(data, columnCount);
 
@@ -2758,8 +2785,8 @@ public class Table extends TableBase implements SchemaObject {
         RowSetNavigator nav   = ins.getNavigator();
         int             count = 0;
 
-        while (nav.hasNext()) {
-            insertSys(session, store, nav.getNext());
+        while (nav.next()) {
+            insertSys(session, store, nav.getCurrent());
 
             count++;
         }
@@ -2775,8 +2802,8 @@ public class Table extends TableBase implements SchemaObject {
 
         RowSetNavigator nav = ins.initialiseNavigator();
 
-        while (nav.hasNext()) {
-            Object[] data = nav.getNext();
+        while (nav.next()) {
+            Object[] data = nav.getCurrent();
             Object[] newData =
                 (Object[]) ArrayUtil.resizeArrayIfDifferent(data, columnCount);
 
@@ -2889,7 +2916,7 @@ public class Table extends TableBase implements SchemaObject {
             for (int i = 0; i < colGenerated.length; i++) {
                 if (colGenerated[i]) {
                     Expression e = getColumn(i).getGeneratingExpression();
-                    RangeIteratorBase range =
+                    RangeIterator range =
                         session.sessionContext.getCheckIterator(
                             getDefaultRanges()[0]);
 
@@ -2957,18 +2984,16 @@ public class Table extends TableBase implements SchemaObject {
             RowIterator it = index.findFirstRow(session, store, data,
                                                 colsSequence);
 
-            row = it.getNextRow();
+            it.next();
+
+            row = it.getCurrentRow();
 
             it.release();
         } else if (bestIndex == null) {
             RowIterator it = rowIterator(session);
 
-            while (true) {
-                row = it.getNextRow();
-
-                if (row == null) {
-                    break;
-                }
+            while (it.next()) {
+                row = it.getCurrentRow();
 
                 if (Table.compareRows(
                         session, row.getData(), data, defaultColumnMap,
@@ -2981,12 +3006,8 @@ public class Table extends TableBase implements SchemaObject {
         } else {
             RowIterator it = bestIndex.findFirstRow(session, store, data);
 
-            while (true) {
-                row = it.getNextRow();
-
-                if (row == null) {
-                    break;
-                }
+            while (it.next()) {
+                row = it.getCurrentRow();
 
                 Object[] rowdata = row.getData();
 
