@@ -502,9 +502,7 @@ public class IndexAVL implements Index {
         long        count = 0;
         RowIterator it    = firstRow(session, store, 0, null);
 
-        while (it.hasNext()) {
-            it.getNextRow();
-
+        while (it.next()) {
             count++;
         }
 
@@ -525,7 +523,7 @@ public class IndexAVL implements Index {
     /**
      * Removes all links between memory nodes
      */
-    public void unlinkNodes(NodeAVL primaryRoot) {
+    public void unlinkNodes(PersistentStore store, NodeAVL primaryRoot) {
 
         NodeAVL x = primaryRoot;
         NodeAVL l = x;
@@ -536,13 +534,13 @@ public class IndexAVL implements Index {
         }
 
         while (x != null) {
-            NodeAVL n = nextUnlink(x);
+            NodeAVL n = nextUnlink(store, x);
 
             x = n;
         }
     }
 
-    private NodeAVL nextUnlink(NodeAVL x) {
+    private NodeAVL nextUnlink(PersistentStore store, NodeAVL x) {
 
         NodeAVL temp = x.getRight(null);
 
@@ -561,7 +559,7 @@ public class IndexAVL implements Index {
         temp = x;
         x    = x.getParent(null);
 
-        while (x != null && x.isRight(temp)) {
+        while (x != null && x.isRight(store, temp)) {
             x.nRight = null;
 
             temp.getRow(null).destroy();
@@ -582,7 +580,9 @@ public class IndexAVL implements Index {
         return x;
     }
 
-    public void checkIndex(PersistentStore store) {
+    public int checkIndex(Session session, PersistentStore store) {
+
+        int errors = 0;
 
         store.readLock();
 
@@ -601,35 +601,70 @@ public class IndexAVL implements Index {
             p = f;
 
             while (f != null) {
-                checkNodes(store, f);
+                errors += checkNodes(store, f);
 
-                f = next(store, f);
+                NodeAVL fnext = next(store, f);
+
+                if (fnext != null) {
+                    int c = compareRowForInsertOrDelete(session,
+                                                        fnext.getRow(store),
+                                                        f.getRow(store), true,
+                                                        0);
+
+                    if (c <= 0) {
+                        if (errors < 10) {
+                            System.out.println("broken index order "
+                                               + getName().name);
+                        }
+
+                        errors++;
+                    }
+                }
+
+                f = fnext;
             }
         } finally {
             store.readUnlock();
         }
+
+        if (errors > 0) {
+            System.out.println("total errors " + getName().name);
+        }
+
+        return errors;
     }
 
-    void checkNodes(PersistentStore store, NodeAVL p) {
+    int checkNodes(PersistentStore store, NodeAVL p) {
 
-        NodeAVL l = p.getLeft(store);
-        NodeAVL r = p.getRight(store);
+        NodeAVL l      = p.nLeft;
+        NodeAVL r      = p.nRight;
+        int     errors = 0;
 
         if (l != null && l.getBalance(store) == -2) {
             System.out.print("broken index - deleted");
+
+            errors++;
         }
 
         if (r != null && r.getBalance(store) == -2) {
             System.out.print("broken index -deleted");
+
+            errors++;
         }
 
         if (l != null && !p.equals(l.getParent(store))) {
             System.out.print("broken index - no parent");
+
+            errors++;
         }
 
         if (r != null && !p.equals(r.getParent(store))) {
             System.out.print("broken index - no parent");
+
+            errors++;
         }
+
+        return errors;
     }
 
     /**
@@ -937,7 +972,7 @@ public class IndexAVL implements Index {
             x = x.setParent(store, dp);
 
             if (dp != null) {
-                if (dp.isRight(d)) {
+                if (dp.isRight(store, d)) {
                     dp = dp.setRight(store, x);
                 } else {
                     dp = dp.setLeft(store, x);
@@ -948,7 +983,7 @@ public class IndexAVL implements Index {
             if (d.equals(xp)) {
                 d = d.setParent(store, x);
 
-                if (d.isLeft(x)) {
+                if (d.isLeft(store, x)) {
                     x = x.setLeft(store, d);
 
                     NodeAVL dr = d.getRight(store);
@@ -1367,7 +1402,7 @@ public class IndexAVL implements Index {
         temp = x;
         x    = x.getParent(store);
 
-        while (x != null && x.isRight(temp)) {
+        while (x != null && x.isRight(store, temp)) {
             temp = x;
             x    = x.getParent(store);
         }
@@ -1410,7 +1445,7 @@ public class IndexAVL implements Index {
 
         depth--;
 
-        while (x != null && x.isRight(temp)) {
+        while (x != null && x.isRight(store, temp)) {
             temp = x;
             x    = x.getParent(store);
 
@@ -1449,7 +1484,7 @@ public class IndexAVL implements Index {
         temp = x;
         x    = x.getParent(store);
 
-        while (x != null && x.isLeft(temp)) {
+        while (x != null && x.isLeft(store, temp)) {
             temp = x;
             x    = x.getParent(store);
         }
@@ -1889,15 +1924,42 @@ public class IndexAVL implements Index {
             nextnode = node;
         }
 
-        public boolean hasNext() {
-            return nextnode != null;
+        public Object getField(int col) {
+
+            if (lastrow == null) {
+                return null;
+            }
+
+            return lastrow.getData()[col];
         }
 
-        public Row getNextRow() {
+        public boolean next() {
+
+            getNextRow();
+
+            return lastrow != null;
+        }
+
+        public Row getCurrentRow() {
+            return lastrow;
+        }
+
+        public Object[] getCurrent() {
+
+            if (lastrow == null) {
+                return null;
+            }
+
+            return lastrow.getData();
+        }
+
+        private Row getNextRow() {
 
             if (nextnode == null) {
                 release();
 
+                lastrow = null;
+
                 return null;
             }
 
@@ -1941,14 +2003,6 @@ public class IndexAVL implements Index {
             return lastrow;
         }
 
-        public Object[] getNext() {
-
-            Row row = getNextRow();
-
-            return row == null ? null
-                               : row.getData();
-        }
-
         public void removeCurrent() {
             store.delete(session, lastrow);
             store.remove(lastrow);
@@ -1957,11 +2011,7 @@ public class IndexAVL implements Index {
         public void release() {}
 
         public long getRowId() {
-            return nextnode.getPos();
-        }
-
-        public TableBase getCurrentTable() {
-            return index.table;
+            return lastrow.getPos();
         }
     }
 }
