@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,21 +35,20 @@ import java.sql.ResultSetMetaData;
 import java.sql.SQLException;
 
 import org.hsqldb.error.ErrorCode;
-import org.hsqldb.persist.HsqlDatabaseProperties;
 import org.hsqldb.result.ResultMetaData;
 import org.hsqldb.types.DateTimeType;
 import org.hsqldb.types.IntervalType;
 import org.hsqldb.types.Type;
 import org.hsqldb.types.Types;
 
-/* $Id: JDBCResultSetMetaData.java 5561 2016-03-18 18:39:11Z fredt $ */
+/* $Id: JDBCResultSetMetaData.java 5737 2017-04-02 10:33:23Z fredt $ */
 
 // fredt@users    - 20040412 - removed DITypeInfo dependencies
-// boucherb@users - 200404xx - removed unused imports;refinement for better
+// campbell-burnet@users - 200404xx - removed unused imports;refinement for better
 //                             usability of getColumnDisplaySize;
 //                             javadoc updates
-// boucherb@users - 20051207 - patch 1.8.0.x initial JDBC 4.0 support work
-// boucherb@users - 20060522 - doc   1.9.0 full synch up to Mustang Build 84
+// campbell-burnet@users - 20051207 - patch 1.8.0.x initial JDBC 4.0 support work
+// campbell-burnet@users - 20060522 - doc   1.9.0 full synch up to Mustang Build 84
 // fredt@users - 1.9.0 usage of types and new ResultMetadata and ColumnMetaData
 // Revision 1.15  2006/07/12 12:29:42  boucherb
 // patch 1.9.0
@@ -92,14 +91,13 @@ import org.hsqldb.types.Types;
  * speed, accuracy, jar-footprint and retention of JDBC resources. <p>
  *
  * (fredt@users) <br>
- * (boucherb@users)<p>
+ * (campbell-burnet@users)<p>
  * </div>
  * <!-- end release-specific documentation -->
  *
- * @author Campbell Burnet (boucherb@users dot sourceforge.net)
+ * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
  * @version 2.2.6
- * @revised JDK 1.6, HSQLDB 2.0
  * @see JDBCStatement#executeQuery
  * @see JDBCStatement#getResultSet
  * @see java.sql.ResultSetMetaData
@@ -331,19 +329,19 @@ public class JDBCResultSetMetaData implements ResultSetMetaData {
      *
      * HSQLDB 2.0 fully supports this feature.  <p>
      *
-     * The current calculation follows these rules: <p>
+     * The current calculation follows these rules:
      *
      * <ol>
      * <li>Long character types and datetime types:<p>
      *
-     *     The maximum length/precision, repectively.<p>
+     *     The maximum length/precision, repectively.
      *
-     * <li>CHAR and VARCHAR types: <p>
+     * <li>CHAR and VARCHAR types:
      *
      *      <ul>
      *      <li> If the result set column is a direct pass through of a table
      *           column value and column size was declared, then the declared
-     *           value is returned. <p>
+     *           value is returned.
      *
      *      <li> Otherwise, the computed length according to SQL Standard is
      *           returned. For very large values, the value of the system property
@@ -360,19 +358,19 @@ public class JDBCResultSetMetaData implements ResultSetMetaData {
      *      and serves only as the current best effort at providing a
      *      value that maximizes usability across a wide range of tools,
      *      given that the HSQLDB database engine allows very large
-     *      lengths to be declared. <p>
+     *      lengths to be declared.
      *
      * <li>Number types: <p>
      *
      *     The max precision, plus the length of the negation character (1),
      *     plus (if applicable) the maximum number of characters that may
      *     occupy the exponent character sequence.  Note that some legacy tools
-     *     do not correctly handle BIGINT values of greater than 18 digits. <p>
+     *     do not correctly handle BIGINT values of greater than 18 digits.
      *
      * <li>BOOLEAN type: <p>
      *
      *     The length of the character sequence "false" (5), the longer of the
-     *     two boolean value String representations. <p>
+     *     two boolean value String representations.
      *
      * <li>Remaining types: <p>
      *
@@ -414,12 +412,12 @@ public class JDBCResultSetMetaData implements ResultSetMetaData {
      * <h3>HSQLDB-Specific Information:</h3> <p>
      *
      * In HSQLDB, a <code>ResultSet</code> column label is determined using the
-     * following order of precedence:<p>
+     * following order of precedence:
      *
      * <OL>
      * <LI>The label (alias) specified in the generating query.</LI>
      * <LI>The name of the underlying column, if no label is specified.<br>
-     * <L1>C1, C2, etc. for computed columns that have no label.</LI>
+     * <LI>C1, C2, etc. for computed columns that have no label.</LI>
      * </OL> <p>
      *
      * </div>
@@ -452,13 +450,13 @@ public class JDBCResultSetMetaData implements ResultSetMetaData {
      * <h3>HSQLDB-Specific Information:</h3> <p>
      *
      * In HSQLDB, a ResultSet column name is determined using the following
-     * order of precedence:<p>
+     * order of precedence:
      *
      * <OL>
      * <LI>The name of the underlying column, if the ResultSet column
      *   represents a column in a table.</LI>
      * <LI>The label or alias specified in the generating query.</LI>
-     * <L1>C1, C2, etc. for computed columns that have no label.</LI>
+     * <LI>C1, C2, etc. for computed columns that have no label.</LI>
      * </OL> <p>
      *
      * If the <code>jdbc.get_column_name</code> property of the JDBC Connection
@@ -804,8 +802,7 @@ public class JDBCResultSetMetaData implements ResultSetMetaData {
      * <code>ResultSet.getObject</code> to retrieve the value in the specified
      * column. This is the class name used for custom mapping.
      * @exception SQLException if a database access error occurs
-     * @since JDK 1.2 (JDK 1.1.x developers: read the overview for
-     *      JDBCResultSet)
+     * @since JDK 1.2
      */
     public String getColumnClassName(int column) throws SQLException {
 
@@ -834,7 +831,7 @@ public class JDBCResultSetMetaData implements ResultSetMetaData {
      * @param iface A Class defining an interface that the result must implement.
      * @return an object that implements the interface. May be a proxy for the actual implementing object.
      * @throws java.sql.SQLException If no object found that implements the interface
-     * @since JDK 1.6, HSQLDB 1.8.x
+     * @since JDK 1.6
      */
 //#ifdef JAVA6
     @SuppressWarnings("unchecked")
@@ -862,7 +859,7 @@ public class JDBCResultSetMetaData implements ResultSetMetaData {
      * @return true if this implements the interface or directly or indirectly wraps an object that does.
      * @throws java.sql.SQLException  if an error occurs while determining whether this is a wrapper
      * for an object with the given interface.
-     * @since JDK 1.6, HSQLDB 1.8.x
+     * @since JDK 1.6
      */
 //#ifdef JAVA6
     public boolean isWrapperFor(
@@ -949,8 +946,6 @@ public class JDBCResultSetMetaData implements ResultSetMetaData {
         if (this.translateTTIType) {
             if (type.isIntervalType()) {
                 type = ((IntervalType) type).getCharacterType();
-            } else if (type.isDateTimeTypeWithZone()) {
-                type = ((DateTimeType) type).getDateTimeTypeWithoutZone();
             }
         }
 
