@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -33,11 +33,18 @@ package org.hsqldb.jdbc;
 
 import java.sql.BatchUpdateException;
 import java.sql.Connection;
+import java.sql.DatabaseMetaData;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.SQLWarning;
 import java.sql.Statement;
 
+//#ifdef JAVA6
+import java.sql.SQLFeatureNotSupportedException;
+import java.sql.SQLTimeoutException;
+
+//#endif JAVA6
+
 import org.hsqldb.HsqlException;
 import org.hsqldb.StatementTypes;
 import org.hsqldb.navigator.RowSetNavigator;
@@ -45,14 +52,13 @@ import org.hsqldb.result.Result;
 import org.hsqldb.result.ResultConstants;
 import org.hsqldb.result.ResultProperties;
 
-/* $Id: JDBCStatement.java 5623 2016-05-05 21:03:57Z fredt $ */
+/* $Id: JDBCStatement.java 5740 2017-04-02 15:57:38Z fredt $ */
 
 // fredt@users   20020320 - patch 1.7.0 - JDBC 2 support and error trapping
 //
-// JDBC 2 methods can now be called from jdk 1.1.x - see javadoc comments
 // SCROLL_INSENSITIVE and FORWARD_ONLY types for ResultSet are now supported
 //
-// boucherb@users 20020509 - added "throws SQLException" to all methods where
+// campbell-burnet@users 20020509 - added "throws SQLException" to all methods where
 //                           it was missing here but specified in the
 //                           java.sql.Statement interface,
 //                           updated generic documentation to JDK 1.4, and
@@ -60,9 +66,9 @@ import org.hsqldb.result.ResultProperties;
 // boucherb &     20020505 - extensive review and update of docs and behaviour
 // fredt@users               to comply with java.sql specification
 // fredt@users    20030620 - patch 1.7.2 - rewritten and simplified
-// boucherb@users 200404xx - javadoc updates toward 1.7.2 final
-// boucherb@users 20051207 - patch 1.8.0.x initial JDBC 4.0 support work
-// boucherb@users 20060522 - doc   1.9.0 full synch up to Mustang Build 84
+// campbell-burnet@users 200404xx - javadoc updates toward 1.7.2 final
+// campbell-burnet@users 20051207 - patch 1.8.0.x initial JDBC 4.0 support work
+// campbell-burnet@users 20060522 - doc   1.9.0 full synch up to Mustang Build 84
 // Revision 1.16  2006/07/12 12:40:59  boucherb
 // patch 1.9.0
 // - full synch up to Mustang b90
@@ -87,55 +93,22 @@ import org.hsqldb.result.ResultProperties;
  * From version 2.0, the implementation meets the JDBC specification
  * requirement that any existing ResultSet is closed when execute() or
  * executeQuery() methods are called. The connection property close_result=true
- * is required for this behaviour.
+ * is required for this behaviour.<p>
+ *
+ * Methods added in JAVA 8 are generally supported when the HSQLDB jar is compiled
+ * with JDK 8.
  * <p>
  *
- * <b>JRE 1.1.x Notes:</b> <p>
- *
- * In general, JDBC 2 support requires Java 1.2 and above, and JDBC3 requires
- * Java 1.4 and above. In HSQLDB, support for methods introduced in different
- * versions of JDBC depends on the JDK version used for compiling and building
- * HSQLDB.<p>
- *
- * Since 1.7.0, all JDBC 2 methods can be called while executing under the
- * version 1.1.x
- * <em>Java Runtime Environment</em><sup><font size="-2">TM</font></sup>.
- * However, in addition to this technique requiring explicit casts to the
- * org.hsqldb.jdbc.* classes, some of these method calls require
- * <code>int</code> values that are defined only in the JDBC 2 or greater
- * version of the {@link java.sql.ResultSet ResultSet} interface.  For this
- * reason these values are defined in {@link JDBCResultSet JDBCResultSet}.<p>
- *
- * In a JRE 1.1.x environment, calling JDBC 2 methods that take or return the
- * JDBC2-only <code>ResultSet</code> values can be achieved by referring
- * to them in parameter specifications and return value comparisons,
- * respectively, as follows: <p>
- *
- * <pre class="JavaCodeExample">
- * JDBCResultSet.FETCH_FORWARD
- * JDBCResultSet.TYPE_FORWARD_ONLY
- * JDBCResultSet.TYPE_SCROLL_INSENSITIVE
- * JDBCResultSet.CONCUR_READ_ONLY
- * //etc.
- * </pre> <p>
- *
- * However, please note that code written to use HSQLDB JDBC 2 features under
- * JDK 1.1.x will not be compatible for use with other JDBC 2 drivers. Please
- * also note that this feature is offered solely as a convenience to developers
- * who must work under JDK 1.1.x due to operating constraints, yet wish to
- * use some of the more advanced features available under the JDBC 2
- * specification. <p>
- *
  * (fredt@users)<br>
- * (boucherb@users)<p>
+ * (campbell-burnet@users)<p>
  *
  * </div>
  * <!-- end release-specific documentation -->
  *
- * @author Campbell Burnet (boucherb@users dot sourceforge.net)
+ * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
- * @revised JDK 1.6, HSQLDB 2.0
+ * @version 2.4.0
+ * @since HSQLDB 1.9.0
  * @see JDBCConnection#createStatement
  * @see JDBCResultSet
  */
@@ -319,7 +292,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @param max the new column size limit in bytes; zero means there is no limit
      * @exception SQLException if a database access error occurs,
      * this method is called on a closed <code>Statement</code>
-     *            or the condition max >= 0 is not satisfied
+     * or the condition {@code max >= 0} is not satisfied
      * @see #getMaxFieldSize
      */
     public void setMaxFieldSize(int max) throws SQLException {
@@ -366,7 +339,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @param max the new max rows limit; zero means there is no limit
      * @exception SQLException if a database access error occurs,
      * this method is called on a closed <code>Statement</code>
-     *            or the condition max >= 0 is not satisfied
+     * or the condition {@code max >= 0} is not satisfied
      * @see #getMaxRows
      */
     public synchronized void setMaxRows(int max) throws SQLException {
@@ -456,7 +429,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      *        there is no limit
      * @exception SQLException if a database access error occurs,
      * this method is called on a closed <code>Statement</code>
-     *            or the condition seconds >= 0 is not satisfied
+     * or the condition {@code seconds >= 0} is not satisfied
      * @see #getQueryTimeout
      */
     public void setQueryTimeout(int seconds) throws SQLException {
@@ -498,7 +471,8 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
     public void cancel() throws SQLException {
         checkClosed();
         String sql = resultOut.getMainString();
-        Result request = Result.newCancelRequest(-1, sql);
+        int randomId = connection.sessionProxy.getRandomId();
+        Result request = Result.newCancelRequest(randomId, -1, sql);
 
         try {
             Result response = connection.sessionProxy.cancel(request);
@@ -698,9 +672,9 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      *
      * <P>There are no more results when the following is true:
      * <PRE>
-     *     // stmt is a Statement object
+     *     // stmt is a Statement object {@code
      *     ((stmt.getMoreResults() == false) && (stmt.getUpdateCount() == -1))
-     * </PRE>
+     * }</PRE>
      * <!-- end generic documentation -->
      *
      * @return <code>true</code> if the next result is a <code>ResultSet</code>
@@ -734,6 +708,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * <h3>HSQLDB-Specific Information:</h3> <p>
      *
      * HSQLDB accepts all valid parameters. <p>
+     * </div>
      * <!-- end release-specific documentation -->
      *
      * @param direction the initial direction for processing rows
@@ -742,8 +717,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * or the given direction
      * is not one of <code>ResultSet.FETCH_FORWARD</code>,
      * <code>ResultSet.FETCH_REVERSE</code>, or <code>ResultSet.FETCH_UNKNOWN</code>
-     * @since JDK 1.2 (JDK 1.1.x developers: read the overview
-     *    for JDBCStatement)
+     * @since JDK 1.2
      * @see #getFetchDirection
      */
     public synchronized void setFetchDirection(
@@ -754,9 +728,9 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
 
         switch (direction) {
 
-            case JDBCResultSet.FETCH_FORWARD :
-            case JDBCResultSet.FETCH_REVERSE :
-            case JDBCResultSet.FETCH_UNKNOWN :
+            case ResultSet.FETCH_FORWARD :
+            case ResultSet.FETCH_REVERSE :
+            case ResultSet.FETCH_UNKNOWN :
                 fetchDirection = direction;
 
                 break;
@@ -787,8 +761,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      *          from this <code>Statement</code> object
      * @exception SQLException if a database access error occurs or
      * this method is called on a closed <code>Statement</code>
-     * @since JDK 1.2 (JDK 1.1.x developers: read the overview
-     *    for JDBCStatement)
+     * @since JDK 1.2
      * @see #setFetchDirection
      */
     public int getFetchDirection() throws SQLException {
@@ -821,9 +794,8 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @exception SQLException if a database access error occurs,
      * this method is called on a closed <code>Statement</code> or the
      *        (JDBC4 modified:)
-     *        condition  <code>rows >= 0</code> is not satisfied.
-     * @since JDK 1.2 (JDK 1.1.x developers: read the overview
-     *   for JDBCStatement)
+     *        condition  {@code rows >= 0} is not satisfied.
+     * @since JDK 1.2
      * @see #getFetchSize
      */
     public synchronized void setFetchSize(int rows) throws SQLException {
@@ -858,8 +830,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      *          from this <code>Statement</code> object
      * @exception SQLException if a database access error occurs or
      * this method is called on a closed <code>Statement</code>
-     * @since JDK 1.2 (JDK 1.1.x developers: read the overview
-     *  for JDBCStatement)
+     * @since JDK 1.2
      * @see #setFetchSize
      */
     public synchronized int getFetchSize() throws SQLException {
@@ -888,8 +859,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * <code>ResultSet.CONCUR_UPDATABLE</code>
      * @exception SQLException if a database access error occurs or
      * this method is called on a closed <code>Statement</code>
-     * @since JDK 1.2 (JDK 1.1.x developers: read the overview
-     *  for JDBCStatement)
+     * @since JDK 1.2
      */
     public synchronized int getResultSetConcurrency() throws SQLException {
 
@@ -918,8 +888,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * <code>ResultSet.TYPE_SCROLL_SENSITIVE</code>
      * @exception SQLException if a database access error occurs or
      * this method is called on a closed <code>Statement</code>
-     * @since JDK 1.2 (JDK 1.1.x developers: read the overview
-     *   for JDBCStatement)
+     * @since JDK 1.2
      */
     public synchronized int getResultSetType() throws SQLException {
 
@@ -954,8 +923,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * this method is called on a closed <code>Statement</code> or the
      * driver does not support batch updates
      * @see #executeBatch
-     * @since JDK 1.2 (JDK 1.1.x developers: read the overview
-     *   for JDBCStatement)
+     * @since JDK 1.2
      */
     public synchronized void addBatch(String sql) throws SQLException {
 
@@ -992,8 +960,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      *  this method is called on a closed <code>Statement</code> or the
      * driver does not support batch updates
      * @see #addBatch
-     * @since JDK 1.2 (JDK 1.1.x developers: read the overview
-     *   for JDBCStatement)
+     * @since JDK 1.2
      */
     public synchronized void clearBatch() throws SQLException {
 
@@ -1032,7 +999,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * <code>BatchUpdateException.getUpdateCounts</code>
      * will contain as many elements as there are commands in the batch, and
      * at least one of the elements will be the following:
-     * <P>
+     *
      * <LI>A value of <code>EXECUTE_FAILED</code> -- indicates that the command failed
      * to execute successfully and occurs only if a driver continues to
      * process commands after a command fails
@@ -1072,8 +1039,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      *
      * @see #addBatch
      * @see java.sql.DatabaseMetaData#supportsBatchUpdates
-     * @since JDK 1.3 (JDK 1.1.x developers: read the overview for
-     * JDBCStatement)
+     * @since JDK 1.3
      */
     public synchronized int[] executeBatch() throws SQLException {
 
@@ -1105,8 +1071,8 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
         RowSetNavigator navigator    = resultIn.getNavigator();
         int[]           updateCounts = new int[navigator.getSize()];
 
-        for (int i = 0; i < updateCounts.length; i++) {
-            Object[] data = navigator.getNext();
+        for (int i = 0; navigator.next(); i++) {
+            Object[] data = navigator.getCurrent();
 
             updateCounts[i] = ((Integer) data[0]).intValue();
         }
@@ -1133,8 +1099,7 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      * @return the connection that produced this statement
      * @exception SQLException if a database access error occurs or
      * this method is called on a closed <code>Statement</code>
-     * @since JDK 1.2 (JDK 1.1.x developers: read the overview
-     *    for JDBCStatement)
+     * @since JDK 1.2
      */
     public synchronized Connection getConnection() throws SQLException {
 
@@ -1154,9 +1119,9 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
      *
      * <P>There are no more results when the following is true:
      * <PRE>
-     *     // stmt is a Statement object
+     *     // stmt is a Statement object{@code
      *     ((stmt.getMoreResults(current) == false) && (stmt.getUpdateCount() == -1))
-     * </PRE>
+     * }</PRE>
      * <!-- end generic documentation -->
      *
      * <!-- start release-specific documentation -->
@@ -1743,6 +1708,348 @@ public class JDBCStatement extends JDBCStatementBase implements Statement {
     }
 
 //#endif JAVA6
+
+    //--------------------------JDBC 4.2 -----------------------------
+
+    /**
+     *  Retrieves the current result as an update count; if the result
+     * is a <code>ResultSet</code> object or there are no more results, -1
+     *  is returned. This method should be called only once per result.
+     * <p>
+     * This method should be used when the returned row count may exceed
+     * {@link Integer#MAX_VALUE}.
+     *<p>
+     * The public implementation will throw {@code UnsupportedOperationException}
+     *
+     * @return the current result as an update count; -1 if the current result
+     * is a <code>ResultSet</code> object or there are no more results
+     * @exception SQLException if a database access error occurs or
+     * this method is called on a closed <code>Statement</code>
+     * @see #execute
+     * @since 1.8
+     */
+//#ifdef JAVA8
+/*
+    public long getLargeUpdateCount() throws SQLException {
+        return super.getUpdateCount();
+    }
+*/
+
+//#endif JAVA8
+    /**
+     * Sets the limit for the maximum number of rows that any
+     * <code>ResultSet</code> object  generated by this <code>Statement</code>
+     * object can contain to the given number.
+     * If the limit is exceeded, the excess
+     * rows are silently dropped.
+     * <p>
+     * This method should be used when the row limit may exceed
+     * {@link Integer#MAX_VALUE}.
+     *<p>
+     * The default implementation will throw {@code UnsupportedOperationException}
+     *
+     * @param max the new max rows limit; zero means there is no limit
+     * @exception SQLException if a database access error occurs,
+     * this method is called on a closed <code>Statement</code>
+     *            or the condition {@code max >= 0} is not satisfied
+     * @see #getMaxRows
+     * @since 1.8
+     */
+//#ifdef JAVA8
+/*
+    public void setLargeMaxRows(long max) throws SQLException {
+
+        int maxRows = max > Integer.MAX_VALUE ? Integer.MAX_VALUE :
+                                                (int) max;
+        setMaxRows(maxRows);
+    }
+*/
+//#endif JAVA8
+
+    /**
+     * Retrieves the maximum number of rows that a
+     * <code>ResultSet</code> object produced by this
+     * <code>Statement</code> object can contain.  If this limit is exceeded,
+     * the excess rows are silently dropped.
+     * <p>
+     * This method should be used when the returned row limit may exceed
+     * {@link Integer#MAX_VALUE}.
+     *<p>
+     * The default implementation will return {@code 0}
+     *
+     * @return the current maximum number of rows for a <code>ResultSet</code>
+     *         object produced by this <code>Statement</code> object;
+     *         zero means there is no limit
+     * @exception SQLException if a database access error occurs or
+     * this method is called on a closed <code>Statement</code>
+     * @see #setMaxRows
+     * @since 1.8
+     */
+//#ifdef JAVA8
+/*
+    public long getLargeMaxRows() throws SQLException {
+        return maxRows;
+    }
+*/
+//#endif JAVA8
+
+    /**
+     * Submits a batch of commands to the database for execution and
+     * if all commands execute successfully, returns an array of update counts.
+     * The <code>long</code> elements of the array that is returned are ordered
+     * to correspond to the commands in the batch, which are ordered
+     * according to the order in which they were added to the batch.
+     * The elements in the array returned by the method {@code executeLargeBatch}
+     * may be one of the following:
+     * <OL>
+     * <LI>A number greater than or equal to zero -- indicates that the
+     * command was processed successfully and is an update count giving the
+     * number of rows in the database that were affected by the command's
+     * execution
+     * <LI>A value of <code>SUCCESS_NO_INFO</code> -- indicates that the command was
+     * processed successfully but that the number of rows affected is
+     * unknown
+     * <P>
+     * If one of the commands in a batch update fails to execute properly,
+     * this method throws a <code>BatchUpdateException</code>, and a JDBC
+     * driver may or may not continue to process the remaining commands in
+     * the batch.  However, the driver's behavior must be consistent with a
+     * particular DBMS, either always continuing to process commands or never
+     * continuing to process commands.  If the driver continues processing
+     * after a failure, the array returned by the method
+     * <code>BatchUpdateException.getLargeUpdateCounts</code>
+     * will contain as many elements as there are commands in the batch, and
+     * at least one of the elements will be the following:
+     *
+     * <LI>A value of <code>EXECUTE_FAILED</code> -- indicates that the command failed
+     * to execute successfully and occurs only if a driver continues to
+     * process commands after a command fails
+     * </OL>
+     * <p>
+     * This method should be used when the returned row count may exceed
+     * {@link Integer#MAX_VALUE}.
+     *<p>
+     * The default implementation will throw {@code UnsupportedOperationException}
+     *
+     * @return an array of update counts containing one element for each
+     * command in the batch.  The elements of the array are ordered according
+     * to the order in which commands were added to the batch.
+     * @exception SQLException if a database access error occurs,
+     * this method is called on a closed <code>Statement</code> or the
+     * driver does not support batch statements. Throws {@link BatchUpdateException}
+     * (a subclass of <code>SQLException</code>) if one of the commands sent to the
+     * database fails to execute properly or attempts to return a result set.
+     * @throws SQLTimeoutException when the driver has determined that the
+     * timeout value that was specified by the {@code setQueryTimeout}
+     * method has been exceeded and has at least attempted to cancel
+     * the currently running {@code Statement}
+     *
+     * @see #addBatch
+     * @see DatabaseMetaData#supportsBatchUpdates
+     * @since 1.8
+     */
+//#ifdef JAVA8
+/*
+    public long[] executeLargeBatch() throws SQLException {
+        int[] updateCounts = executeBatch();
+
+        long[] longCounts = new long[updateCounts.length];
+
+        for(int i = 0; i < updateCounts.length; i++) {
+            longCounts[i] = updateCounts[i];
+        }
+
+        return longCounts;
+    }
+*/
+//#endif JAVA8
+
+    /**
+     * Executes the given SQL statement, which may be an <code>INSERT</code>,
+     * <code>UPDATE</code>, or <code>DELETE</code> statement or an
+     * SQL statement that returns nothing, such as an SQL DDL statement.
+     * <p>
+     * This method should be used when the returned row count may exceed
+     * {@link Integer#MAX_VALUE}.
+     * <p>
+     * <strong>Note:</strong>This method cannot be called on a
+     * <code>PreparedStatement</code> or <code>CallableStatement</code>.
+     *<p>
+     * The default implementation will throw {@code UnsupportedOperationException}
+     *
+     * @param sql an SQL Data Manipulation Language (DML) statement,
+     * such as <code>INSERT</code>, <code>UPDATE</code> or
+     * <code>DELETE</code>; or an SQL statement that returns nothing,
+     * such as a DDL statement.
+     *
+     * @return either (1) the row count for SQL Data Manipulation Language
+     * (DML) statements or (2) 0 for SQL statements that return nothing
+     *
+     * @exception SQLException if a database access error occurs,
+     * this method is called on a closed <code>Statement</code>, the given
+     * SQL statement produces a <code>ResultSet</code> object, the method is called on a
+     * <code>PreparedStatement</code> or <code>CallableStatement</code>
+     * @throws SQLTimeoutException when the driver has determined that the
+     * timeout value that was specified by the {@code setQueryTimeout}
+     * method has been exceeded and has at least attempted to cancel
+     * the currently running {@code Statement}
+     * @since 1.8
+     */
+//#ifdef JAVA8
+/*
+    public long executeLargeUpdate(String sql) throws SQLException {
+        return executeUpdate(sql);
+    }
+*/
+//#endif JAVA8
+
+    /**
+     * Executes the given SQL statement and signals the driver with the
+     * given flag about whether the
+     * auto-generated keys produced by this <code>Statement</code> object
+     * should be made available for retrieval.  The driver will ignore the
+     * flag if the SQL statement
+     * is not an <code>INSERT</code> statement, or an SQL statement able to return
+     * auto-generated keys (the list of such statements is vendor-specific).
+     * <p>
+     * This method should be used when the returned row count may exceed
+     * {@link Integer#MAX_VALUE}.
+     * <p>
+     * <strong>Note:</strong>This method cannot be called on a
+     * <code>PreparedStatement</code> or <code>CallableStatement</code>.
+     *<p>
+     * The default implementation will throw {@code SQLFeatureNotSupportedException}
+     *
+     * @param sql an SQL Data Manipulation Language (DML) statement,
+     * such as <code>INSERT</code>, <code>UPDATE</code> or
+     * <code>DELETE</code>; or an SQL statement that returns nothing,
+     * such as a DDL statement.
+     *
+     * @param autoGeneratedKeys a flag indicating whether auto-generated keys
+     *        should be made available for retrieval;
+     *         one of the following constants:
+     *         <code>Statement.RETURN_GENERATED_KEYS</code>
+     *         <code>Statement.NO_GENERATED_KEYS</code>
+     * @return either (1) the row count for SQL Data Manipulation Language (DML) statements
+     *         or (2) 0 for SQL statements that return nothing
+     *
+     * @exception SQLException if a database access error occurs,
+     *  this method is called on a closed <code>Statement</code>, the given
+     *            SQL statement returns a <code>ResultSet</code> object,
+     *            the given constant is not one of those allowed, the method is called on a
+     * <code>PreparedStatement</code> or <code>CallableStatement</code>
+     * @exception SQLFeatureNotSupportedException if the JDBC driver does not support
+     * this method with a constant of Statement.RETURN_GENERATED_KEYS
+     * @throws SQLTimeoutException when the driver has determined that the
+     * timeout value that was specified by the {@code setQueryTimeout}
+     * method has been exceeded and has at least attempted to cancel
+     * the currently running {@code Statement}
+     * @since 1.8
+     */
+//#ifdef JAVA8
+/*
+    public long executeLargeUpdate(String sql, int autoGeneratedKeys)
+            throws SQLException {
+        return executeUpdate(sql, autoGeneratedKeys);
+    }
+*/
+//#endif JAVA8
+
+    /**
+     * Executes the given SQL statement and signals the driver that the
+     * auto-generated keys indicated in the given array should be made available
+     * for retrieval.   This array contains the indexes of the columns in the
+     * target table that contain the auto-generated keys that should be made
+     * available. The driver will ignore the array if the SQL statement
+     * is not an <code>INSERT</code> statement, or an SQL statement able to return
+     * auto-generated keys (the list of such statements is vendor-specific).
+     * <p>
+     * This method should be used when the returned row count may exceed
+     * {@link Integer#MAX_VALUE}.
+     * <p>
+     * <strong>Note:</strong>This method cannot be called on a
+     * <code>PreparedStatement</code> or <code>CallableStatement</code>.
+     *<p>
+     * The default implementation will throw {@code SQLFeatureNotSupportedException}
+     *
+     * @param sql an SQL Data Manipulation Language (DML) statement,
+     * such as <code>INSERT</code>, <code>UPDATE</code> or
+     * <code>DELETE</code>; or an SQL statement that returns nothing,
+     * such as a DDL statement.
+     *
+     * @param columnIndexes an array of column indexes indicating the columns
+     *        that should be returned from the inserted row
+     * @return either (1) the row count for SQL Data Manipulation Language (DML) statements
+     *         or (2) 0 for SQL statements that return nothing
+     *
+     * @exception SQLException if a database access error occurs,
+     * this method is called on a closed <code>Statement</code>, the SQL
+     * statement returns a <code>ResultSet</code> object,the second argument
+     * supplied to this method is not an
+     * <code>int</code> array whose elements are valid column indexes, the method is called on a
+     * <code>PreparedStatement</code> or <code>CallableStatement</code>
+     * @throws SQLFeatureNotSupportedException  if the JDBC driver does not support this method
+     * @throws SQLTimeoutException when the driver has determined that the
+     * timeout value that was specified by the {@code setQueryTimeout}
+     * method has been exceeded and has at least attempted to cancel
+     * the currently running {@code Statement}
+     * @since 1.8
+     */
+//#ifdef JAVA8
+/*
+    public long executeLargeUpdate(String sql, int columnIndexes[]) throws SQLException {
+        return executeUpdate(sql, columnIndexes);
+    }
+*/
+//#endif JAVA8
+
+    /**
+     * Executes the given SQL statement and signals the driver that the
+     * auto-generated keys indicated in the given array should be made available
+     * for retrieval.   This array contains the names of the columns in the
+     * target table that contain the auto-generated keys that should be made
+     * available. The driver will ignore the array if the SQL statement
+     * is not an <code>INSERT</code> statement, or an SQL statement able to return
+     * auto-generated keys (the list of such statements is vendor-specific).
+     * <p>
+     * This method should be used when the returned row count may exceed
+     * {@link Integer#MAX_VALUE}.
+     * <p>
+     * <strong>Note:</strong>This method cannot be called on a
+     * <code>PreparedStatement</code> or <code>CallableStatement</code>.
+     *<p>
+     * The default implementation will throw {@code SQLFeatureNotSupportedException}
+     *
+     * @param sql an SQL Data Manipulation Language (DML) statement,
+     * such as <code>INSERT</code>, <code>UPDATE</code> or
+     * <code>DELETE</code>; or an SQL statement that returns nothing,
+     * such as a DDL statement.
+     * @param columnNames an array of the names of the columns that should be
+     *        returned from the inserted row
+     * @return either the row count for <code>INSERT</code>, <code>UPDATE</code>,
+     *         or <code>DELETE</code> statements, or 0 for SQL statements
+     *         that return nothing
+     * @exception SQLException if a database access error occurs,
+     *  this method is called on a closed <code>Statement</code>, the SQL
+     *            statement returns a <code>ResultSet</code> object, the
+     *            second argument supplied to this method is not a <code>String</code> array
+     *            whose elements are valid column names, the method is called on a
+     * <code>PreparedStatement</code> or <code>CallableStatement</code>
+     * @throws SQLFeatureNotSupportedException  if the JDBC driver does not support this method
+     * @throws SQLTimeoutException when the driver has determined that the
+     * timeout value that was specified by the {@code setQueryTimeout}
+     * method has been exceeded and has at least attempted to cancel
+     * the currently running {@code Statement}
+     * @since 1.8
+     */
+//#ifdef JAVA8
+/*
+    public long executeLargeUpdate(String sql, String columnNames[])
+            throws SQLException {
+        return executeUpdate(sql, columnNames);
+    }
+*/
+//#endif JAVA8
     // -------------------- Internal Implementation ----------------------------
 
     /**
