@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -46,7 +46,7 @@ import org.hsqldb.lib.ObjectComparator;
  * Special getOrAddXXX() methods are used for object maps in some subclasses.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.4
  * @since 1.7.2
  */
 public class BaseHashMap {
@@ -90,6 +90,8 @@ public class BaseHashMap {
     boolean           isObjectValue;
     protected boolean isTwoObjectValue;
     protected boolean isList;
+    protected boolean isAccessCount;
+    protected boolean isLastAccessCount;
 
     //
     private ValuesIterator valuesIterator;
@@ -193,6 +195,8 @@ public class BaseHashMap {
             isNoValue = true;
         }
 
+        isLastAccessCount = hasAccessCount;
+
         if (hasAccessCount) {
             accessTable = new int[arraySize];
         }
@@ -302,6 +306,8 @@ public class BaseHashMap {
 
     /**
      * generic method for adding or removing keys
+     *
+     * returns existing Object value if any (or Object key if this is a set)
      */
     protected Object addOrRemove(long longKey, long longValue,
                                  Object objectKey, Object objectValue,
@@ -390,6 +396,10 @@ public class BaseHashMap {
                 return returnValue;
             }
 
+            if (isObjectKey) {
+                returnValue = objectKeyTable[lookup];
+            }
+
             if (isObjectValue) {
                 returnValue              = objectValueTable[lookup];
                 objectValueTable[lookup] = objectValue;
@@ -399,8 +409,10 @@ public class BaseHashMap {
                 longValueTable[lookup] = longValue;
             }
 
-            if (accessTable != null) {
+            if (isLastAccessCount) {
                 accessTable[lookup] = ++accessCount;
+            } else if (isAccessCount) {
+                accessTable[lookup]++;
             }
 
             return returnValue;
@@ -450,8 +462,10 @@ public class BaseHashMap {
         }
 
         //
-        if (accessTable != null) {
+        if (isLastAccessCount) {
             accessTable[lookup] = ++accessCount;
+        } else if (isAccessCount) {
+            accessTable[lookup] = 1;
         }
 
         return returnValue;
@@ -648,8 +662,10 @@ public class BaseHashMap {
         }
 
         //
-        if (accessTable != null) {
+        if (isLastAccessCount) {
             accessTable[lookup] = ++accessCount;
+        } else if (isAccessCount) {
+            accessTable[lookup] = 1;
         }
 
         return returnValue;
@@ -719,8 +735,10 @@ public class BaseHashMap {
                 objectKeyTable[lookup] = objectValueTwo;
             }
 
-            if (accessTable != null) {
+            if (isLastAccessCount) {
                 accessTable[lookup] = ++accessCount;
+            } else if (isAccessCount) {
+                accessTable[lookup]++;
             }
 
             return returnValue;
@@ -759,8 +777,10 @@ public class BaseHashMap {
             objectKeyTable[lookup] = objectValueTwo;
         }
 
-        if (accessTable != null) {
+        if (isLastAccessCount) {
             accessTable[lookup] = ++accessCount;
+        } else if (isAccessCount) {
+            accessTable[lookup] = 1;
         }
 
         return returnValue;
@@ -846,8 +866,10 @@ public class BaseHashMap {
             } else {
                 objectKeyTable[lookup] = object;
 
-                if (accessTable != null) {
+                if (isLastAccessCount) {
                     accessTable[lookup] = ++accessCount;
+                } else if (isAccessCount) {
+                    accessTable[lookup]++;
                 }
             }
 
@@ -867,8 +889,10 @@ public class BaseHashMap {
         lookup                 = hashIndex.linkNode(index, lastLookup);
         objectKeyTable[lookup] = object;
 
-        if (accessTable != null) {
+        if (isLastAccessCount) {
             accessTable[lookup] = ++accessCount;
+        } else if (isAccessCount) {
+            accessTable[lookup] = 1;
         }
 
         return returnValue;
@@ -970,7 +994,7 @@ public class BaseHashMap {
     }
 
     /**
-     * resize the arrays contianing the key / value data
+     * resize the arrays containing the key / value data
      */
     private void resizeElementArrays(int dataLength, int newLength) {
 
@@ -1258,7 +1282,7 @@ public class BaseHashMap {
      * Return the max accessCount value for count elements with the lowest
      * access count. Always return at least accessMin + 1
      */
-    public int getAccessCountCeiling(int count, int margin) {
+    protected int getAccessCountCeiling(int count, int margin) {
         return ArrayCounter.rank(accessTable, hashIndex.newNodePointer, count,
                                  accessMin, accessCount, margin);
     }
@@ -1267,11 +1291,11 @@ public class BaseHashMap {
      * This is called after all elements below count accessCount have been
      * removed
      */
-    public void setAccessCountFloor(int count) {
+    protected void setAccessCountFloor(int count) {
         accessMin = count;
     }
 
-    public int incrementAccessCount() {
+    protected int incrementAccessCount() {
         return ++accessCount;
     }
 
@@ -1326,7 +1350,7 @@ public class BaseHashMap {
         accessMin   = 0;
     }
 
-    public int capacity() {
+    protected int capacity() {
         return hashIndex.linkTable.length;
     }
 
@@ -1512,6 +1536,7 @@ public class BaseHashMap {
             toNextLookup();
 
             oldKey = value;
+
             return value;
         }
 
@@ -1611,7 +1636,7 @@ public class BaseHashMap {
 
         public long nextLong() throws NoSuchElementException {
 
-            if ((!isLongKey || !keys)) {
+            if ((keys && !isLongKey) || (!keys && !isLongValue)) {
                 throw new NoSuchElementException("Hash Iterator");
             }
 
@@ -1622,11 +1647,8 @@ public class BaseHashMap {
 
                 lookup = nextLookup(lookup);
 
-                if (keys) {
-                    return longKeyTable[lookup];
-                } else {
-                    return longValueTable[lookup];
-                }
+                return keys ? longKeyTable[lookup]
+                            : longValueTable[lookup];
             }
 
             throw new NoSuchElementException("Hash Iterator");
