@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -48,11 +48,11 @@ import java.util.Arrays;
  * of linkTable which is the first element in the list of reclaimed nodes
  * (nodes no longer in index) or -1 if there is no such node.
  *
- * elemenet at and above linkTable[newNodePointer] have never been used
+ * elements at and above linkTable[newNodePointer] have never been used
  * as a node and their contents is not significant.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.4
  * @since 1.7.2
  */
 public class HashIndex {
@@ -137,6 +137,13 @@ public class HashIndex {
         return hashTable[index];
     }
 
+    /**
+     * Return the pointer
+     */
+    public int getNewNodePointer() {
+        return newNodePointer;
+    }
+
     /**
      * This looks from a given node, so the parameter is always > -1.
      *
