@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -63,7 +63,7 @@ import org.hsqldb.Session;
  * Rewritten for 1.8.0 and 2.x
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.5
  * @since 1.7.2
  */
 public class DataFileCache {
@@ -96,7 +96,9 @@ public class DataFileCache {
     protected Database database;
     protected boolean  logEvents = true;
 
-    // this flag is used externally to determine if a backup is required
+    /**
+     * this flag is used externally to determine if a backup is required
+     */
     protected boolean fileModified;
     protected boolean cacheModified;
     protected int     dataFileScale;
@@ -1178,7 +1180,7 @@ public class DataFileCache {
 
             HsqlException ex = Error.error(ErrorCode.DATA_FILE_ERROR, t);
 
-            if (rowIn.getPos() != pos) {
+            if (rowIn.getFilePosition() != pos) {
                 rowIn.resetRow(pos, 0);
             }
 
@@ -1652,7 +1654,7 @@ public class DataFileCache {
         }
     }
 
-    void logSevereEvent(String message, Throwable t) {
+    public void logSevereEvent(String message, Throwable t) {
 
         if (logEvents) {
             database.logger.logSevereEvent(message, t);
