@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -46,7 +46,7 @@ import org.hsqldb.lib.StringUtil;
  * Manages a .properties file for a database.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.4.0
  * @since 1.7.0
  */
 public class HsqlDatabaseProperties extends HsqlProperties {
@@ -148,13 +148,26 @@ public class HsqlDatabaseProperties extends HsqlProperties {
 
     // versions
     public static final String VERSION_STRING_1_8_0 = "1.8.0";
-    public static final String THIS_VERSION         = "2.3.4";
-    public static final String THIS_FULL_VERSION    = "2.3.4";
-    public static final String THIS_CACHE_VERSION   = "2.0.0";
     public static final String PRODUCT_NAME         = "HSQL Database Engine";
+
+//#ifdef JAVA8
+/*
+
+    public static final String THIS_VERSION         = "2.4.0";
+    public static final String THIS_FULL_VERSION    = "2.4.0";
     public static final int    MAJOR                = 2,
-                               MINOR                = 3,
-                               REVISION             = 4;
+                               MINOR                = 4,
+                               REVISION             = 0;
+*/
+
+//#else
+    public static final String THIS_VERSION      = "2.3.5";
+    public static final String THIS_FULL_VERSION = "2.3.5";
+    public static final int    MAJOR             = 2,
+                               MINOR             = 3,
+                               REVISION          = 5;
+
+//#endif JAVA8
 
     /**
      * system properties supported by HSQLDB
@@ -248,34 +261,36 @@ public class HsqlDatabaseProperties extends HsqlProperties {
         "jdbc.translate_tti_types";
 
     //
+    public static final String sql_restrict_exec       = "sql.restrict_exec";
     public static final String sql_ref_integrity       = "sql.ref_integrity";
     public static final String sql_compare_in_locale = "sql.compare_in_locale";
     public static final String sql_enforce_size        = "sql.enforce_size";
     public static final String sql_enforce_strict_size =
         "sql.enforce_strict_size";    // synonym for sql_enforce_size
-    public static final String sql_enforce_refs   = "sql.enforce_refs";
-    public static final String sql_enforce_names  = "sql.enforce_names";
-    public static final String sql_regular_names  = "sql.regular_names";
-    public static final String sql_enforce_types  = "sql.enforce_types";
-    public static final String sql_enforce_tdcd   = "sql.enforce_tdc_delete";
-    public static final String sql_enforce_tdcu   = "sql.enforce_tdc_update";
-    public static final String sql_char_literal   = "sql.char_literal";
-    public static final String sql_concat_nulls   = "sql.concat_nulls";
-    public static final String sql_nulls_first    = "sql.nulls_first";
-    public static final String sql_nulls_order    = "sql.nulls_order";
-    public static final String sql_unique_nulls   = "sql.unique_nulls";
-    public static final String sql_convert_trunc  = "sql.convert_trunc";
-    public static final String sql_avg_scale      = "sql.avg_scale";
-    public static final String sql_double_nan     = "sql.double_nan";
-    public static final String sql_syntax_db2     = "sql.syntax_db2";
-    public static final String sql_syntax_mss     = "sql.syntax_mss";
-    public static final String sql_syntax_mys     = "sql.syntax_mys";
-    public static final String sql_syntax_ora     = "sql.syntax_ora";
-    public static final String sql_syntax_pgs     = "sql.syntax_pgs";
-    public static final String sql_longvar_is_lob = "sql.longvar_is_lob";
-    public static final String sql_pad_space      = "sql.pad_space";
-    public static final String sql_ignore_case    = "sql.ignore_case";
-    public static final String sql_live_object    = "sql.live_object";
+    public static final String sql_enforce_refs    = "sql.enforce_refs";
+    public static final String sql_enforce_names   = "sql.enforce_names";
+    public static final String sql_regular_names   = "sql.regular_names";
+    public static final String sql_enforce_types   = "sql.enforce_types";
+    public static final String sql_enforce_tdcd    = "sql.enforce_tdc_delete";
+    public static final String sql_enforce_tdcu    = "sql.enforce_tdc_update";
+    public static final String sql_char_literal    = "sql.char_literal";
+    public static final String sql_concat_nulls    = "sql.concat_nulls";
+    public static final String sql_nulls_first     = "sql.nulls_first";
+    public static final String sql_nulls_order     = "sql.nulls_order";
+    public static final String sql_unique_nulls    = "sql.unique_nulls";
+    public static final String sql_convert_trunc   = "sql.convert_trunc";
+    public static final String sql_avg_scale       = "sql.avg_scale";
+    public static final String sql_double_nan      = "sql.double_nan";
+    public static final String sql_syntax_db2      = "sql.syntax_db2";
+    public static final String sql_syntax_mss      = "sql.syntax_mss";
+    public static final String sql_syntax_mys      = "sql.syntax_mys";
+    public static final String sql_syntax_ora      = "sql.syntax_ora";
+    public static final String sql_syntax_pgs      = "sql.syntax_pgs";
+    public static final String sql_longvar_is_lob  = "sql.longvar_is_lob";
+    public static final String sql_pad_space       = "sql.pad_space";
+    public static final String sql_ignore_case     = "sql.ignore_case";
+    public static final String sql_live_object     = "sql.live_object";
+    public static final String sql_sys_index_names = "sql.sys_index_names";
 
     //
     public static final String textdb_cache_scale = "textdb.cache_scale";
@@ -399,6 +414,9 @@ public class HsqlDatabaseProperties extends HsqlProperties {
         dbMeta.put(sql_ref_integrity,
                    HsqlProperties.getMeta(sql_ref_integrity, SQL_PROPERTY,
                                           true));
+        dbMeta.put(sql_restrict_exec,
+                   HsqlProperties.getMeta(sql_restrict_exec, SQL_PROPERTY,
+                                          false));
 
         // SQL reserved words not allowed as some identifiers
         dbMeta.put(sql_enforce_names,
@@ -542,9 +560,13 @@ public class HsqlDatabaseProperties extends HsqlProperties {
         dbMeta.put(hsqldb_nio_max_size,
                    HsqlProperties.getMeta(hsqldb_nio_max_size, SQL_PROPERTY,
                                           256, 64, 262144));
+        dbMeta.put(sql_sys_index_names,
+                   HsqlProperties.getMeta(sql_sys_index_names, SQL_PROPERTY,
+                                          false));
         dbMeta.put(hsqldb_min_reuse,
                    HsqlProperties.getMeta(hsqldb_min_reuse, SQL_PROPERTY, 0,
                                           0, 1024 * 1024));
+
     }
 
     private Database database;
