@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -77,7 +77,7 @@ import org.hsqldb.scriptio.ScriptWriterText;
 import org.hsqldb.types.RowType;
 import org.hsqldb.types.Type;
 
-// boucherb@users 20030510 - patch 1.7.2 - added cooperative file locking
+// campbell-burnet@users 20030510 - patch 1.7.2 - added cooperative file locking
 
 /**
  *  The public interface of persistence and logging classes.<p>
@@ -88,7 +88,7 @@ import org.hsqldb.types.Type;
  *  storage.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.3.5
  * @since 1.7.0
  */
 public class Logger implements EventLogInterface {
@@ -395,7 +395,7 @@ public class Logger implements EventLogInterface {
             checkpoint(null, false, false);
         }
 
-        if (database.getNameString() == null) {
+        if (database.getNameString().isEmpty()) {
             database.setDatabaseName(newUniqueName());
         }
 
@@ -546,6 +546,8 @@ public class Logger implements EventLogInterface {
         database.txConflictRollback =
             database.databaseProperties.isPropertyTrue(
                 HsqlDatabaseProperties.hsqldb_tx_conflict_rollback);
+        database.sqlRestrictExec = database.databaseProperties.isPropertyTrue(
+            HsqlDatabaseProperties.sql_restrict_exec);
         database.sqlEnforceNames = database.databaseProperties.isPropertyTrue(
             HsqlDatabaseProperties.sql_enforce_names);
         database.sqlRegularNames = database.databaseProperties.isPropertyTrue(
@@ -595,6 +597,8 @@ public class Logger implements EventLogInterface {
             HsqlDatabaseProperties.sql_syntax_ora);
         database.sqlSyntaxPgs = database.databaseProperties.isPropertyTrue(
             HsqlDatabaseProperties.sql_syntax_pgs);
+        database.sqlSysIndexNames = database.databaseProperties.isPropertyTrue(
+            HsqlDatabaseProperties.sql_sys_index_names);
 
         if (database.databaseProperties.isPropertyTrue(
                 HsqlDatabaseProperties.sql_compare_in_locale)) {
@@ -671,7 +675,7 @@ public class Logger implements EventLogInterface {
                 HsqlDatabaseProperties.hsqldb_min_reuse));
     }
 
-// fredt@users 20020130 - patch 495484 by boucherb@users
+// fredt@users 20020130 - patch 495484 by campbell-burnet@users
 
     /**
      *  Shuts down the logging process using the specified mode. <p>
@@ -796,7 +800,7 @@ public class Logger implements EventLogInterface {
 
         String name = database.getNameString();
 
-        if (name == null) {
+        if (name.isEmpty()) {
 
             // The database unique name is set up at different times
             // depending on upgraded / exiting / new databases.
@@ -1456,7 +1460,7 @@ public class Logger implements EventLogInterface {
                 return new RowStoreAVLHybridExtended(session, table, true);
 
             case TableBase.CHANGE_SET_TABLE :
-                return new RowStoreDataChange(session, collection, table);
+                return new RowStoreDataChange(session, table);
 
             case TableBase.FUNCTION_TABLE :
             case TableBase.RESULT_TABLE :
@@ -1658,6 +1662,10 @@ public class Logger implements EventLogInterface {
             return database.granteeManager.getDigestAlgo();
         }
 
+        if (HsqlDatabaseProperties.sql_restrict_exec.equals(name)) {
+            return String.valueOf(database.sqlRestrictExec);
+        }
+
         if (HsqlDatabaseProperties.sql_avg_scale.equals(name)) {
             return String.valueOf(database.sqlAvgScale);
         }
@@ -1762,6 +1770,10 @@ public class Logger implements EventLogInterface {
             return String.valueOf(this.propMinReuse);
         }
 
+        if (HsqlDatabaseProperties.sql_sys_index_names.equals(name)) {
+            return String.valueOf(database.sqlSysIndexNames);
+        }
+
 /*
         if (HsqlDatabaseProperties.textdb_all_quoted.equals(name)) {
             return null;
@@ -1898,7 +1910,12 @@ public class Logger implements EventLogInterface {
 
         if (database.schemaManager.getDefaultTableType()
                 == TableBase.CACHED_TABLE) {
-            list.add("SET DATABASE DEFAULT TABLE TYPE CACHED");
+            sb.append("SET DATABASE ").append(Tokens.T_DEFAULT).append(' ');
+            sb.append(Tokens.T_TABLE).append(' ');
+            sb.append(Tokens.T_TYPE).append(' ');
+            sb.append(Tokens.T_CACHED);
+            list.add(sb.toString());
+            sb.setLength(0);
         }
 
         //
@@ -1909,6 +1926,16 @@ public class Logger implements EventLogInterface {
         list.add(sb.toString());
         sb.setLength(0);
 
+        if (database.sqlRestrictExec) {
+            sb.append("SET DATABASE ").append(Tokens.T_SQL).append(' ');
+            sb.append(Tokens.T_RESTRICT).append(' ');
+            sb.append(Tokens.T_EXEC).append(' ');
+            sb.append(database.sqlRestrictExec ? Tokens.T_TRUE
+                                               : Tokens.T_FALSE);
+            list.add(sb.toString());
+            sb.setLength(0);
+        }
+
         if (!database.sqlRegularNames) {
             sb.append("SET DATABASE ").append(Tokens.T_SQL).append(' ');
             sb.append(Tokens.T_REGULAR).append(' ');
@@ -1950,24 +1977,37 @@ public class Logger implements EventLogInterface {
         sb.append(database.sqlEnforceTDCU ? Tokens.T_TRUE
                                           : Tokens.T_FALSE);
         list.add(sb.toString());
-        sb.setLength(0);
-        sb.append("SET DATABASE ").append(Tokens.T_SQL).append(' ');
-        sb.append(Tokens.T_TRANSLATE).append(' ').append(Tokens.T_TTI);
-        sb.append(' ').append(Tokens.T_TYPES).append(' ');
-        sb.append(database.sqlTranslateTTI ? Tokens.T_TRUE
-                                           : Tokens.T_FALSE);
-        if (!database.sqlCharLiteral) {
+
+        if (!database.sqlTranslateTTI) {
+            sb.setLength(0);
+            sb.append("SET DATABASE ").append(Tokens.T_SQL).append(' ');
+            sb.append(Tokens.T_TRANSLATE).append(' ').append(Tokens.T_TTI);
+            sb.append(' ').append(Tokens.T_TYPES).append(' ');
+            sb.append(database.sqlTranslateTTI ? Tokens.T_TRUE
+                                               : Tokens.T_FALSE);
             list.add(sb.toString());
+        }
+
+        if (database.sqlSysIndexNames) {
+            sb.setLength(0);
+            sb.append("SET DATABASE ").append(Tokens.T_SQL).append(' ');
+            sb.append(Tokens.T_SYS).append(' ').append(Tokens.T_INDEX);
+            sb.append(' ').append(Tokens.T_NAMES).append(' ');
+            sb.append(database.sqlSysIndexNames ? Tokens.T_TRUE
+                                                : Tokens.T_FALSE);
+            list.add(sb.toString());
+        }
+
+        if (!database.sqlCharLiteral) {
             sb.setLength(0);
             sb.append("SET DATABASE ").append(Tokens.T_SQL).append(' ');
             sb.append(Tokens.T_CHARACTER).append(' ');
             sb.append(Tokens.T_LITERAL).append(' ');
             sb.append(database.sqlCharLiteral ? Tokens.T_TRUE
-                      : Tokens.T_FALSE);
+                                              : Tokens.T_FALSE);
+            list.add(sb.toString());
         }
 
-        list.add(sb.toString());
-        list.add(sb.toString());
         sb.setLength(0);
         sb.append("SET DATABASE ").append(Tokens.T_SQL).append(' ');
         sb.append(Tokens.T_CONCAT_WORD).append(' ');
@@ -2192,13 +2232,11 @@ public class Logger implements EventLogInterface {
             sb.setLength(0);
         }
 
-        {
-            if (propFileSpaceValue != 0) {
-                sb.append("SET FILES ").append(Tokens.T_SPACE).append(' ');
-                sb.append(propFileSpaceValue);
-                list.add(sb.toString());
-                sb.setLength(0);
-            }
+        if (propFileSpaceValue != 0) {
+            sb.append("SET FILES ").append(Tokens.T_SPACE).append(' ');
+            sb.append(propFileSpaceValue);
+            list.add(sb.toString());
+            sb.setLength(0);
         }
 
         String[] array = new String[list.size()];
@@ -2366,6 +2404,8 @@ public class Logger implements EventLogInterface {
                             dataFileCache.getShadowFile();
 
                         if (shadowFile == null) {
+
+                            // non-incremental backup - ignore .data file
                             backup.setFileIgnore(dataFileExtension);
                         } else {
                             file = new File(dataFileCache.dataFileName);
