@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -66,7 +66,7 @@ final class RAFile implements RandomAccessInterface {
     //
     static final int  bufferScale = 12;
     static final int  bufferSize  = 1 << bufferScale;
-    static final long bufferMask  = 0xffffffffffffffffl << bufferScale;
+    static final long bufferMask  = 0xffffffffffffffffL << bufferScale;
 
     //
     final EventLogInterface         logger;
