@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -58,7 +58,7 @@ import org.hsqldb.rowio.RowOutputInterface;
  * Implementation of PersistentStore for CACHED tables.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.5
  * @since 1.9.0
  */
 public class RowStoreAVLDisk extends RowStoreAVL implements PersistentStore {
@@ -148,9 +148,9 @@ public class RowStoreAVLDisk extends RowStoreAVL implements PersistentStore {
 
         try {
             if (largeData) {
-                return new RowAVLDiskLarge(table, in);
+                return new RowAVLDiskLarge(this, in);
             } else {
-                return new RowAVLDisk(table, in);
+                return new RowAVLDisk(this, in);
             }
         } catch (IOException e) {
             throw Error.error(ErrorCode.DATA_FILE_ERROR, e);
@@ -299,16 +299,17 @@ public class RowStoreAVLDisk extends RowStoreAVL implements PersistentStore {
                 break;
 
             case RowAction.ACTION_INSERT_DELETE :
-                if (txModel != TransactionManager.LOCKS) {
+                if (txModel == TransactionManager.LOCKS) {
+                    remove(row);
+                } else {
 
                     // INSERT + DELETE
                     delete(session, row);
 
                     // remove info after delete but before removing persistence
                     database.txManager.removeTransactionInfo(row);
+                    remove(row);
                 }
-
-                remove(row);
                 break;
         }
     }
@@ -336,7 +337,13 @@ public class RowStoreAVLDisk extends RowStoreAVL implements PersistentStore {
 
     public CachedObject getAccessor(Index key) {
 
-        NodeAVL node = (NodeAVL) accessorList[key.getPosition()];
+        int position = key.getPosition();
+
+        if (position >= accessorList.length) {
+            throw Error.runtimeError(ErrorCode.U_S0500, "RowStoreAVL");
+        }
+
+        NodeAVL node = (NodeAVL) accessorList[position];
 
         if (node == null) {
             return null;
@@ -413,8 +420,8 @@ public class RowStoreAVLDisk extends RowStoreAVL implements PersistentStore {
             RowIterator it = rowIterator();
 
             // todo - check this - must remove from old space, not new one
-            while (it.hasNext()) {
-                Row row = it.getNextRow();
+            while (it.next()) {
+                Row row = it.getCurrentRow();
 
                 cache.remove(row);
                 tableSpace.release(row.getPos(), row.getStorageSize());
@@ -440,8 +447,8 @@ public class RowStoreAVLDisk extends RowStoreAVL implements PersistentStore {
 
         RowIterator it = indexList[0].firstRow(this);
 
-        while (it.hasNext()) {
-            CachedObject row = it.getNextRow();
+        while (it.next()) {
+            CachedObject row = it.getCurrentRow();
 
             pointerLookup.addUnsorted(row.getPos(), row.getStorageSize());
         }
@@ -457,8 +464,8 @@ public class RowStoreAVLDisk extends RowStoreAVL implements PersistentStore {
 
         it = indexList[0].firstRow(this);
 
-        while (it.hasNext()) {
-            CachedObject row    = it.getNextRow();
+        while (it.next()) {
+            CachedObject row    = it.getCurrentRow();
             long         newPos = pointerLookup.lookup(row.getPos());
 
             // write
