@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -54,7 +54,7 @@ import org.hsqldb.rowio.RowInputInterface;
  * Implementation of PersistentStore for result sets.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.5
  * @since 1.9.0
  */
 public class RowStoreAVLHybrid extends RowStoreAVL implements PersistentStore {
@@ -174,7 +174,7 @@ public class RowStoreAVLHybrid extends RowStoreAVL implements PersistentStore {
 
         try {
             if (isCached) {
-                return new RowAVLDisk(table, in);
+                return new RowAVLDisk(this, in);
             }
         } catch (HsqlException e) {
             return null;
@@ -324,7 +324,8 @@ public class RowStoreAVLHybrid extends RowStoreAVL implements PersistentStore {
 
     public CachedObject getAccessor(Index key) {
 
-        NodeAVL node = (NodeAVL) accessorList[key.getPosition()];
+        int     position = key.getPosition();
+        NodeAVL node     = (NodeAVL) accessorList[position];
 
         if (node == null) {
             return null;
@@ -387,14 +388,14 @@ public class RowStoreAVLHybrid extends RowStoreAVL implements PersistentStore {
         ArrayUtil.fillArray(nullsList, false);
         elementCount.set(0);
 
-        while (iterator.hasNext()) {
-            Row row = iterator.getNextRow();
+        while (iterator.next()) {
+            Row row = iterator.getCurrentRow();
             Row newRow = (Row) getNewCachedObject(session, row.getData(),
                                                   false);
 
             indexRow(session, newRow);
         }
 
-        idx.unlinkNodes(root);
+        idx.unlinkNodes(this, root);
     }
 }
