@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2015, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,7 +45,7 @@ import org.hsqldb.navigator.RowIterator;
  * Implementation of PersistentStore for information schema and temp tables.
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.5
  * @since 2.0.1
  */
 public class RowStoreAVLHybridExtended extends RowStoreAVLHybrid {
@@ -62,6 +62,11 @@ public class RowStoreAVLHybridExtended extends RowStoreAVLHybrid {
 
     public CachedObject getNewCachedObject(Session session, Object object,
                                            boolean tx) {
+
+        if (!resettingAccessor && table.getIndexCount() != indexList.length) {
+            resetAccessorKeys(session, table.getIndexList());
+        }
+
         return super.getNewCachedObject(session, object, tx);
     }
 
@@ -86,7 +91,7 @@ public class RowStoreAVLHybridExtended extends RowStoreAVLHybrid {
             node = node.nNext;
         }
 
-        if ((isCached ^ !row.isMemory()) || count != indexList.length) {
+        if (isCached && row.isMemory() || count != indexList.length) {
             row = (Row) getNewCachedObject(session, row.getData(), true);
         }
 
@@ -107,7 +112,7 @@ public class RowStoreAVLHybridExtended extends RowStoreAVLHybrid {
             node = node.nNext;
         }
 
-        if ((isCached ^ !row.isMemory()) || count != indexList.length) {
+        if (isCached && row.isMemory() || count != indexList.length) {
             row = ((Table) table).getDeleteRowFromLog(session, row.getData());
         }
 
@@ -116,52 +121,69 @@ public class RowStoreAVLHybridExtended extends RowStoreAVLHybrid {
         }
     }
 
+    boolean resettingAccessor = false;
+
     public CachedObject getAccessor(Index key) {
+
+        if (!resettingAccessor && key.getPosition() > 0
+                && table.getIndexCount() != indexList.length) {
+            resetAccessorKeys(session, table.getIndexList());
+        }
+
         return super.getAccessor(key);
     }
 
     public synchronized void resetAccessorKeys(Session session, Index[] keys) {
 
-        if (indexList.length == 0 || accessorList[0] == null) {
-            indexList    = keys;
-            accessorList = new CachedObject[indexList.length];
+        resettingAccessor = true;
+        try {
+            if (indexList.length == 0 || accessorList[0] == null) {
+                indexList = keys;
+                accessorList = new CachedObject[indexList.length];
 
-            return;
+                return;
+            }
+
+            boolean result;
+
+            if (isCached) {
+                resetAccessorKeysCached(keys);
+            }
+            else {
+                super.resetAccessorKeys(session, keys);
+            }
+        } finally {
+            resettingAccessor = false;
         }
 
-        if (isCached) {
-            resetAccessorKeysForCached(keys);
-
-            return;
-        }
-
-        super.resetAccessorKeys(session, keys);
     }
 
-    private void resetAccessorKeysForCached(Index[] keys) {
-
-        TableBase newTable = table.duplicate();
-
-        newTable.persistenceId = table.persistenceId;
-
-        newTable.setIndexes(keys);
+    private void resetAccessorKeysCached(Index[] keys) {
 
         RowStoreAVLHybrid tempStore = new RowStoreAVLHybridExtended(session,
-            newTable, true);
+            table, true);
 
         tempStore.changeToDiskTable(session);
 
-        RowIterator iterator = rowIterator();
+        tempStore.indexList    = indexList;
+        tempStore.accessorList = accessorList;
 
-        while (iterator.hasNext()) {
-            Row row = iterator.getNextRow();
-            Row newRow = (Row) tempStore.getNewCachedObject(session,
-                row.getData(), false);
-
-            tempStore.indexRow(session, newRow);
-        }
+        tempStore.elementCount.set(elementCount.get());
 
+        //
         indexList    = keys;
-        accessorList = tempStore.accessorList;
+        accessorList = new CachedObject[indexList.length];
+
+        elementCount.set(0);
+
+        RowIterator iterator = tempStore.rowIterator();
+
+        while (iterator.next()) {
+            Row row = iterator.getCurrentRow();
+            Row newRow = (Row) getNewCachedObject(session, row.getData(),
+                                                  false);
+
+            indexRow(session, newRow);
+        }
     }
 }
