@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2014, The HSQL Development Group
+/* Copyright (c) 2001-2016, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -633,7 +633,7 @@ public final class Right {
         }
     }
 
-    public boolean canAcesssNonSelect() {
+    public boolean canAccesssNonSelect() {
 
         if (isFull) {
             return true;
@@ -982,6 +982,8 @@ public final class Right {
 
                 triggerColumnSet = set;
                 break;
+
+            default:
         }
     }
 
