@@ -57,7 +57,7 @@ import org.hsqldb.types.Types;
  *
  * @author Bob Preston (sqlbob@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.5
  * @since 1.7.0
  */
 abstract class RowOutputBase extends HsqlByteArrayOutputStream
@@ -96,6 +96,8 @@ implements RowOutputInterface {
         super(buffer);
     }
 
+    public abstract long scalePosition(long position);
+
     public abstract void setStorageSize(int size);
 
 // fredt@users - comment - methods for writing Result column type, name and data size
@@ -144,6 +146,8 @@ implements RowOutputInterface {
 
     protected abstract void writeBit(BinaryData o);
 
+    protected abstract void writeUUID(BinaryData o);
+
     protected abstract void writeBinary(BinaryData o);
 
     protected abstract void writeClob(ClobData o, Type type);
@@ -284,6 +288,9 @@ implements RowOutputInterface {
                 break;
 
             case Types.SQL_GUID :
+                writeUUID((BinaryData) o);
+                break;
+
             case Types.SQL_BINARY :
             case Types.SQL_VARBINARY :
                 writeBinary((BinaryData) o);
