@@ -56,7 +56,7 @@ import org.hsqldb.types.Types;
  *
  * @author Bob Preston (sqlbob@users dot sourceforge.net)
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.3.5
  * @since 1.7.0
  */
 public class RowOutputBinary extends RowOutputBase {
@@ -112,6 +112,10 @@ public class RowOutputBinary extends RowOutputBase {
         super.writeData(row, types);
     }
 
+    public long scalePosition(long position) {
+        return (position + scale - 1) & (long) mask;
+    }
+
     public void setStorageSize(int size) {
         storageSize = size;
     }
@@ -261,6 +265,10 @@ public class RowOutputBinary extends RowOutputBase {
         write(o.getBytes(), 0, o.getBytes().length);
     }
 
+    protected void writeUUID(BinaryData o) {
+        writeByteArray(o.getBytes());
+    }
+
     protected void writeBinary(BinaryData o) {
         writeByteArray(o.getBytes());
     }
@@ -300,6 +308,10 @@ public class RowOutputBinary extends RowOutputBase {
         write(b, 0, b.length);
     }
 
+    public void writeByteArray16(byte[] b) {
+        write(b, 0, 16);
+    }
+
     // fredt@users - comment - helper and conversion methods
     public void writeCharArray(char[] c) {
         writeInt(c.length);
