@@ -102,6 +102,10 @@ public class RowOutputText extends RowOutputBase {
 
     public void setStorageSize(int size) {}
 
+    public long scalePosition(long pos) {
+        return pos;
+    }
+
     public void writeEnd() {
 
         // terminate at the end of row
@@ -352,6 +356,13 @@ public class RowOutputText extends RowOutputBase {
         writeString(s);
     }
 
+    protected void writeUUID(BinaryData o) {
+
+        String s = StringConverter.toStringUUID(o.getBytes());
+
+        writeString(s);
+    }
+
     protected void writeBinary(BinaryData o) {
         writeByteArray(o.getBytes());
     }
