@@ -269,7 +269,7 @@ class ServerConnection implements Runnable {
             socket.setTcpNoDelay(true);
 
             dataInput = new DataInputStream(
-                new BufferedInputStream(socket.getInputStream()));
+                new BufferedInputStream(socket.getInputStream(), 16 * 1024));
             dataOutput = new DataOutputStream(socket.getOutputStream());
 
             int firstInt = handshake();
@@ -290,7 +290,7 @@ class ServerConnection implements Runnable {
                         throw Error.error(
                             null, ErrorCode.SERVER_VERSIONS_INCOMPATIBLE, 0,
                             new String[] {
-                            verString, HsqlDatabaseProperties.THIS_VERSION
+                            HsqlDatabaseProperties.THIS_VERSION, verString
                         });
                     }
 
