@@ -485,7 +485,7 @@ class TestBench {
 
             Query =
                 "CREATE PROCEDURE UPDATE_PROC(IN paid INT, IN ptid INT, IN pbid INT, IN pdelta INT, OUT pbalance INT) "
-                + "MODIFIES SQL DATA BEGIN ATOMIC "
+                + "MODIFIES SQL DATA DYNAMIC RESULT SETS 1 BEGIN ATOMIC "
                 + "DECLARE account CURSOR WITH RETURN FOR SELECT Abalance FROM accounts WHERE  Aid = paid;"
                 + "UPDATE accounts SET Abalance = abalance + pdelta WHERE  aid = paid;"
                 + "OPEN account;"
@@ -635,27 +635,24 @@ class TestBench {
         return ret;
     }
 
+    /**
+     * changed to generate correct own branch for each account id
+     */
     public static int getRandomID(int type) {
 
-        int min = 0,
-            max = 0;
+        int min     = 0,
+            max     = naccounts * tps - 1;
+        int account = getRandomInt(min, max);
 
         switch (type) {
 
             case TELLER :
                 max = ntellers * tps - 1;
-                break;
 
-            case BRANCH :
-                max = nbranches * tps - 1;
-                break;
-
-            case ACCOUNT :
-                max = naccounts * tps - 1;
-                break;
+                return getRandomInt(min, max);
         }
 
-        return (getRandomInt(min, max));
+        return account;
     }
 
     public static Connection connect(String DBUrl, String DBUser,
@@ -817,7 +814,7 @@ class TestBench {
 
             while (count-- > 0) {
                 int account = TestBench.getRandomID(ACCOUNT);
-                int branch  = TestBench.getRandomID(BRANCH);
+                int branch  = account / naccounts;
                 int teller  = TestBench.getRandomID(TELLER);
                 int delta   = TestBench.getRandomInt(-1000, 1000);
 
@@ -898,6 +895,7 @@ class TestBench {
                     aBalance = RS.getInt(1);
                 }
 
+                RS.close();
                 pstmt3.setInt(1, delta);
                 pstmt3.setInt(2, tid);
                 pstmt3.executeUpdate();
@@ -976,7 +974,7 @@ class TestBench {
 
             while (count-- > 0) {
                 int account = TestBench.getRandomID(ACCOUNT);
-                int branch  = TestBench.getRandomID(BRANCH);
+                int branch  = account / naccounts;
                 int teller  = TestBench.getRandomID(TELLER);
                 int delta   = TestBench.getRandomInt(-1000, 1000);
 
@@ -1017,14 +1015,18 @@ class TestBench {
                 pstmt1.setInt(3, bid);
                 pstmt1.setInt(4, delta);
                 pstmt1.execute();
+                pstmt1.getUpdateCount();
 
-                ResultSet rs = pstmt1.getResultSet();
+                if (pstmt1.getMoreResults()) {
+                    ResultSet rs = pstmt1.getResultSet();
 
-                while (rs.next()) {
-                    aBalance = rs.getInt(1);
+                    while (rs.next()) {
+                        aBalance = rs.getInt(1);
+                    }
+
+                    rs.close();
                 }
 
-                rs.close();
                 pstmt1.clearWarnings();
                 Conn.commit();
 
