@@ -51,7 +51,6 @@ import org.hsqldb.jdbc.JDBCBlob;
 import org.hsqldb.jdbc.JDBCClob;
 import org.hsqldb.lib.HsqlByteArrayInputStream;
 import org.hsqldb.lib.StopWatch;
-import org.hsqldb.persist.HsqlDatabaseProperties;
 
 public class TestLobs extends TestBase {
 
@@ -61,9 +60,8 @@ public class TestLobs extends TestBase {
     public TestLobs(String name) {
 
 //        super(name);
-        super(name, "jdbc:hsqldb:file:test3", false, false);
-
-//        super(name, "jdbc:hsqldb:mem:test3", false, false);
+//        super(name, "jdbc:hsqldb:file:test3", false, false);
+        super(name, "jdbc:hsqldb:mem:test3", false, false);
     }
 
     protected void setUp() throws Exception {
@@ -933,8 +931,9 @@ public class TestLobs extends TestBase {
 
             while (result.next()) {
                 byte[] data = result.getBytes(1);
-                String s = data == null ? "null"
-                                        : String.valueOf(data.length);
+                String s    = data == null ? "null"
+                                           : String.valueOf(data.length);
+
                 System.out.println("Result: " + s);
             }
         } catch (SQLException e) {
