@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,7 @@ import org.hsqldb.lib.StringConverter;
  * Type subclass for DATE, TIME and TIMESTAMP.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.4
+ * @version 2.4.0
  * @since 1.9.0
  */
 public final class DateTimeType extends DTIType {
@@ -103,8 +103,19 @@ public final class DateTimeType extends DTIType {
 
     public int getJDBCTypeCode() {
 
-        // JDBC numbers happen to be the same as SQL
-        return typeCode;
+        switch (typeCode) {
+
+            case Types.SQL_TIME_WITH_TIME_ZONE :
+                return Types.TIME_WITH_TIMEZONE;
+
+            case Types.SQL_TIMESTAMP_WITH_TIME_ZONE :
+                return Types.TIMESTAMP_WITH_TIMEZONE;
+
+            default :
+
+                // JDBC numbers happen to be the same as SQL
+                return typeCode;
+        }
     }
 
     public Class getJDBCClass() {
@@ -115,12 +126,27 @@ public final class DateTimeType extends DTIType {
                 return java.sql.Date.class;
 
             case Types.SQL_TIME :
-            case Types.SQL_TIME_WITH_TIME_ZONE :
                 return java.sql.Time.class;
 
             case Types.SQL_TIMESTAMP :
+                return java.sql.Timestamp.class;
+
+
+//#ifdef JAVA8
+/*
+            case Types.SQL_TIME_WITH_TIME_ZONE :
+                return java.time.OffsetTime.class;
+
+            case Types.SQL_TIMESTAMP_WITH_TIME_ZONE :
+                return java.time.OffsetDateTime.class;
+*/
+//#else
+            case Types.SQL_TIME_WITH_TIME_ZONE :
+                return java.sql.Time.class;
+
             case Types.SQL_TIMESTAMP_WITH_TIME_ZONE :
                 return java.sql.Timestamp.class;
+//#endif JAVA8
 
             default :
                 throw Error.runtimeError(ErrorCode.U_S0500, "DateTimeType");
@@ -135,12 +161,26 @@ public final class DateTimeType extends DTIType {
                 return "java.sql.Date";
 
             case Types.SQL_TIME :
-            case Types.SQL_TIME_WITH_TIME_ZONE :
                 return "java.sql.Time";
 
             case Types.SQL_TIMESTAMP :
+                return "java.sql.Timestamp";
+
+//#ifdef JAVA8
+/*
+            case Types.SQL_TIME_WITH_TIME_ZONE :
+                return "java.time.OffsetTime";
+
+            case Types.SQL_TIMESTAMP_WITH_TIME_ZONE :
+                return "java.time.OffsetDateTime";
+*/
+//#else
+            case Types.SQL_TIME_WITH_TIME_ZONE :
+                return "java.sql.Time";
+
             case Types.SQL_TIMESTAMP_WITH_TIME_ZONE :
                 return "java.sql.Timestamp";
+//#endif JAVA8
 
             default :
                 throw Error.runtimeError(ErrorCode.U_S0500, "DateTimeType");
@@ -257,6 +297,19 @@ public final class DateTimeType extends DTIType {
         }
     }
 
+    public boolean isTimestampType() {
+
+        switch (typeCode) {
+
+            case Types.SQL_TIMESTAMP :
+            case Types.SQL_TIMESTAMP_WITH_TIME_ZONE :
+                return true;
+
+            default :
+                throw Error.runtimeError(ErrorCode.U_S0500, "DateTimeType");
+        }
+    }
+
     public boolean isDateTimeTypeWithZone() {
         return withTimeZone;
     }
@@ -560,10 +613,10 @@ public final class DateTimeType extends DTIType {
                     case Types.SQL_TIMESTAMP_WITH_TIME_ZONE : {
                         long seconds = ((TimestampData) a).getSeconds()
                                        + ((TimestampData) a).getZone();
-                        long l = HsqlDateTime.getNormalisedDate(
+                        long millis = HsqlDateTime.getNormalisedDate(
                             session.getCalendarGMT(), seconds * 1000);
 
-                        return new TimestampData(l / 1000);
+                        return new TimestampData(millis / 1000);
                     }
                     case Types.SQL_TIMESTAMP : {
                         long l = HsqlDateTime.getNormalisedDate(
@@ -786,6 +839,12 @@ public final class DateTimeType extends DTIType {
                     return new TimeData((int) millis / 1000, nanos,
                                         zoneSeconds);
                 }
+
+                TimeData time = convertJavaTimeObject(session, a);
+
+                if (time != null) {
+                    return time;
+                }
                 break;
 
             case Types.SQL_DATE : {
@@ -811,6 +870,13 @@ public final class DateTimeType extends DTIType {
                     return new TimestampData(seconds);
                 }
 
+                TimestampData timestamp = convertJavaTimeObject(session, a,
+                    false);
+
+                if (timestamp != null) {
+                    return timestamp;
+                }
+
                 break;
             }
             case Types.SQL_TIMESTAMP :
@@ -850,6 +916,13 @@ public final class DateTimeType extends DTIType {
                     return new TimestampData(seconds, nanos, zoneSeconds);
                 }
 
+                TimestampData timestamp = convertJavaTimeObject(session, a,
+                    true);
+
+                if (timestamp != null) {
+                    return timestamp;
+                }
+
                 break;
             }
         }
@@ -857,6 +930,135 @@ public final class DateTimeType extends DTIType {
         throw Error.error(ErrorCode.X_42561);
     }
 
+//#ifdef JAVA8
+/*
+    TimestampData convertJavaTimeObject(SessionInterface session, Object a, boolean timestamp) {
+        if (a instanceof java.time.OffsetDateTime) {
+            java.time.OffsetDateTime odt = (java.time.OffsetDateTime) a;
+            long seconds = odt.toEpochSecond();
+            int nanos = 0;
+            int zoneSeconds = 0;
+
+            if(timestamp) {
+                nanos = odt.getNano();
+                nanos = DateTimeType.normaliseFraction(nanos, scale);
+
+                if(withTimeZone) {
+                    zoneSeconds = odt.get(java.time.temporal.ChronoField.OFFSET_SECONDS);
+                }
+            } else {
+                seconds = HsqlDateTime.getNormalisedDate(session.getCalendarGMT(),seconds * 1000) / 1000;
+            }
+            return new TimestampData(seconds, nanos, zoneSeconds);
+        } else if (a instanceof java.time.LocalDateTime) {
+            java.time.LocalDateTime odt = (java.time.LocalDateTime) a;
+            int year = odt.getYear();
+            int month = odt.getMonthValue() - 1;
+            int day = odt.getDayOfMonth();
+            int hour = 0;
+            int minute = 0;
+            int second = 0;
+            int nanos = 0;
+            int zoneSeconds = 0;
+
+            if(timestamp) {
+                hour = odt.getHour();
+                minute = odt.getMinute();
+                second = odt.getSecond();
+                nanos = odt.getNano();
+                nanos = DateTimeType.normaliseFraction(nanos, scale);
+
+                if(withTimeZone) {
+                    zoneSeconds = session.getZoneSeconds();
+                }
+            }
+
+            Calendar cal = session.getCalendarGMT();
+            cal.clear();
+            cal.set(year, month, day, hour, minute, second);
+            long seconds = cal.getTimeInMillis() / 1000;
+            return new TimestampData(seconds, nanos, zoneSeconds);
+        } else if (a instanceof java.time.LocalDate) {
+            java.time.LocalDate odt = (java.time.LocalDate) a;
+            int year = odt.getYear();
+            int month = odt.getMonthValue() - 1;
+            int day = odt.getDayOfMonth();
+            int zoneSeconds = 0;
+
+            if(timestamp) {
+                if(withTimeZone) {
+                    zoneSeconds = session.getZoneSeconds();
+                }
+            }
+
+            Calendar cal = session.getCalendarGMT();
+            cal.clear();
+
+            cal.set(year, month, day);
+            long milis = cal.getTimeInMillis();
+            return new TimestampData(milis, 0, zoneSeconds);
+        }
+
+        return null;
+    }
+
+    TimeData convertJavaTimeObject(SessionInterface session, Object a) {
+        if (a instanceof java.time.OffsetTime) {
+            java.time.OffsetTime odt = (java.time.OffsetTime) a;
+            int zoneSeconds = 0;
+
+            int hour = odt.getHour();
+            int minute = odt.getMinute();
+            int second = odt.getSecond();
+            int nanos = odt.getNano();
+            nanos = DateTimeType.normaliseFraction(nanos, scale);
+
+            if(withTimeZone) {
+                zoneSeconds = odt.get(java.time.temporal.ChronoField.OFFSET_SECONDS);
+            }
+
+            Calendar cal = session.getCalendarGMT();
+            cal.clear();
+            cal.set(1970, 1, 1, hour, minute, second);
+            int seconds = (int) cal.getTimeInMillis() / 1000;
+            return new TimeData(seconds, nanos, zoneSeconds);
+
+        } else if (a instanceof java.time.LocalTime) {
+            java.time.LocalTime odt = (java.time.LocalTime) a;
+            int zoneSeconds = 0;
+
+            int hour = odt.getHour();
+            int minute = odt.getMinute();
+            int second = odt.getSecond();
+            int nanos = odt.getNano();
+            nanos = DateTimeType.normaliseFraction(nanos, scale);
+
+            if(withTimeZone) {
+                zoneSeconds = session.getZoneSeconds();
+            }
+
+            Calendar cal = session.getCalendarGMT();
+            cal.clear();
+            cal.set(1970, 1, 1, hour, minute, second);
+            int seconds = (int) cal.getTimeInMillis() / 1000;
+            return new TimeData(seconds, nanos, zoneSeconds);
+        }
+
+        return null;
+    }
+*/
+
+//#else
+    TimestampData convertJavaTimeObject(SessionInterface session, Object a,
+                                        boolean timestamp) {
+        return null;
+    }
+
+    TimeData convertJavaTimeObject(SessionInterface session, Object a) {
+        return null;
+    }
+
+//#endif JAVA8
     public Object convertSQLToJavaGMT(SessionInterface session, Object a) {
 
         long millis;
@@ -901,7 +1103,7 @@ public final class DateTimeType extends DTIType {
             case Types.SQL_TIME : {
                 Calendar cal = session.getCalendar();
                 long millis = HsqlDateTime.convertMillisToCalendar(cal,
-                    ((TimeData) a).getSeconds() * 1000L);
+                    ((TimeData) a).getMillis());
 
                 millis = HsqlDateTime.getNormalisedTime(cal, millis);
 
@@ -910,14 +1112,14 @@ public final class DateTimeType extends DTIType {
                 return value;
             }
             case Types.SQL_TIME_WITH_TIME_ZONE : {
-                int seconds = ((TimeData) a).getSeconds();
+                long millis = ((TimeData) a).getMillis();
 
-                return new java.sql.Time(seconds * 1000L);
+                return new java.sql.Time(millis);
             }
             case Types.SQL_DATE : {
                 Calendar cal = session.getCalendar();
                 long millis = HsqlDateTime.convertMillisToCalendar(cal,
-                    ((TimestampData) a).getSeconds() * 1000);
+                    ((TimestampData) a).getMillis());
 
                 // millis = HsqlDateTime.getNormalisedDate(cal, millis);
                 java.sql.Date value = new java.sql.Date(millis);
@@ -927,7 +1129,7 @@ public final class DateTimeType extends DTIType {
             case Types.SQL_TIMESTAMP : {
                 Calendar cal = session.getCalendar();
                 long millis = HsqlDateTime.convertMillisToCalendar(cal,
-                    ((TimestampData) a).getSeconds() * 1000);
+                    ((TimestampData) a).getMillis());
                 java.sql.Timestamp value = new java.sql.Timestamp(millis);
 
                 value.setNanos(((TimestampData) a).getNanos());
@@ -935,9 +1137,8 @@ public final class DateTimeType extends DTIType {
                 return value;
             }
             case Types.SQL_TIMESTAMP_WITH_TIME_ZONE : {
-                long seconds = ((TimestampData) a).getSeconds();
-                java.sql.Timestamp value = new java.sql.Timestamp(seconds
-                    * 1000);
+                long               millis = ((TimestampData) a).getMillis();
+                java.sql.Timestamp value  = new java.sql.Timestamp(millis);
 
                 value.setNanos(((TimestampData) a).getNanos());
 
@@ -1204,6 +1405,8 @@ public final class DateTimeType extends DTIType {
 
             case Types.SQL_TIME_WITH_TIME_ZONE :
                 millis = HsqlDateTime.getNormalisedTime(millis);
+
+            //fall through
             case Types.SQL_TIME : {
                 return new TimeData((int) (millis / 1000), 0,
                                     ((TimeData) a).getZone());
@@ -1335,6 +1538,12 @@ public final class DateTimeType extends DTIType {
                 } else {
                     return ((TimeData) dateTime).getZone() / 60 % 60;
                 }
+            case TIMEZONE :
+                if (typeCode == Types.SQL_TIMESTAMP_WITH_TIME_ZONE) {
+                    return ((TimestampData) dateTime).getZone() / 60;
+                } else {
+                    return ((TimeData) dateTime).getZone() / 60;
+                }
             case QUARTER :
                 increment    = 1;
                 divisor      = 3;
@@ -1345,6 +1554,18 @@ public final class DateTimeType extends DTIType {
                 calendarPart = Calendar.DAY_OF_YEAR;
                 break;
 
+            case MILLISECOND :
+                if (this.isDateOrTimestampType()) {
+                    return ((TimestampData) dateTime).getNanos() / 1000000;
+                } else {
+                    return ((TimeData) dateTime).getNanos() / 1000000;
+                }
+            case NANOSECOND :
+                if (this.isDateOrTimestampType()) {
+                    return ((TimestampData) dateTime).getNanos();
+                } else {
+                    return ((TimeData) dateTime).getNanos();
+                }
             default :
                 throw Error.runtimeError(ErrorCode.U_S0500,
                                          "DateTimeType - " + part);
@@ -1438,9 +1659,11 @@ public final class DateTimeType extends DTIType {
         long seconds = getPart(null, dateTime, Types.SQL_INTERVAL_SECOND);
         int  nanos   = 0;
 
-        if (typeCode == Types.SQL_TIMESTAMP) {
+        if (typeCode == Types.SQL_TIMESTAMP
+                || typeCode == Types.SQL_TIMESTAMP_WITH_TIME_ZONE) {
             nanos = ((TimestampData) dateTime).getNanos();
-        } else if (typeCode == Types.SQL_TIME) {
+        } else if (typeCode == Types.SQL_TIME
+                   || typeCode == Types.SQL_TIME_WITH_TIME_ZONE) {
             nanos = ((TimeData) dateTime).getNanos();
         }
 
@@ -1568,6 +1791,10 @@ public final class DateTimeType extends DTIType {
                     return SQL_TIMESTAMP;
                 }
 
+                if (scale == 0) {
+                    return SQL_TIMESTAMP_NO_FRACTION;
+                }
+
                 return new DateTimeType(Types.SQL_TIMESTAMP, type, scale);
 
             case Types.SQL_TIMESTAMP_WITH_TIME_ZONE :
@@ -2299,24 +2526,42 @@ public final class DateTimeType extends DTIType {
                         && ((Session) session).database.sqlSyntaxOra) {
                     String pattern;
 
-                    if (s.length() == 9) {
-                        pattern = "DD-MON-YY";
-                    } else if (s.length() == 11) {
-                        pattern = "DD-MON-YYYY";
-                    } else if (s.length() == 20) {
-                        pattern = "DD-MON-YYYY HH24:MI:SS";
-                    } else if (s.length() > 20) {
-                        pattern = "DD-MON-YYYY HH24:MI:SS.FF";
-                    } else {
-                        break;
+                    switch (s.length()) {
+
+                        case 8 :
+                        case 9 : {
+                            pattern = "DD-MON-YY";
+
+                            break;
+                        }
+                        case 10 :
+                        case 11 : {
+                            pattern = "DD-MON-YYYY";
+
+                            break;
+                        }
+                        case 19 :
+                        case 20 : {
+                            pattern = "DD-MON-YYYY HH24:MI:SS";
+
+                            break;
+                        }
+                        default :
+
+                        // if (s.length() > 20)
+                        {
+                            pattern = "DD-MON-YYYY HH24:MI:SS.FF";
+
+                            break;
+                        }
                     }
 
                     SimpleDateFormat format = session.getSimpleDateFormatGMT();
 
-                    return HsqlDateTime.toDate(s, pattern, format);
+                    return HsqlDateTime.toDate(s, pattern, format, true);
                 }
 
-            //
+            // fall through
             case Types.SQL_TIMESTAMP_WITH_TIME_ZONE :
 
             //
@@ -2328,4 +2573,63 @@ public final class DateTimeType extends DTIType {
 
         throw Error.error(ErrorCode.X_22007);
     }
+
+    public static TimestampData nextDayOfWeek(Session session,
+            TimestampData d, int day) {
+
+        Calendar cal = session.getCalendarGMT();
+
+        cal.setTimeInMillis(d.getMillis());
+
+        int start = cal.get(Calendar.DAY_OF_WEEK);
+
+        if (start >= day) {
+            day += 7;
+        }
+
+        int diff = day - start;
+
+        cal.add(Calendar.DAY_OF_MONTH, diff);
+
+        long millis = cal.getTimeInMillis();
+
+        millis = HsqlDateTime.getNormalisedDate(cal, millis);
+
+        return new TimestampData(millis / 1000);
+    }
+
+    public static int getDayOfWeek(String name) {
+
+        int c = Character.toUpperCase(name.charAt(0));
+
+        switch (c) {
+
+            case 'M' :
+                return 2;
+
+            case 'T' :
+                if (Character.toUpperCase(name.charAt(1)) == 'U') {
+                    return 3;
+                } else if (Character.toUpperCase(name.charAt(1)) == 'H') {
+                    return 5;
+                }
+                break;
+
+            case 'W' :
+                return 4;
+
+            case 'F' :
+                return 6;
+
+            case 'S' :
+                if (Character.toUpperCase(name.charAt(1)) == 'A') {
+                    return 7;
+                } else if (Character.toUpperCase(name.charAt(1)) == 'U') {
+                    return 1;
+                }
+                break;
+        }
+
+        throw Error.error(ErrorCode.X_22007);
+    }
 }
