@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -49,7 +49,7 @@ import org.hsqldb.rights.Grantee;
  * Base class for type objects.<p>
  *
  * @author Fred Toussi (fredt@users dot sourceforge.net)
- * @version 2.3.3
+ * @version 2.3.5
  * @since 1.9.0
  */
 public abstract class Type implements SchemaObject, Cloneable {
@@ -417,14 +417,30 @@ public abstract class Type implements SchemaObject, Cloneable {
         return false;
     }
 
+    public boolean isTimestampType() {
+        return false;
+    }
+
     public boolean isIntervalType() {
         return false;
     }
 
+    public boolean isIntervalYearMonthType() {
+        return false;
+    }
+
+    public boolean isIntervalDaySecondType() {
+        return false;
+    }
+
     public boolean isBinaryType() {
         return false;
     }
 
+    public boolean isUUIDType() {
+        return false;
+    }
+
     public boolean isBooleanType() {
         return false;
     }
@@ -555,6 +571,10 @@ public abstract class Type implements SchemaObject, Cloneable {
         return 0;
     }
 
+    public boolean isNegative(Object a) {
+        return false;
+    }
+
     int hashCode(Object a) {
 
         if (a == null) {
@@ -660,7 +680,7 @@ public abstract class Type implements SchemaObject, Cloneable {
         new BinaryType(Types.SQL_VARBINARY, 32 * 1024);
     public static final BlobType SQL_BLOB =
         new BlobType(BlobType.defaultBlobSize);
-    public static final BinaryType BINARY_UUID = new BinaryUUIDType();
+    public static final BinaryType SQL_GUID = new BinaryUUIDType();
 
     // other type
     public static final OtherType OTHER = OtherType.getOtherType();
@@ -1079,7 +1099,7 @@ public abstract class Type implements SchemaObject, Cloneable {
                 return BinaryType.getBinaryType(type, precision);
 
             case Types.SQL_GUID :
-                return BINARY_UUID;
+                return SQL_GUID;
 
             case Types.SQL_BIT :
             case Types.SQL_BIT_VARYING :
