@@ -1,4 +1,4 @@
-/* Copyright (c) 2001-2016, The HSQL Development Group
+/* Copyright (c) 2001-2017, The HSQL Development Group
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -37,7 +37,7 @@ import org.hsqldb.lib.HashSet;
 import org.hsqldb.lib.IntValueHashMap;
 import org.hsqldb.persist.HsqlDatabaseProperties;
 
-// boucherb@users 20051207 - patch 1.8.x initial JDBC 4.0 support work
+// campbell-burnet@users 20051207 - patch 1.8.x initial JDBC 4.0 support work
 // fredt@users - 2.0.0 code changes
 
 /**
@@ -47,8 +47,8 @@ import org.hsqldb.persist.HsqlDatabaseProperties;
  * or those defined by ansi/iso SQL 2003 otherwise. A type sub-identifier
  * has been added to differentiate HSQLDB-specific type specializations.
  *
- * @author Campbell Burnet (boucherb@users dot sourceforge.net)
- * @version 2.3.4
+ * @author Campbell Burnet (campbell-burnet@users dot sourceforge.net)
+ * @version 2.4.0
  * @since 1.7.2
  */
 public class Types {
@@ -339,6 +339,15 @@ public class Types {
      */
     public static final int REF = 2006;
 
+    /**
+     * The constant in the Java programming language, sometimes referred to
+     * as a type code, that identifies the generic SQL type
+     * <code>REF_CURSOR</code>.
+     *
+     * @since JDK 1.8
+     */
+    public static final int REF_CURSOR = 2012;
+
     /**
      * <P>The constant in the Java programming language, sometimes referred
      * to as a type code, that identifies the generic SQL type
@@ -362,6 +371,15 @@ public class Types {
      */
     public static final int TIME = SQL_TIME;
 
+    /**
+     * <P>The constant in the Java programming language, sometimes referred
+     * to as a type code, that identifies the generic SQL type
+     * <code>TIME_WITH_TIMEZONE </code>.
+     *
+     * @since JDK 1.8
+     */
+    public static final int TIME_WITH_TIMEZONE = 2013;
+
     /**
      * <P>The constant in the Java programming language, sometimes referred
      * to as a type code, that identifies the generic SQL type
@@ -369,6 +387,15 @@ public class Types {
      */
     public static final int TIMESTAMP = SQL_TIMESTAMP;
 
+    /**
+     * <P>The constant in the Java programming language, sometimes referred
+     * to as a type code, that identifies the generic SQL type
+     * <code>TIMESTAMP_WITH_TIMEZONE </code>.
+     *
+     * @since JDK 1.8
+     */
+    public static final int TIMESTAMP_WITH_TIMEZONE = 2014;
+
     /**
      * <P>The constant in the Java programming language, sometimes referred
      * to as a type code, that identifies the generic SQL type
@@ -536,7 +563,7 @@ public class Types {
 // lookup for types
     static final IntValueHashMap javaTypeNumbers;
 
-//  boucherb@users - We can't handle method invocations in
+//  campbell-burnet@users - We can't handle method invocations in
 //                   Function.java whose number class is
 //                   narrower than the corresponding internal
 //                   wrapper
@@ -566,11 +593,18 @@ public class Types {
         javaTypeNumbers.put("java.lang.Short", Types.SQL_SMALLINT);
         javaTypeNumbers.put("long", Types.SQL_BIGINT);
         javaTypeNumbers.put("java.lang.Long", Types.SQL_BIGINT);
-        javaTypeNumbers.put("[B", Types.SQL_BINARY);
+        javaTypeNumbers.put("[B", Types.SQL_VARBINARY);
         javaTypeNumbers.put("java.lang.Object", Types.OTHER);
         javaTypeNumbers.put("java.lang.Void", Types.SQL_ALL_TYPES);
+        javaTypeNumbers.put("java.util.UUID", Types.SQL_GUID);
+        javaTypeNumbers.put("java.time.LocalDate", Types.SQL_DATE);
+        javaTypeNumbers.put("java.time.LocalTime", Types.SQL_TIME);
+        javaTypeNumbers.put("java.time.LocalDateTime", Types.SQL_TIMESTAMP);
+        javaTypeNumbers.put("java.time.OffsetDateTime", Types.SQL_TIMESTAMP_WITH_TIME_ZONE);
+        javaTypeNumbers.put("java.time.OffsetTime", Types.SQL_TIME_WITH_TIME_ZONE);
+        javaTypeNumbers.put("java.time.Duration", Types.SQL_INTERVAL_SECOND);
+        javaTypeNumbers.put("java.time.Period", Types.SQL_INTERVAL_MONTH);
 
-        //
         illegalParameterClasses = new HashSet();
 
         illegalParameterClasses.add(Byte.TYPE);
@@ -590,7 +624,7 @@ public class Types {
      *
      * @param  c a Class instance
      * @return java.sql.Types int value
-     * @throws  HsqlException
+     * @throws  org.hsqldb.HsqlException
      */
     public static Type getParameterSQLType(Class c) {
 
